@@ -55,6 +55,8 @@ pub struct Styles {
     pub buffer: Buffer,
     #[serde(default)]
     pub buttons: Buttons,
+    #[serde(default)]
+    pub formatting: Formatting,
 }
 
 impl Default for Styles {
@@ -182,6 +184,8 @@ pub struct Buffer {
     pub topic: TextStyle,
     pub url: TextStyle,
     pub nickname_offline: OptionalTextStyle,
+    #[serde(with = "color_serde_maybe")]
+    pub backlog_rule: Option<Color>,
 }
 
 impl Default for Buffer {
@@ -202,6 +206,7 @@ impl Default for Buffer {
             topic: TextStyle::default(),
             url: TextStyle::default(),
             nickname_offline: OptionalTextStyle::default(),
+            backlog_rule: None,
         }
     }
 }
@@ -212,6 +217,7 @@ pub struct ServerMessages {
     pub join: OptionalTextStyle,
     pub part: OptionalTextStyle,
     pub quit: OptionalTextStyle,
+    #[serde(alias = "topic")]
     pub reply_topic: OptionalTextStyle,
     pub change_host: OptionalTextStyle,
     pub change_mode: OptionalTextStyle,
@@ -241,6 +247,43 @@ pub struct Text {
     pub trace: OptionalTextStyle,
 }
 
+#[derive(Debug, Clone, Copy, Serialize, Deserialize, Default)]
+#[serde(default)]
+pub struct Formatting {
+    #[serde(with = "color_serde_maybe")]
+    pub white: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub black: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub blue: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub green: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub red: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub brown: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub magenta: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub orange: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub yellow: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub lightgreen: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub cyan: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub lightcyan: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub lightblue: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub pink: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub grey: Option<Color>,
+    #[serde(with = "color_serde_maybe")]
+    pub lightgrey: Option<Color>,
+}
+
 #[derive(Clone, Copy, Debug)]
 pub struct TextStyle {
     pub color: Color,
@@ -603,7 +646,7 @@ mod binary {
     use iced_core::Color;
     use strum::{IntoEnumIterator, VariantArray};
 
-    use super::{Buffer, Buttons, General, Styles, Text};
+    use super::{Buffer, Buttons, Formatting, General, Styles, Text};
 
     pub fn encode(styles: &Styles) -> Vec<u8> {
         let mut bytes = Vec::with_capacity(Tag::VARIANTS.len() * (1 + 4));
@@ -624,6 +667,7 @@ mod binary {
             text: Text::default(),
             buffer: Buffer::default(),
             buttons: Buttons::default(),
+            formatting: Formatting::default(),
         };
 
         for chunk in bytes.chunks(5) {
