@@ -42,7 +42,7 @@ pub enum Internal {
     Bloop,
     Drop,
     Tweep,
-    Zone
+    Zone,
 }
 
 impl Internal {
@@ -59,9 +59,13 @@ impl Internal {
             }
             Internal::Bonk => include_bytes!("../../sounds/bonk.ogg").to_vec(),
             Internal::Sing => include_bytes!("../../sounds/sing.ogg").to_vec(),
-            Internal::Bloop => include_bytes!("../../sounds/bloop.ogg").to_vec(),
+            Internal::Bloop => {
+                include_bytes!("../../sounds/bloop.ogg").to_vec()
+            }
             Internal::Drop => include_bytes!("../../sounds/drop.ogg").to_vec(),
-            Internal::Tweep => include_bytes!("../../sounds/tweep.ogg").to_vec(),
+            Internal::Tweep => {
+                include_bytes!("../../sounds/tweep.ogg").to_vec()
+            }
             Internal::Zone => include_bytes!("../../sounds/zone.ogg").to_vec(),
         }
     }
