@@ -23,21 +23,15 @@ pub enum Upstream {
 }
 
 #[derive(
-    Debug,
-    Clone,
-    Copy,
-    PartialEq,
-    Eq,
-    Hash,
-    Serialize,
-    Deserialize,
-    strum::Display,
+    Debug, Clone, PartialEq, Eq, Hash, Serialize, Deserialize, strum::Display,
 )]
 pub enum Internal {
     #[strum(serialize = "File Transfers")]
     FileTransfers,
     Logs,
     Highlights,
+    #[strum(serialize = "Channel Discovery")]
+    ChannelDiscovery(Option<Server>),
 }
 
 impl Buffer {
@@ -122,14 +116,19 @@ impl Upstream {
 }
 
 impl Internal {
-    pub const ALL: &'static [Self] =
-        &[Self::FileTransfers, Self::Logs, Self::Highlights];
+    pub const ALL: &'static [Self] = &[
+        Self::FileTransfers,
+        Self::Logs,
+        Self::Highlights,
+        Self::ChannelDiscovery(None),
+    ];
 
     pub fn key(&self) -> String {
         match self {
             Internal::FileTransfers => "file-transfers",
             Internal::Logs => "logs",
             Internal::Highlights => "highlights",
+            Internal::ChannelDiscovery(_) => "channel-discovery",
         }
         .to_string()
     }
@@ -148,82 +147,6 @@ impl From<config::Buffer> for Settings {
     }
 }
 
-#[derive(Debug, Clone, Deserialize)]
-#[serde(default)]
-pub struct TextInputNickname {
-    pub enabled: bool,
-    pub show_access_level: bool,
-}
-
-impl Default for TextInputNickname {
-    fn default() -> Self {
-        Self {
-            enabled: true,
-            show_access_level: true,
-        }
-    }
-}
-
-#[derive(Debug, Clone, Default, Deserialize)]
-#[serde(default)]
-pub struct TextInput {
-    pub visibility: TextInputVisibility,
-    pub auto_format: AutoFormat,
-    pub autocomplete: Autocomplete,
-    pub nickname: TextInputNickname,
-}
-
-#[derive(Debug, Clone, Copy, Default, Deserialize)]
-#[serde(rename_all = "kebab-case")]
-pub enum OrderBy {
-    Alpha,
-    #[default]
-    Recent,
-}
-
-#[derive(Debug, Clone, Copy, Default, Deserialize)]
-#[serde(rename_all = "kebab-case")]
-pub enum SortDirection {
-    #[default]
-    Asc,
-    Desc,
-}
-
-#[derive(Debug, Clone, Deserialize)]
-#[serde(default)]
-pub struct Autocomplete {
-    pub order_by: OrderBy,
-    pub sort_direction: SortDirection,
-    pub completion_suffixes: [String; 2],
-}
-
-impl Default for Autocomplete {
-    fn default() -> Self {
-        Self {
-            order_by: OrderBy::default(),
-            sort_direction: SortDirection::default(),
-            completion_suffixes: [": ".to_string(), " ".to_string()],
-        }
-    }
-}
-
-#[derive(Debug, Clone, Copy, Default, Deserialize)]
-#[serde(rename_all = "kebab-case")]
-pub enum TextInputVisibility {
-    Focused,
-    #[default]
-    Always,
-}
-
-#[derive(Debug, Clone, Copy, Default, Deserialize)]
-#[serde(rename_all = "kebab-case")]
-pub enum AutoFormat {
-    #[default]
-    Disabled,
-    Markdown,
-    All,
-}
-
 #[derive(Debug, Clone, Deserialize)]
 #[serde(default)]
 pub struct Timestamp {
@@ -247,10 +170,57 @@ impl Default for Timestamp {
     }
 }
 
-#[derive(Debug, Clone, Default, Deserialize)]
+#[derive(Debug, Clone)]
+pub enum BacklogText {
+    Text(String),
+    Hidden,
+}
+
+impl Default for BacklogText {
+    fn default() -> Self {
+        Self::Text("backlog".to_string())
+    }
+}
+
+impl<'de> Deserialize<'de> for BacklogText {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        #[derive(Deserialize)]
+        #[serde(untagged)]
+        enum T {
+            String(String),
+            Bool(bool),
+        }
+
+        match T::deserialize(deserializer)? {
+            T::String(conf) => Ok(Self::Text(conf)),
+            T::Bool(val) => {
+                if !val {
+                    Ok(Self::Hidden)
+                } else {
+                    Ok(Self::default())
+                }
+            }
+        }
+    }
+}
+
+#[derive(Debug, Clone, Deserialize)]
 #[serde(default)]
 pub struct BacklogSeparator {
     pub hide_when_all_read: bool,
+    pub text: BacklogText,
+}
+
+impl Default for BacklogSeparator {
+    fn default() -> Self {
+        Self {
+            hide_when_all_read: true,
+            text: BacklogText::default(),
+        }
+    }
 }
 
 #[derive(Debug, Clone, Deserialize)]
@@ -309,6 +279,10 @@ impl Alignment {
     pub fn is_right(&self) -> bool {
         matches!(self, Self::Right)
     }
+
+    pub fn is_top(&self) -> bool {
+        matches!(self, Self::Top)
+    }
 }
 
 #[derive(Debug, Clone, Copy)]
