@@ -37,15 +37,17 @@ impl fmt::Debug for Stream {
 
 pub fn on_connect(
     handle: server::Handle,
-    config: Arc<config::Server>,
+    server_config: Arc<config::Server>,
     our_nickname: NickRef,
     isupport: &HashMap<isupport::Kind, isupport::Parameter>,
+    config: &config::Config,
 ) -> Stream {
-    let commands = config
+    let commands = server_config
         .on_connect
         .iter()
         .filter_map(|command| {
-            command::parse(command, None, Some(our_nickname), isupport).ok()
+            command::parse(command, None, Some(our_nickname), isupport, config)
+                .ok()
         })
         .collect::<Vec<_>>();
 
@@ -87,8 +89,12 @@ pub fn on_connect(
                                 time::sleep(Duration::from_secs(seconds)).await;
                                 None
                             }
-                            // We don't handle hop, clear-buffer, sysinfo when called from connected.
+                            // We don't handle hop, clear-buffer, sysinfo,
+                            // channel_discovery, or connect when called on
+                            // connection.
                             command::Internal::ClearBuffer
+                            | command::Internal::ChannelDiscovery
+                            | command::Internal::Connect(_)
                             | command::Internal::Hop(_, _)
                             | command::Internal::SysInfo => None,
                         },
