@@ -4,12 +4,13 @@ use std::str::FromStr;
 use fancy_regex::Regex;
 use irc::proto;
 use itertools::Itertools;
+use serde::{Deserialize, Serialize};
 
 use crate::buffer::{self, Upstream};
 use crate::isupport::{self, find_target_limit};
 use crate::message::{self, formatting};
-use crate::user::NickRef;
-use crate::{Target, ctcp, target};
+use crate::user::{ChannelUsers, NickRef};
+use crate::{Config, Message, Target, Url, User, ctcp, target};
 
 #[derive(Debug, Clone)]
 pub enum Command {
@@ -27,13 +28,16 @@ pub enum Internal {
     /// - Channel to join
     /// - Part message
     Hop(Option<String>, Option<String>),
+    ChannelDiscovery,
     Delay(u64),
     SysInfo,
     Detach(Vec<target::Channel>),
+    Connect(String),
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq, Deserialize, Serialize)]
 pub enum Irc {
+    List(Option<String>, Option<String>),
     Join(String, Option<String>),
     Motd(Option<String>),
     Nick(String),
@@ -41,6 +45,7 @@ pub enum Irc {
     Msg(String, String),
     Me(String, String),
     Whois(Option<String>, String),
+    Whowas(String, Option<String>),
     Part(String, Option<String>),
     Topic(String, Option<String>),
     Kick(String, String, Option<String>),
@@ -53,6 +58,106 @@ pub enum Irc {
     Ctcp(ctcp::Command, String, Option<String>),
 }
 
+impl Irc {
+    pub fn messages(
+        &self,
+        user: User,
+        channel_users: Option<&ChannelUsers>,
+        chantypes: &[char],
+        statusmsg: &[char],
+        casemapping: isupport::CaseMap,
+        supports_echoes: bool,
+    ) -> Option<Vec<Message>> {
+        let to_targets = |target: &str, source| {
+            let target =
+                Target::parse(target, chantypes, statusmsg, casemapping);
+
+            let message_target = match &target {
+                Target::Channel(channel) => message::Target::Channel {
+                    channel: channel.clone(),
+                    source,
+                },
+
+                Target::Query(query) => message::Target::Query {
+                    query: query.clone(),
+                    source,
+                },
+            };
+
+            (target, message_target)
+        };
+
+        match self {
+            Irc::Msg(targets, text) => Some(
+                targets
+                    .split(',')
+                    .map(|target| {
+                        let (target, message_target) = to_targets(
+                            target,
+                            message::Source::User(user.clone()),
+                        );
+
+                        Message::sent(
+                            message_target,
+                            message::parse_fragments_with_users(
+                                text.clone(),
+                                channel_users,
+                                casemapping,
+                            ),
+                            supports_echoes.then_some(Irc::Msg(
+                                target.to_string(),
+                                text.clone(),
+                            )),
+                        )
+                    })
+                    .collect(),
+            ),
+            Irc::Notice(targets, text) => Some(
+                targets
+                    .split(',')
+                    .map(|target| {
+                        let (target, message_target) = to_targets(
+                            target,
+                            message::Source::User(user.clone()),
+                        );
+
+                        Message::sent(
+                            message_target,
+                            message::parse_fragments_with_users(
+                                text.clone(),
+                                channel_users,
+                                casemapping,
+                            ),
+                            supports_echoes.then_some(Irc::Notice(
+                                target.to_string(),
+                                text.clone(),
+                            )),
+                        )
+                    })
+                    .collect(),
+            ),
+            Irc::Me(target, action) => {
+                let (_, message_target) = to_targets(
+                    target,
+                    message::Source::Action(Some(user.clone())),
+                );
+
+                Some(vec![Message::sent(
+                    message_target,
+                    message::action_text(
+                        &user,
+                        Some(action),
+                        channel_users,
+                        casemapping,
+                    ),
+                    supports_echoes.then_some(self.clone()),
+                )])
+            }
+            _ => None,
+        }
+    }
+}
+
 #[derive(Debug, Clone, Copy)]
 enum Kind {
     Join,
@@ -67,6 +172,7 @@ enum Kind {
     Kick,
     Mode,
     Format,
+    Plain,
     Away,
     SetName,
     Ctcp,
@@ -74,9 +180,11 @@ enum Kind {
     Notice,
     Delay,
     Clear,
+    List,
     ClearTopic,
     SysInfo,
     Detach,
+    Connect,
     Raw,
 }
 
@@ -97,6 +205,7 @@ impl FromStr for Kind {
             "kick" => Ok(Kind::Kick),
             "mode" | "m" => Ok(Kind::Mode),
             "format" | "f" => Ok(Kind::Format),
+            "plain" | "p" => Ok(Kind::Plain),
             "away" => Ok(Kind::Away),
             "setname" => Ok(Kind::SetName),
             "notice" => Ok(Kind::Notice),
@@ -105,9 +214,11 @@ impl FromStr for Kind {
             "hop" | "rejoin" => Ok(Kind::Hop),
             "delay" => Ok(Kind::Delay),
             "clear" => Ok(Kind::Clear),
+            "list" => Ok(Kind::List),
             "cleartopic" | "ct" => Ok(Kind::ClearTopic),
             "sysinfo" => Ok(Kind::SysInfo),
             "detach" => Ok(Kind::Detach),
+            "connect" => Ok(Kind::Connect),
             _ => Err(()),
         }
     }
@@ -118,6 +229,7 @@ pub fn parse(
     buffer: Option<&buffer::Upstream>,
     our_nickname: Option<NickRef>,
     isupport: &HashMap<isupport::Kind, isupport::Parameter>,
+    config: &Config,
 ) -> Result<Command, Error> {
     let (head, rest) = s.split_once('/').ok_or(Error::MissingSlash)?;
     // Don't allow leading whitespace before slash
@@ -248,7 +360,9 @@ pub fn parse(
                 Ok(Command::Irc(Irc::Nick(nick)))
             }),
             Kind::Quit => validated::<0, 1, true>(args, |_, [comment]| {
-                Ok(Command::Irc(Irc::Quit(comment)))
+                Ok(Command::Irc(Irc::Quit(comment.or_else(|| {
+                    config.buffer.commands.quit.default_reason.clone()
+                }))))
             }),
             Kind::Msg => validated::<1, 1, true>(args, |[targets], [msg]| {
                 let target_limit = find_target_limit(isupport, "PRIVMSG")
@@ -383,7 +497,10 @@ pub fn parse(
                     }
 
                     Ok(Command::Internal(Internal::LeaveBuffers(
-                        targets, reason,
+                        targets,
+                        reason.or_else(|| {
+                            config.buffer.commands.part.default_reason.clone()
+                        }),
                     )))
                 })
             }
@@ -751,6 +868,16 @@ pub fn parse(
                     Ok(unknown())
                 }
             }
+            Kind::Plain => {
+                if let Some(target) = buffer.and_then(Upstream::target) {
+                    Ok(Command::Irc(Irc::Msg(
+                        target.to_string(),
+                        raw.to_string(),
+                    )))
+                } else {
+                    Ok(unknown())
+                }
+            }
             Kind::Ctcp => {
                 validated::<1, 2, true>(
                     args.clone(),
@@ -803,6 +930,9 @@ pub fn parse(
             Kind::Clear => validated::<0, 0, false>(args, |_, _| {
                 Ok(Command::Internal(Internal::ClearBuffer))
             }),
+            Kind::List => validated::<0, 0, false>(args, |_, _| {
+                Ok(Command::Internal(Internal::ChannelDiscovery))
+            }),
             Kind::SysInfo => validated::<0, 0, false>(args, |_, _| {
                 Ok(Command::Internal(Internal::SysInfo))
             }),
@@ -921,6 +1051,15 @@ pub fn parse(
                     Err(Error::NotPositiveInteger)
                 }
             }),
+            Kind::Connect => validated::<1, 0, false>(args, |[server], _| {
+                if let Ok(url) = Url::from_str(&server)
+                    && matches!(url, Url::ServerConnect { .. })
+                {
+                    Ok(Command::Internal(Internal::Connect(server)))
+                } else {
+                    Err(Error::InvalidServerUrl)
+                }
+            }),
         },
         Err(()) => Ok(unknown()),
     }
@@ -1008,6 +1147,7 @@ impl TryFrom<Irc> for proto::Command {
                 ctcp::query_command(&ctcp::Command::Action, target, Some(text))
             }
             Irc::Whois(channel, user) => proto::Command::WHOIS(channel, user),
+            Irc::Whowas(user, count) => proto::Command::WHOWAS(user, count),
             Irc::Part(chanlist, reason) => {
                 proto::Command::PART(chanlist, reason)
             }
@@ -1026,6 +1166,9 @@ impl TryFrom<Irc> for proto::Command {
             Irc::Ctcp(command, target, params) => {
                 ctcp::query_command(&command, target, params)
             }
+            Irc::List(channels, elistcond) => {
+                proto::Command::LIST(channels, elistcond)
+            }
         })
     }
 }
@@ -1072,6 +1215,8 @@ pub enum Error {
     NotPositiveInteger,
     #[error("invalid channel name ({requirements}")]
     InvalidChannelName { requirements: String },
+    #[error("invalid server url")]
+    InvalidServerUrl,
 }
 
 fn fmt_incorrect_arg_count(min: usize, max: usize, actual: usize) -> String {
