@@ -12,6 +12,7 @@ use tokio_stream::wrappers::ReadDirStream;
 
 pub use self::actions::Actions;
 pub use self::buffer::Buffer;
+pub use self::context_menu::ContextMenu;
 pub use self::ctcp::Ctcp;
 pub use self::file_transfer::FileTransfer;
 pub use self::highlights::Highlights;
@@ -33,9 +34,11 @@ use crate::{Theme, environment};
 
 pub mod actions;
 pub mod buffer;
+pub mod context_menu;
 pub mod ctcp;
 pub mod file_transfer;
 pub mod highlights;
+pub mod inclusivities;
 pub mod keys;
 pub mod logs;
 pub mod notification;
@@ -52,6 +55,7 @@ const DEFAULT_THEME_NAME: &str = "ferra";
 #[derive(Debug, Clone, Default)]
 pub struct Config {
     pub appearance: Appearance,
+    pub context_menu: ContextMenu,
     pub servers: ServerMap,
     pub proxy: Option<Proxy>,
     pub font: Font,
@@ -121,6 +125,8 @@ pub struct Font {
     #[serde(deserialize_with = "deserialize_optional_font_weight_from_string")]
     #[serde(alias = "bold-weight")] // For backwards compatibility
     pub bold_weight: Option<font::Weight>,
+    #[serde(deserialize_with = "deserialize_positive_integer_maybe")]
+    pub only_emojis_size: Option<u8>,
 }
 
 impl Default for Font {
@@ -130,6 +136,7 @@ impl Default for Font {
             size: None,
             weight: font::Weight::Normal,
             bold_weight: None,
+            only_emojis_size: None,
         }
     }
 }
@@ -301,6 +308,7 @@ impl Config {
         pub struct Configuration {
             pub theme: ThemeKeys,
             pub servers: IndexMap<ServerName, Server>,
+            pub context_menu: ContextMenu,
             pub proxy: Option<Proxy>,
             pub font: Font,
             pub scale_factor: ScaleFactor,
@@ -324,6 +332,7 @@ impl Config {
                 Self {
                     theme: ThemeKeys::default(),
                     servers: IndexMap::<ServerName, Server>::default(),
+                    context_menu: ContextMenu::default(),
                     proxy: None,
                     font: Font::default(),
                     scale_factor: ScaleFactor::default(),
@@ -357,6 +366,7 @@ impl Config {
         let Configuration {
             theme,
             servers,
+            context_menu,
             font,
             proxy,
             scale_factor,
@@ -386,6 +396,7 @@ impl Config {
 
         Ok(Config {
             appearance,
+            context_menu,
             servers,
             font,
             proxy,
