@@ -2,6 +2,12 @@ use fancy_regex::{Regex, RegexBuilder};
 use itertools::Itertools;
 use serde::{Deserialize, Deserializer};
 
+use crate::config::inclusivities::{Inclusivities, is_target_included};
+use crate::isupport;
+use crate::server::Server;
+use crate::target::Target;
+use crate::user::User;
+
 #[derive(Debug, Clone, Deserialize, Default)]
 #[serde(default)]
 pub struct Highlights {
@@ -13,32 +19,46 @@ pub struct Highlights {
 #[derive(Debug, Clone, Deserialize)]
 #[serde(default)]
 pub struct Nickname {
-    pub exclude: Vec<String>,
-    pub include: Vec<String>,
+    pub exclude: Option<Inclusivities>,
+    pub include: Option<Inclusivities>,
     pub case_insensitive: bool,
 }
 
 impl Default for Nickname {
     fn default() -> Self {
         Self {
-            exclude: Vec::default(),
-            include: Vec::default(),
+            exclude: None,
+            include: None,
             case_insensitive: true,
         }
     }
 }
 
 impl Nickname {
-    pub fn is_target_included(&self, target: &str) -> bool {
-        is_target_included(&self.include, &self.exclude, target)
+    pub fn is_target_included(
+        &self,
+        user: Option<&User>,
+        target: &Target,
+        server: &Server,
+        casemapping: isupport::CaseMap,
+    ) -> bool {
+        is_target_included(
+            self.include.as_ref(),
+            self.exclude.as_ref(),
+            user.map(User::nickname),
+            target,
+            server,
+            casemapping,
+        )
     }
 }
 
 #[derive(Debug, Clone)]
 pub struct Match {
     pub regex: Regex,
-    pub exclude: Vec<String>,
-    pub include: Vec<String>,
+    pub exclude: Option<Inclusivities>,
+    pub include: Option<Inclusivities>,
+    pub sound: Option<String>,
 }
 
 impl<'de> Deserialize<'de> for Match {
@@ -52,18 +72,22 @@ impl<'de> Deserialize<'de> for Match {
             Words {
                 words: Vec<String>,
                 #[serde(default)]
-                exclude: Vec<String>,
+                exclude: Option<Inclusivities>,
                 #[serde(default)]
-                include: Vec<String>,
+                include: Option<Inclusivities>,
                 #[serde(default)]
                 case_insensitive: bool,
+                #[serde(default)]
+                sound: Option<String>,
             },
             Regex {
                 regex: String,
                 #[serde(default)]
-                exclude: Vec<String>,
+                exclude: Option<Inclusivities>,
                 #[serde(default)]
-                include: Vec<String>,
+                include: Option<Inclusivities>,
+                #[serde(default)]
+                sound: Option<String>,
             },
         }
 
@@ -73,6 +97,7 @@ impl<'de> Deserialize<'de> for Match {
                 exclude,
                 include,
                 case_insensitive,
+                sound,
             } => {
                 let words =
                     words.iter().map(|s| fancy_regex::escape(s)).join("|");
@@ -92,12 +117,14 @@ impl<'de> Deserialize<'de> for Match {
                     regex,
                     exclude,
                     include,
+                    sound,
                 })
             }
             Inner::Regex {
                 regex,
                 exclude,
                 include,
+                sound,
             } => {
                 let regex =
                     RegexBuilder::new(&regex).build().map_err(|err| {
@@ -110,6 +137,7 @@ impl<'de> Deserialize<'de> for Match {
                     regex,
                     exclude,
                     include,
+                    sound,
                 })
             }
         }
@@ -117,24 +145,20 @@ impl<'de> Deserialize<'de> for Match {
 }
 
 impl Match {
-    pub fn is_target_included(&self, target: &str) -> bool {
-        is_target_included(&self.include, &self.exclude, target)
+    pub fn is_target_included(
+        &self,
+        user: Option<&User>,
+        target: &Target,
+        server: &Server,
+        casemapping: isupport::CaseMap,
+    ) -> bool {
+        is_target_included(
+            self.include.as_ref(),
+            self.exclude.as_ref(),
+            user.map(User::nickname),
+            target,
+            server,
+            casemapping,
+        )
     }
 }
-
-fn is_target_included(
-    include: &[String],
-    exclude: &[String],
-    target: &str,
-) -> bool {
-    let is_channel_filtered = |list: &[String], target: &str| -> bool {
-        let wildcards = ["*", "all"];
-        list.iter()
-            .any(|item| wildcards.contains(&item.as_str()) || item == target)
-    };
-
-    let channel_included = is_channel_filtered(include, target);
-    let channel_excluded = is_channel_filtered(exclude, target);
-
-    channel_included || !channel_excluded
-}
