@@ -3,6 +3,13 @@ use std::collections::HashMap;
 use serde::Deserialize;
 
 use crate::audio::Sound;
+use crate::config::inclusivities::{
+    Inclusivities, is_user_channel_server_included,
+};
+use crate::isupport;
+use crate::server::Server;
+use crate::target::Channel;
+use crate::user::User;
 
 #[derive(Debug, Clone, Deserialize)]
 #[serde(default)]
@@ -11,8 +18,8 @@ pub struct Notification {
     pub show_content: bool,
     pub sound: Option<String>,
     pub delay: Option<u32>,
-    pub exclude: Vec<String>,
-    pub include: Vec<String>,
+    pub exclude: Option<Inclusivities>,
+    pub include: Option<Inclusivities>,
 }
 
 impl Default for Notification {
@@ -22,26 +29,28 @@ impl Default for Notification {
             show_content: false,
             sound: None,
             delay: Some(500),
-            exclude: Vec::default(),
-            include: Vec::default(),
+            exclude: None,
+            include: None,
         }
     }
 }
 
 impl Notification {
-    pub fn should_notify(&self, targets: Vec<String>) -> bool {
-        let is_target_filtered =
-            |list: &Vec<String>, targets: &Vec<String>| -> bool {
-                let wildcards = ["*", "all"];
-
-                list.iter().any(|item| {
-                    wildcards.contains(&item.as_str()) || targets.contains(item)
-                })
-            };
-        let target_included = is_target_filtered(&self.include, &targets);
-        let target_excluded = is_target_filtered(&self.exclude, &targets);
-
-        target_included || !target_excluded
+    pub fn should_notify(
+        &self,
+        user: &User,
+        channel: Option<&Channel>,
+        server: &Server,
+        casemapping: isupport::CaseMap,
+    ) -> bool {
+        is_user_channel_server_included(
+            self.include.as_ref(),
+            self.exclude.as_ref(),
+            user.nickname(),
+            channel,
+            server,
+            casemapping,
+        )
     }
 }
 
@@ -56,10 +65,15 @@ pub struct Notifications {
     pub file_transfer_request: Notification,
     pub monitored_online: Notification,
     pub monitored_offline: Notification,
+    #[serde(rename = "channel")]
+    pub channels: HashMap<String, Notification>,
 }
 
 impl Notifications {
-    pub fn load_sounds(&self) -> HashMap<String, Sound> {
+    pub fn load_sounds<'a>(
+        &self,
+        highlight_matches_sounds: impl Iterator<Item = &'a str>,
+    ) -> HashMap<String, Sound> {
         let mut sounds = HashMap::new();
 
         // Helper function to load a sound and add it to the map
@@ -101,6 +115,14 @@ impl Notifications {
         if let Some(sound_name) = self.monitored_offline.sound.as_deref() {
             load_and_insert(sound_name);
         }
+        for notification in self.channels.values() {
+            if let Some(sound_name) = notification.sound.as_deref() {
+                load_and_insert(sound_name);
+            }
+        }
+        for sound_name in highlight_matches_sounds {
+            load_and_insert(sound_name);
+        }
 
         sounds
     }
