@@ -1,6 +1,8 @@
+use std::fmt;
+
 use serde::Deserialize;
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Deserialize, PartialEq, Eq)]
 #[serde(rename_all = "snake_case")]
 pub enum Proxy {
     Http {
@@ -49,3 +51,18 @@ impl From<Proxy> for irc::connection::Proxy {
         }
     }
 }
+
+impl fmt::Display for Proxy {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Proxy::Http { host, port, .. } => {
+                write!(f, "http://{host}:{port}")
+            }
+            Proxy::Socks5 { host, port, .. } => {
+                write!(f, "socks5://{host}:{port}")
+            }
+            #[cfg(feature = "tor")]
+            Proxy::Tor => write!(f, "tor"),
+        }
+    }
+}
