@@ -2,14 +2,18 @@ use std::collections::HashMap;
 use std::path::PathBuf;
 use std::time::Duration;
 
+use fancy_regex::{Regex, RegexBuilder};
 use irc::connection;
 use serde::{Deserialize, Deserializer};
 
-use crate::config;
+use crate::config::inclusivities::{
+    Inclusivities, is_target_channel_included, is_target_query_included,
+};
 use crate::serde::{
     deserialize_path_buf_with_path_transformations,
     deserialize_path_buf_with_path_transformations_maybe,
 };
+use crate::{config, isupport, target};
 
 const DEFAULT_PORT: u16 = 6667;
 const DEFAULT_TLS_PORT: u16 = 6697;
@@ -101,6 +105,8 @@ pub struct Server {
     pub anti_flood: Duration,
     #[serde(skip)]
     pub order: u16,
+    pub proxy: Option<config::Proxy>,
+    pub confirm_message_delivery: ConfirmMessageDelivery,
 }
 
 impl Server {
@@ -213,6 +219,8 @@ impl Default for Server {
             chathistory: true,
             anti_flood: Duration::from_millis(2000),
             order: 0,
+            proxy: None,
+            confirm_message_delivery: ConfirmMessageDelivery::default(),
         }
     }
 }
@@ -332,7 +340,161 @@ impl Sasl {
 #[derive(PartialEq, Eq, Debug, Clone, Deserialize, Default)]
 #[serde(default)]
 pub struct Filters {
-    pub ignore: Vec<String>,
+    pub ignore: Vec<Ignore>,
+    #[serde(deserialize_with = "deserialize_fancy_regexes")]
+    pub regex: Vec<FancyRegex>,
+}
+
+#[derive(PartialEq, Eq, Debug, Clone)]
+pub enum Ignore {
+    User(String),
+    UserInChannel { user: String, channel: String },
+    Regex { regex: FancyRegex },
+    RegexInChannel { regex: FancyRegex, channel: String },
+}
+
+impl<'de> Deserialize<'de> for Ignore {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        #[derive(Debug, Clone, Deserialize)]
+        #[serde(untagged)]
+        pub enum Inner {
+            User(String),
+            UserInChannel { user: String, channel: String },
+            Regex { regex: String },
+            RegexInChannel { regex: String, channel: String },
+        }
+
+        match Inner::deserialize(deserializer)? {
+            Inner::User(user) => Ok(Ignore::User(user)),
+            Inner::UserInChannel { user, channel } => {
+                Ok(Ignore::UserInChannel { user, channel })
+            }
+            Inner::Regex { regex } => {
+                let regex =
+                    RegexBuilder::new(&regex).build().map_err(|err| {
+                        serde::de::Error::custom(format!(
+                            "invalid regex '{regex}': {err}"
+                        ))
+                    })?;
+
+                Ok(Ignore::Regex {
+                    regex: FancyRegex(regex),
+                })
+            }
+            Inner::RegexInChannel { regex, channel } => {
+                let regex =
+                    RegexBuilder::new(&regex).build().map_err(|err| {
+                        serde::de::Error::custom(format!(
+                            "invalid regex '{regex}': {err}"
+                        ))
+                    })?;
+
+                Ok(Ignore::RegexInChannel {
+                    regex: FancyRegex(regex),
+                    channel,
+                })
+            }
+        }
+    }
+}
+
+// We want to build the regex on deserialization to present any errors to the
+// user then, but we need to be able to define PartialEq and Eq. So, we use this
+// Regex wrapper.
+#[derive(Debug, Clone)]
+pub struct FancyRegex(Regex);
+
+impl PartialEq for FancyRegex {
+    fn eq(&self, other: &FancyRegex) -> bool {
+        self.0.as_str() == other.0.as_str()
+    }
+}
+
+impl Eq for FancyRegex {}
+
+impl From<FancyRegex> for Regex {
+    fn from(fancy_regex: FancyRegex) -> Regex {
+        fancy_regex.0
+    }
+}
+
+pub fn deserialize_fancy_regexes<'de, D>(
+    deserializer: D,
+) -> Result<Vec<FancyRegex>, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    let regex_strings = Vec::<String>::deserialize(deserializer)?;
+
+    regex_strings
+        .iter()
+        .map(|regex_string| {
+            RegexBuilder::new(regex_string)
+                .build()
+                .map_err(|err| {
+                    serde::de::Error::custom(format!(
+                        "invalid regex '{regex_string}': {err}"
+                    ))
+                })
+                .map(FancyRegex)
+        })
+        .collect::<Result<Vec<FancyRegex>, D::Error>>()
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, Deserialize)]
+#[serde(default)]
+pub struct ConfirmMessageDelivery {
+    pub enabled: bool,
+    pub exclude: Option<Inclusivities>,
+    pub include: Option<Inclusivities>,
+}
+
+impl Default for ConfirmMessageDelivery {
+    fn default() -> Self {
+        Self {
+            enabled: true,
+            exclude: None,
+            include: None,
+        }
+    }
+}
+
+impl ConfirmMessageDelivery {
+    pub fn is_target_channel_included(
+        &self,
+        channel: &target::Channel,
+        server: &crate::server::Server,
+        casemapping: isupport::CaseMap,
+    ) -> bool {
+        self.enabled
+            && is_target_channel_included(
+                self.include.as_ref(),
+                self.exclude.as_ref(),
+                None,
+                channel,
+                server,
+                casemapping,
+            )
+    }
+
+    pub fn is_target_query_included(
+        &self,
+        query: &target::Query,
+        server: &crate::server::Server,
+        casemapping: isupport::CaseMap,
+    ) -> bool {
+        self.enabled
+            && is_target_query_included(
+                self.include.as_ref(),
+                self.exclude.as_ref(),
+                query,
+                server,
+                casemapping,
+            )
+    }
 }
 
 fn deserialize_anti_flood<'de, D>(deserializer: D) -> Result<Duration, D::Error>
