@@ -2,38 +2,118 @@ use serde::{Deserialize, Deserializer};
 use serde_untagged::UntaggedEnumVisitor;
 
 use crate::config::Scrollbar;
+use crate::config::inclusivities::{Inclusivities, is_target_channel_included};
 use crate::serde::deserialize_positive_integer;
+use crate::server::Server;
+use crate::{isupport, target};
 
-#[derive(Debug, Copy, Clone, Deserialize)]
+#[derive(Debug, Clone, Default, Deserialize)]
 #[serde(default)]
 pub struct Sidebar {
     pub max_width: Option<u16>,
     #[serde(deserialize_with = "deserialize_unread_indicator")]
     pub unread_indicator: UnreadIndicator,
     pub position: Position,
-    pub show_user_menu: bool,
     pub order_by: OrderBy,
     pub scrollbar: Scrollbar,
-    #[serde(deserialize_with = "deserialize_positive_integer")]
-    pub server_icon_size: u32,
+    #[serde(
+        deserialize_with = "deserialize_server_icon",
+        alias = "server_icon_size"
+    )]
+    pub server_icon: ServerIcon,
+    pub user_menu: UserMenu,
+    pub padding: Padding,
+    pub spacing: Spacing,
 }
 
-impl Default for Sidebar {
+#[derive(Debug, Clone, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum ServerIcon {
+    Size(u32),
+    Hidden,
+}
+
+impl Default for ServerIcon {
     fn default() -> Self {
-        Sidebar {
-            max_width: None,
-            unread_indicator: UnreadIndicator::default(),
-            position: Position::default(),
-            show_user_menu: true,
-            order_by: OrderBy::default(),
-            scrollbar: Scrollbar::default(),
-            server_icon_size: 12,
-        }
+        Self::Size(12)
+    }
+}
+
+#[allow(clippy::redundant_closure_for_method_calls)]
+pub fn deserialize_server_icon<'de, D>(
+    deserializer: D,
+) -> Result<ServerIcon, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    UntaggedEnumVisitor::new()
+        .u32(|value| {
+            if value > 0 {
+                Ok(ServerIcon::Size(value))
+            } else {
+                Err(serde::de::Error::invalid_value(
+                    serde::de::Unexpected::Unsigned(value as u64),
+                    &"a positive integer",
+                ))
+            }
+        })
+        .string(|string| match string {
+            "hidden" | "none" => Ok(ServerIcon::Hidden),
+            _ => Err(serde::de::Error::invalid_value(
+                serde::de::Unexpected::Str(string),
+                &"\"hidden\" or a size (positive integer)",
+            )),
+        })
+        .bool(|value| match value {
+            true => Ok(ServerIcon::Size(12)),
+            false => Ok(ServerIcon::Hidden),
+        })
+        .map(|map| map.deserialize())
+        .deserialize(deserializer)
+}
+
+#[derive(Debug, Copy, Clone, Deserialize)]
+#[serde(default)]
+pub struct Padding {
+    pub buffer: [u16; 2],
+}
+
+impl Default for Padding {
+    fn default() -> Self {
+        Self { buffer: [5, 5] }
     }
 }
 
 #[derive(Debug, Copy, Clone, Deserialize)]
 #[serde(default)]
+pub struct Spacing {
+    pub server: u32,
+}
+
+impl Default for Spacing {
+    fn default() -> Self {
+        Self { server: 12 }
+    }
+}
+
+#[derive(Debug, Copy, Clone, Deserialize)]
+#[serde(default)]
+pub struct UserMenu {
+    pub enabled: bool,
+    pub show_new_version_indicator: bool,
+}
+
+impl Default for UserMenu {
+    fn default() -> Self {
+        Self {
+            enabled: true,
+            show_new_version_indicator: true,
+        }
+    }
+}
+
+#[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
 pub struct UnreadIndicator {
     pub title: bool,
     pub icon: Icon,
@@ -42,6 +122,9 @@ pub struct UnreadIndicator {
     pub highlight_icon: Icon,
     #[serde(deserialize_with = "deserialize_positive_integer")]
     pub highlight_icon_size: u32,
+    pub query_as_highlight: bool,
+    pub exclude: Option<Inclusivities>,
+    pub include: Option<Inclusivities>,
 }
 
 impl Default for UnreadIndicator {
@@ -52,6 +135,9 @@ impl Default for UnreadIndicator {
             icon_size: 6,
             highlight_icon: Icon::CircleEmpty,
             highlight_icon_size: 8,
+            query_as_highlight: false,
+            exclude: None,
+            include: None,
         }
     }
 }
@@ -64,6 +150,22 @@ impl UnreadIndicator {
     pub fn has_unread_highlight_icon(&self) -> bool {
         !matches!(self.highlight_icon, Icon::None)
     }
+
+    pub fn should_indicate_unread(
+        &self,
+        channel: &target::Channel,
+        server: &Server,
+        casemapping: isupport::CaseMap,
+    ) -> bool {
+        is_target_channel_included(
+            self.include.as_ref(),
+            self.exclude.as_ref(),
+            None,
+            channel,
+            server,
+            casemapping,
+        )
+    }
 }
 
 pub fn deserialize_unread_indicator<'de, D>(
