@@ -11,11 +11,8 @@ use crate::history::{self, History, MessageReferences, ReadMarker};
 use crate::message::broadcast::{self, Broadcast};
 use crate::message::{self, Limit};
 use crate::target::{self, Target};
-use crate::user::{ChannelUsers, Nick};
-use crate::{
-    Config, Input, Server, User, buffer, client, config, input, isupport,
-    server,
-};
+use crate::user::Nick;
+use crate::{Config, Server, buffer, client, config, input, isupport, server};
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub struct Resource {
@@ -49,12 +46,14 @@ pub enum Message {
     Flushed(history::Kind, Result<(), history::Error>),
     Exited(Vec<(history::Kind, Result<(), history::Error>)>),
     SentMessageUpdated(history::Kind, history::ReadMarker),
+    ResendMessage(history::Kind, message::Message),
 }
 
 pub enum Event {
     Loaded(history::Kind),
     Exited,
     SentMessageUpdated(history::Kind, history::ReadMarker),
+    ResendMessage(history::Kind, message::Message),
 }
 
 #[derive(Debug, Default)]
@@ -198,6 +197,9 @@ impl Manager {
             Message::SentMessageUpdated(kind, read_marker) => {
                 return Some(Event::SentMessageUpdated(kind, read_marker));
             }
+            Message::ResendMessage(kind, message) => {
+                return Some(Event::ResendMessage(kind, message));
+            }
         }
 
         None
@@ -257,54 +259,36 @@ impl Manager {
 
     pub fn record_input_message(
         &mut self,
-        input: Input,
-        user: User,
-        channel_users: Option<&ChannelUsers>,
-        chantypes: &[char],
-        statusmsg: &[char],
+        message: message::Message,
+        server: &Server,
         casemapping: isupport::CaseMap,
         config: &Config,
     ) -> Vec<BoxFuture<'static, Message>> {
         let mut tasks = vec![];
 
-        if let Some(messages) = input.messages(
-            user,
-            channel_users,
-            chantypes,
-            statusmsg,
-            casemapping,
-            config,
-        ) {
-            for message in messages {
-                if config.buffer.mark_as_read.on_message_sent
-                    && let Some(kind) = history::Kind::from_server_message(
-                        input.server().clone(),
-                        &message,
-                    )
-                {
-                    tasks.extend(
-                        self.update_read_marker(
-                            kind,
-                            history::ReadMarker::from_date_time(
-                                message.server_time,
-                            ),
-                        )
-                        .map(futures::FutureExt::boxed),
-                    );
-                }
-
-                tasks.extend(
-                    self.record_message(
-                        input.server(),
-                        casemapping,
-                        message,
-                        &config.buffer,
-                    )
-                    .map(futures::FutureExt::boxed),
-                );
-            }
+        if config.buffer.mark_as_read.on_message_sent
+            && let Some(kind) =
+                history::Kind::from_server_message(server.clone(), &message)
+        {
+            tasks.extend(
+                self.update_read_marker(
+                    kind,
+                    history::ReadMarker::from_date_time(message.server_time),
+                )
+                .map(futures::FutureExt::boxed),
+            );
         }
 
+        tasks.extend(
+            self.block_and_record_message(
+                server,
+                casemapping,
+                message,
+                &config.buffer,
+            )
+            .map(futures::FutureExt::boxed),
+        );
+
         tasks
     }
 
@@ -320,26 +304,17 @@ impl Manager {
         self.data.input.store_draft(raw_input);
     }
 
-    pub fn record_text(&mut self, raw_input: input::RawInput) {
-        self.data.input.store_text(raw_input);
-    }
-
+    // The message's blocked state should be determined prior to using this
+    // function. In most cases, the best way to do that is by using the
+    // block_and_record_message function.
     pub fn record_message(
         &mut self,
         server: &Server,
-        casemapping: isupport::CaseMap,
-        mut message: crate::Message,
+        message: crate::Message,
         buffer_config: &config::Buffer,
     ) -> Option<impl Future<Output = Message> + use<>> {
         history::Kind::from_server_message(server.clone(), &message).and_then(
             |kind| {
-                self.block_message(
-                    &mut message,
-                    &kind,
-                    casemapping,
-                    buffer_config,
-                );
-
                 let condensers = (message
                     .can_condense(&buffer_config.server_messages.condense)
                     && !message.blocked)
@@ -360,6 +335,28 @@ impl Manager {
         )
     }
 
+    pub fn block_and_record_message(
+        &mut self,
+        server: &Server,
+        casemapping: isupport::CaseMap,
+        mut message: crate::Message,
+        buffer_config: &config::Buffer,
+    ) -> Option<impl Future<Output = Message> + use<>> {
+        if let Some(kind) =
+            history::Kind::from_server_message(server.clone(), &message)
+        {
+            self.block_message(
+                &mut message,
+                &kind,
+                server,
+                casemapping,
+                buffer_config,
+            );
+        }
+
+        self.record_message(server, message, buffer_config)
+    }
+
     pub fn record_log(
         &mut self,
         record: crate::log::Record,
@@ -368,8 +365,9 @@ impl Manager {
             .add_message(history::Kind::Logs, crate::Message::log(record))
     }
 
-    // Unlike record_message, the message's blocked status should be determined
-    // before recording a highlight in order to block highlight notifications.
+    // Unlike block_and_record_message, the message's blocked status should be
+    // determined before recording a highlight in order to block highlight
+    // notifications.
     pub fn record_highlight(
         &mut self,
         message: crate::Message,
@@ -377,6 +375,38 @@ impl Manager {
         self.data.add_message(history::Kind::Highlights, message)
     }
 
+    pub fn remove_message(
+        &mut self,
+        kind: history::Kind,
+        server_time: DateTime<Utc>,
+        hash: message::Hash,
+        resend: bool,
+    ) -> Option<impl Future<Output = Message> + use<>> {
+        self.data.remove_message(kind, server_time, hash, resend)
+    }
+
+    pub fn expand_condensed_message(
+        &mut self,
+        kind: history::Kind,
+        server_time: DateTime<Utc>,
+        hash: message::Hash,
+        config: &config::buffer::Condensation,
+    ) {
+        self.data
+            .expand_condensed_message(kind, server_time, hash, config);
+    }
+
+    pub fn contract_condensed_message(
+        &mut self,
+        kind: history::Kind,
+        server_time: DateTime<Utc>,
+        hash: message::Hash,
+        config: &config::buffer::Condensation,
+    ) {
+        self.data
+            .contract_condensed_message(kind, server_time, hash, config);
+    }
+
     pub fn update_read_marker<T: Into<history::Kind>>(
         &mut self,
         kind: T,
@@ -448,10 +478,9 @@ impl Manager {
             .filter_map(|kind| match kind {
                 #[allow(clippy::bool_comparison)] // easy to miss exclaimation
                 history::Kind::Query(s, query) => (s == server
-                    && self
-                        .filters
-                        .iter()
-                        .all(|filter| filter.match_query(query) == false))
+                    && self.filters.iter().all(|filter| {
+                        filter.match_query(query, server) == false
+                    }))
                 .then_some(query),
                 _ => None,
             })
@@ -549,7 +578,7 @@ impl Manager {
         messages
             .into_iter()
             .filter_map(|message| {
-                self.record_message(
+                self.block_and_record_message(
                     server,
                     casemapping,
                     message,
@@ -576,6 +605,7 @@ impl Manager {
         &self,
         message: &mut crate::Message,
         kind: &history::Kind,
+        server: &Server,
         casemapping: isupport::CaseMap,
         buffer_config: &config::Buffer,
     ) {
@@ -587,7 +617,12 @@ impl Manager {
         {
             // Check if target is a channel, and if included/excluded.
             if let message::Target::Channel { channel, .. } = &message.target
-                && !server_message.should_send_message(channel.as_str())
+                && !server_message.should_send_message(
+                    source.nick().map(Nick::as_nickref),
+                    channel,
+                    server,
+                    casemapping,
+                )
             {
                 message.blocked = true;
                 return;
@@ -684,12 +719,21 @@ impl Manager {
                 .position(|stored| stored.hash == message.hash)
                 .map(|position| position + start_index)
             {
+                let insert_date =
+                    message.server_time.with_timezone(&Local).date_naive();
+
                 let start = messages
                     .iter()
                     .take(insert_position)
                     .rev()
                     .position(|message| {
-                        !message.blocked && !message.can_condense(config)
+                        !message.blocked
+                            && (!message.can_condense(config)
+                                || message
+                                    .server_time
+                                    .with_timezone(&Local)
+                                    .date_naive()
+                                    != insert_date)
                     })
                     .map_or(0, |position| insert_position - position);
 
@@ -697,7 +741,13 @@ impl Manager {
                     .iter()
                     .skip(insert_position)
                     .position(|message| {
-                        !message.blocked && !message.can_condense(config)
+                        !message.blocked
+                            && (!message.can_condense(config)
+                                || message
+                                    .server_time
+                                    .with_timezone(&Local)
+                                    .date_naive()
+                                    != insert_date)
                     })
                     .map_or(messages.len(), |position| {
                         insert_position + position
@@ -705,8 +755,8 @@ impl Manager {
 
                 let mut condensable_messages = messages[start..end]
                     .iter_mut()
-                    .collect::<Vec<&mut message::Message>>(
-                );
+                    .filter(|message| !message.blocked)
+                    .collect::<Vec<&mut message::Message>>();
 
                 let condensed_message = message::condense(
                     &condensable_messages
@@ -753,6 +803,21 @@ impl Manager {
 
                 match message.target.source() {
                     message::Source::Server(Some(source)) => {
+                        let server = if let Some(server) = kind.server() {
+                            Some(server)
+                        } else if let message::Target::Highlights {
+                            server,
+                            ..
+                        } = &message.target
+                        {
+                            Some(server)
+                        } else {
+                            None
+                        };
+
+                        let casemapping =
+                            clients.get_casemapping_or_default(server);
+
                         if let Some(server_message) =
                             buffer_config.server_messages.get(source)
                         {
@@ -761,27 +826,19 @@ impl Manager {
                             | message::Target::Highlights {
                                 channel, ..
                             } = &message.target
-                                && !server_message
-                                    .should_send_message(channel.as_str())
+                                && let Some(server) = server
+                                && !server_message.should_send_message(
+                                    source.nick().map(Nick::as_nickref),
+                                    channel,
+                                    server,
+                                    casemapping,
+                                )
                             {
                                 message.blocked = true;
                             } else if let Some(seconds) = server_message.smart {
                                 let nick = match source.nick() {
                                     Some(nick) => Some(nick.clone()),
                                     None => message.plain().and_then(|s| {
-                                        let server = if let Some(server) = kind.server() {
-                                            Some(server)
-                                        } else if let message::Target::Highlights { server, .. } =
-                                            &message.target
-                                        {
-                                            Some(server)
-                                        } else {
-                                            None
-                                        };
-
-                                        let casemapping = clients
-                                            .get_casemapping_or_default(server);
-
                                         s.split(' ').nth(1).map(|nick| {
                                             Nick::from_str(nick, casemapping)
                                         })
@@ -1024,7 +1081,14 @@ impl Data {
                 } else if message
                     .can_condense(&buffer_config.server_messages.condense)
                 {
-                    message.condensed.as_ref().map(std::convert::AsRef::as_ref)
+                    if message.expanded {
+                        Some(message)
+                    } else {
+                        message
+                            .condensed
+                            .as_ref()
+                            .map(std::convert::AsRef::as_ref)
+                    }
                 } else {
                     match message.target.source() {
                         message::Source::Internal(
@@ -1112,7 +1176,7 @@ impl Data {
         // The right-aligned nicknames setting expects timestamps to have a
         // constant character count to function, so we can utilize that
         // expectation in this calculation
-        let max_excess_timestamp_chars = (buffer_config
+        let range_timestamp_extra_chars = (buffer_config
             .nickname
             .alignment
             .is_right()
@@ -1142,9 +1206,7 @@ impl Data {
                                 .unwrap_or_default()
                                 - buffer_config
                                     .format_timestamp(&message.server_time)
-                                    .map(|timestamp| {
-                                        timestamp.chars().count() + 1
-                                    })
+                                    .map(|timestamp| timestamp.chars().count())
                                     .unwrap_or_default(),
                         )
                     } else {
@@ -1196,7 +1258,7 @@ impl Data {
             new_messages: new.to_vec(),
             max_nick_chars,
             max_prefix_chars,
-            max_excess_timestamp_chars,
+            range_timestamp_extra_chars,
             cleared: *cleared,
         })
     }
@@ -1210,12 +1272,22 @@ impl Data {
             hash_map::Entry::Occupied(mut entry) => {
                 let read_marker = entry.get_mut().add_message(message);
 
-                read_marker.map(|read_marker| {
-                    async move {
-                        Message::SentMessageUpdated(kind.clone(), read_marker)
-                    }
-                    .boxed()
-                })
+                if let Some(read_marker) = read_marker {
+                    // Update the read marker immediately so the split is correct
+                    entry.get_mut().update_read_marker(read_marker);
+
+                    Some(
+                        async move {
+                            Message::SentMessageUpdated(
+                                kind.clone(),
+                                read_marker,
+                            )
+                        }
+                        .boxed(),
+                    )
+                } else {
+                    None
+                }
             }
             hash_map::Entry::Vacant(entry) => {
                 let _ = entry
@@ -1234,6 +1306,59 @@ impl Data {
         }
     }
 
+    fn remove_message(
+        &mut self,
+        kind: history::Kind,
+        server_time: DateTime<Utc>,
+        hash: message::Hash,
+        resend: bool,
+    ) -> Option<impl Future<Output = Message> + use<>> {
+        self.map.get_mut(&kind).and_then(|history| {
+            history
+                .remove_message(server_time, hash)
+                .and_then(|message| {
+                    resend.then_some(
+                        async move { Message::ResendMessage(kind, message) }
+                            .boxed(),
+                    )
+                })
+        })
+    }
+
+    fn expand_condensed_message(
+        &mut self,
+        kind: history::Kind,
+        server_time: DateTime<Utc>,
+        hash: message::Hash,
+        config: &config::buffer::Condensation,
+    ) {
+        if let Some(history) = self.map.get_mut(&kind) {
+            history
+                .get_condensed_messages(server_time, hash, config)
+                .iter_mut()
+                .for_each(|message| {
+                    message.expanded = true;
+                });
+        }
+    }
+
+    fn contract_condensed_message(
+        &mut self,
+        kind: history::Kind,
+        server_time: DateTime<Utc>,
+        hash: message::Hash,
+        config: &config::buffer::Condensation,
+    ) {
+        if let Some(history) = self.map.get_mut(&kind) {
+            history
+                .get_condensed_messages(server_time, hash, config)
+                .iter_mut()
+                .for_each(|message| {
+                    message.expanded = false;
+                });
+        }
+    }
+
     fn update_read_marker<T: Into<history::Kind>>(
         &mut self,
         kind: T,
