@@ -442,6 +442,7 @@ pub enum Numeric {
     RPL_CREATED = 3,
     RPL_MYINFO = 4,
     RPL_ISUPPORT = 5,
+    RPL_SNOMASK = 8,
     RPL_BOUNCE = 10,
     RPL_STATSCOMMANDS = 212,
     RPL_ENDOFSTATS = 219,
@@ -588,6 +589,7 @@ impl TryFrom<u16> for Numeric {
             3 => RPL_CREATED,
             4 => RPL_MYINFO,
             5 => RPL_ISUPPORT,
+            8 => RPL_SNOMASK,
             10 => RPL_BOUNCE,
             212 => RPL_STATSCOMMANDS,
             219 => RPL_ENDOFSTATS,
