@@ -1,7 +1,7 @@
 use std::io;
 
 use bytes::BytesMut;
-use proto::{format, parse, Message};
+use proto::{Message, format, parse};
 use tokio_util::codec::{Decoder, Encoder};
 
 pub type ParseResult<T = Message, E = parse::Error> = std::result::Result<T, E>;
@@ -12,7 +12,10 @@ impl Decoder for Codec {
     type Item = ParseResult;
     type Error = Error;
 
-    fn decode(&mut self, src: &mut BytesMut) -> Result<Option<Self::Item>, Self::Error> {
+    fn decode(
+        &mut self,
+        src: &mut BytesMut,
+    ) -> Result<Option<Self::Item>, Self::Error> {
         let Some(pos) = src.windows(2).position(|b| b == [b'\r', b'\n']) else {
             return Ok(None);
         };
@@ -26,7 +29,11 @@ impl Decoder for Codec {
 impl Encoder<Message> for Codec {
     type Error = Error;
 
-    fn encode(&mut self, message: Message, dst: &mut BytesMut) -> Result<(), Self::Error> {
+    fn encode(
+        &mut self,
+        message: Message,
+        dst: &mut BytesMut,
+    ) -> Result<(), Self::Error> {
         let encoded = format::message(message);
 
         dst.extend(encoded.into_bytes());
