@@ -2,6 +2,7 @@ use std::net::IpAddr;
 use std::path::PathBuf;
 use std::pin::Pin;
 
+#[cfg(feature = "tor")]
 use arti_client::DataStream as TorStream;
 use futures::{Sink, SinkExt, Stream, StreamExt};
 use tokio::io::{AsyncRead, AsyncWrite, AsyncWriteExt};
@@ -17,6 +18,7 @@ mod tls;
 
 pub enum IrcStream {
     Tcp(TcpStream),
+    #[cfg(feature = "tor")]
     Tor(TorStream),
 }
 
@@ -47,7 +49,9 @@ pub struct Config<'a> {
 impl<Codec> Connection<Codec> {
     pub async fn new(config: Config<'_>, codec: Codec) -> Result<Self, Error> {
         let stream = match config.proxy {
-            None => IrcStream::Tcp(TcpStream::connect((config.server, config.port)).await?),
+            None => IrcStream::Tcp(
+                TcpStream::connect((config.server, config.port)).await?,
+            ),
             Some(proxy) => proxy.connect(config.server, config.port).await?,
         };
 
@@ -88,7 +92,9 @@ impl<Codec> Connection<Codec> {
         let stream = IrcStream::Tcp(tcp);
 
         match security {
-            Security::Unsecured => Ok(Self::Unsecured(Framed::new(stream, codec))),
+            Security::Unsecured => {
+                Ok(Self::Unsecured(Framed::new(stream, codec)))
+            }
             Security::Secured { .. } => {
                 todo!();
             }
@@ -154,7 +160,10 @@ where
         delegate!(self.get_mut(), poll_ready_unpin(cx))
     }
 
-    fn start_send(self: std::pin::Pin<&mut Self>, item: Item) -> Result<(), Self::Error> {
+    fn start_send(
+        self: std::pin::Pin<&mut Self>,
+        item: Item,
+    ) -> Result<(), Self::Error> {
         delegate!(self.get_mut(), start_send_unpin(item))
     }
 
@@ -181,6 +190,7 @@ impl AsyncRead for IrcStream {
     ) -> std::task::Poll<std::io::Result<()>> {
         match self.get_mut() {
             IrcStream::Tcp(s) => Pin::new(s).poll_read(cx, buf),
+            #[cfg(feature = "tor")]
             IrcStream::Tor(s) => Pin::new(s).poll_read(cx, buf),
         }
     }
@@ -190,6 +200,7 @@ impl AsyncWrite for IrcStream {
     fn is_write_vectored(&self) -> bool {
         match self {
             IrcStream::Tcp(s) => s.is_write_vectored(),
+            #[cfg(feature = "tor")]
             IrcStream::Tor(s) => s.is_write_vectored(),
         }
     }
@@ -199,6 +210,7 @@ impl AsyncWrite for IrcStream {
     ) -> std::task::Poll<Result<(), std::io::Error>> {
         match self.get_mut() {
             IrcStream::Tcp(s) => Pin::new(s).poll_flush(cx),
+            #[cfg(feature = "tor")]
             IrcStream::Tor(s) => Pin::new(s).poll_flush(cx),
         }
     }
@@ -208,6 +220,7 @@ impl AsyncWrite for IrcStream {
     ) -> std::task::Poll<Result<(), std::io::Error>> {
         match self.get_mut() {
             IrcStream::Tcp(s) => Pin::new(s).poll_shutdown(cx),
+            #[cfg(feature = "tor")]
             IrcStream::Tor(s) => Pin::new(s).poll_shutdown(cx),
         }
     }
@@ -218,6 +231,7 @@ impl AsyncWrite for IrcStream {
     ) -> std::task::Poll<Result<usize, std::io::Error>> {
         match self.get_mut() {
             IrcStream::Tcp(s) => Pin::new(s).poll_write(cx, buf),
+            #[cfg(feature = "tor")]
             IrcStream::Tor(s) => Pin::new(s).poll_write(cx, buf),
         }
     }
@@ -228,6 +242,7 @@ impl AsyncWrite for IrcStream {
     ) -> std::task::Poll<Result<usize, std::io::Error>> {
         match self.get_mut() {
             IrcStream::Tcp(s) => Pin::new(s).poll_write_vectored(cx, bufs),
+            #[cfg(feature = "tor")]
             IrcStream::Tor(s) => Pin::new(s).poll_write_vectored(cx, bufs),
         }
     }
