@@ -1,5 +1,5 @@
 pub use data::appearance::theme::{
-    Buffer, Button, Buttons, General, ServerMessages, Styles, Text,
+    Buffer, Button, Buttons, Formatting, General, ServerMessages, Styles, Text,
     color_to_hex, hex_to_color,
 };
 use data::config;
@@ -18,7 +18,9 @@ pub mod progress_bar;
 pub mod rule;
 pub mod scrollable;
 pub mod selectable_text;
+pub mod svg;
 pub mod text;
+pub mod text_editor;
 pub mod text_input;
 
 // TODO: If we use non-standard font sizes, we should consider
@@ -94,6 +96,12 @@ impl iced::theme::Base for Theme {
     fn mode(&self) -> iced::theme::Mode {
         iced::theme::Mode::Dark
     }
+    fn name(&self) -> &str {
+        match self {
+            Theme::Selected(selected) => selected.name.as_str(),
+            Theme::Preview { preview, .. } => preview.name.as_str(),
+        }
+    }
 }
 
 impl combo_box::Catalog for Theme {}
