@@ -1,7 +1,5 @@
-use iced::{
-    widget::pane_grid::{Catalog, Highlight, Line, Style, StyleFn},
-    Background, Border, Color,
-};
+use iced::widget::pane_grid::{Catalog, Highlight, Line, Style, StyleFn};
+use iced::{Background, Border, Color};
 
 use super::Theme;
 
@@ -18,7 +16,7 @@ impl Catalog for Theme {
 }
 
 pub fn primary(theme: &Theme) -> Style {
-    let general = theme.colors().general;
+    let general = theme.styles().general;
 
     Style {
         hovered_region: Highlight {
