@@ -1,6 +1,8 @@
 use std::path::PathBuf;
 
-pub use data::buffer::{Autocomplete, Internal, Settings, Upstream};
+use chrono::{DateTime, Utc};
+pub use data::buffer::{Internal, Settings, Upstream};
+use data::config::buffer::text_input::Autocomplete;
 use data::dashboard::BufferAction;
 use data::target::{self, Target};
 use data::user::Nick;
@@ -8,6 +10,7 @@ use data::{Config, buffer, file_transfer, history, message, preview};
 use iced::{Size, Task};
 
 pub use self::channel::Channel;
+pub use self::channel_discovery::ChannelDiscovery;
 pub use self::file_transfers::FileTransfers;
 pub use self::highlights::Highlights;
 pub use self::logs::Logs;
@@ -16,8 +19,11 @@ pub use self::server::Server;
 use crate::Theme;
 use crate::screen::dashboard::sidebar;
 use crate::widget::Element;
+use crate::window::Window;
 
 pub mod channel;
+pub mod channel_discovery;
+pub mod context_menu;
 pub mod empty;
 pub mod file_transfers;
 pub mod highlights;
@@ -27,7 +33,6 @@ mod message_view;
 pub mod query;
 mod scroll_view;
 pub mod server;
-pub mod context_menu;
 
 #[derive(Clone, Debug)]
 pub enum Buffer {
@@ -38,6 +43,7 @@ pub enum Buffer {
     FileTransfers(FileTransfers),
     Logs(Logs),
     Highlights(Highlights),
+    ChannelDiscovery(ChannelDiscovery),
 }
 
 #[derive(Debug, Clone)]
@@ -48,12 +54,16 @@ pub enum Message {
     FileTransfers(file_transfers::Message),
     Logs(logs::Message),
     Highlights(highlights::Message),
+    ChannelList(channel_discovery::Message),
 }
 
 pub enum Event {
     ContextMenu(context_menu::Event),
-    OpenBuffers(Vec<(Target, BufferAction)>),
+    OpenBuffers(data::Server, Vec<(Target, BufferAction)>),
+    OpenInternalBuffer(buffer::Internal),
+    OpenServer(String),
     LeaveBuffers(Vec<Target>, Option<String>),
+    SelectedServer(data::Server),
     GoToMessage(data::Server, target::Channel, message::Hash),
     History(Task<history::manager::Message>),
     RequestOlderChatHistory,
@@ -62,11 +72,19 @@ pub enum Event {
     MarkAsRead(history::Kind),
     OpenUrl(String),
     ImagePreview(PathBuf, url::Url),
+    ExpandCondensedMessage(DateTime<Utc>, message::Hash),
+    ContractCondensedMessage(DateTime<Utc>, message::Hash),
+    InputSent {
+        history_task: Task<history::manager::Message>,
+        open_buffers: Vec<(Target, BufferAction)>,
+    },
+    SendUnsafeList(data::Server),
 }
 
 impl Buffer {
     pub fn from_data(
         buffer: data::Buffer,
+        history: &history::Manager,
         pane_size: Size,
         config: &Config,
     ) -> Self {
@@ -76,11 +94,11 @@ impl Buffer {
                     Self::Server(Server::new(server, pane_size, config))
                 }
                 buffer::Upstream::Channel(server, channel) => Self::Channel(
-                    Channel::new(server, channel, pane_size, config),
+                    Channel::new(server, channel, history, pane_size, config),
+                ),
+                buffer::Upstream::Query(server, query) => Self::Query(
+                    Query::new(server, query, history, pane_size, config),
                 ),
-                buffer::Upstream::Query(server, query) => {
-                    Self::Query(Query::new(server, query, pane_size, config))
-                }
             },
             data::Buffer::Internal(internal) => match internal {
                 buffer::Internal::FileTransfers => {
@@ -92,6 +110,9 @@ impl Buffer {
                 buffer::Internal::Highlights => {
                     Self::Highlights(Highlights::new(pane_size, config))
                 }
+                buffer::Internal::ChannelDiscovery(server) => {
+                    Self::ChannelDiscovery(ChannelDiscovery::new(server))
+                }
             },
         }
     }
@@ -107,7 +128,8 @@ impl Buffer {
             Buffer::Empty
             | Buffer::FileTransfers(_)
             | Buffer::Logs(_)
-            | Buffer::Highlights(_) => None,
+            | Buffer::Highlights(_)
+            | Buffer::ChannelDiscovery(_) => None,
         }
     }
 
@@ -120,6 +142,9 @@ impl Buffer {
             Buffer::FileTransfers(_) => Some(buffer::Internal::FileTransfers),
             Buffer::Logs(_) => Some(buffer::Internal::Logs),
             Buffer::Highlights(_) => Some(buffer::Internal::Highlights),
+            Buffer::ChannelDiscovery(state) => {
+                Some(buffer::Internal::ChannelDiscovery(state.server.clone()))
+            }
         }
     }
 
@@ -144,6 +169,9 @@ impl Buffer {
             Buffer::Highlights(_) => {
                 Some(data::Buffer::Internal(buffer::Internal::Highlights))
             }
+            Buffer::ChannelDiscovery(state) => Some(data::Buffer::Internal(
+                buffer::Internal::ChannelDiscovery(state.server.clone()),
+            )),
         }
     }
 
@@ -155,7 +183,8 @@ impl Buffer {
             Buffer::Empty
             | Buffer::FileTransfers(_)
             | Buffer::Logs(_)
-            | Buffer::Highlights(_) => None,
+            | Buffer::Highlights(_)
+            | Buffer::ChannelDiscovery(_) => None,
         }
     }
 
@@ -169,7 +198,8 @@ impl Buffer {
             | Buffer::Server(_)
             | Buffer::FileTransfers(_)
             | Buffer::Logs(_)
-            | Buffer::Highlights(_) => None,
+            | Buffer::Highlights(_)
+            | Buffer::ChannelDiscovery(_) => None,
         }
     }
 
@@ -179,19 +209,31 @@ impl Buffer {
         clients: &mut data::client::Map,
         history: &mut history::Manager,
         file_transfers: &mut file_transfer::Manager,
+        main_window: &Window,
         config: &Config,
     ) -> (Task<Message>, Option<Event>) {
         match (self, message) {
             (Buffer::Channel(state), Message::Channel(message)) => {
-                let (command, event) =
-                    state.update(message, clients, history, config);
+                let (command, event) = state.update(
+                    message,
+                    clients,
+                    history,
+                    main_window,
+                    config,
+                );
 
                 let event = event.map(|event| match event {
                     channel::Event::ContextMenu(event) => {
                         Event::ContextMenu(event)
                     }
-                    channel::Event::OpenBuffers(targets) => {
-                        Event::OpenBuffers(targets)
+                    channel::Event::OpenBuffers(server, targets) => {
+                        Event::OpenBuffers(server, targets)
+                    }
+                    channel::Event::OpenInternalBuffer(buffer) => {
+                        Event::OpenInternalBuffer(buffer)
+                    }
+                    channel::Event::OpenServer(server) => {
+                        Event::OpenServer(server)
                     }
                     channel::Event::LeaveBuffers(targets, reason) => {
                         Event::LeaveBuffers(targets, reason)
@@ -209,20 +251,46 @@ impl Buffer {
                     channel::Event::ImagePreview(path, url) => {
                         Event::ImagePreview(path, url)
                     }
+                    channel::Event::ExpandCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Event::ExpandCondensedMessage(server_time, hash),
+                    channel::Event::ContractCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Event::ContractCondensedMessage(server_time, hash),
+                    channel::Event::InputSent {
+                        history_task,
+                        open_buffers,
+                    } => Event::InputSent {
+                        history_task,
+                        open_buffers,
+                    },
                 });
 
                 (command.map(Message::Channel), event)
             }
             (Buffer::Server(state), Message::Server(message)) => {
-                let (command, event) =
-                    state.update(message, clients, history, config);
+                let (command, event) = state.update(
+                    message,
+                    clients,
+                    history,
+                    main_window,
+                    config,
+                );
 
                 let event = event.map(|event| match event {
                     server::Event::ContextMenu(event) => {
                         Event::ContextMenu(event)
                     }
-                    server::Event::OpenBuffers(targets) => {
-                        Event::OpenBuffers(targets)
+                    server::Event::OpenInternalBuffer(buffer) => {
+                        Event::OpenInternalBuffer(buffer)
+                    }
+                    server::Event::OpenServer(server) => {
+                        Event::OpenServer(server)
+                    }
+                    server::Event::OpenBuffers(server, targets) => {
+                        Event::OpenBuffers(server, targets)
                     }
                     server::Event::LeaveBuffers(targets, reason) => {
                         Event::LeaveBuffers(targets, reason)
@@ -233,20 +301,46 @@ impl Buffer {
                     server::Event::ImagePreview(path, url) => {
                         Event::ImagePreview(path, url)
                     }
+                    server::Event::ExpandCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Event::ExpandCondensedMessage(server_time, hash),
+                    server::Event::ContractCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Event::ContractCondensedMessage(server_time, hash),
+                    server::Event::InputSent {
+                        history_task,
+                        open_buffers,
+                    } => Event::InputSent {
+                        history_task,
+                        open_buffers,
+                    },
                 });
 
                 (command.map(Message::Server), event)
             }
             (Buffer::Query(state), Message::Query(message)) => {
-                let (command, event) =
-                    state.update(message, clients, history, config);
+                let (command, event) = state.update(
+                    message,
+                    clients,
+                    history,
+                    main_window,
+                    config,
+                );
 
                 let event = event.map(|event| match event {
                     query::Event::ContextMenu(event) => {
                         Event::ContextMenu(event)
                     }
-                    query::Event::OpenBuffers(targets) => {
-                        Event::OpenBuffers(targets)
+                    query::Event::OpenBuffers(server, targets) => {
+                        Event::OpenBuffers(server, targets)
+                    }
+                    query::Event::OpenInternalBuffer(buffer) => {
+                        Event::OpenInternalBuffer(buffer)
+                    }
+                    query::Event::OpenServer(server) => {
+                        Event::OpenServer(server)
                     }
                     query::Event::LeaveBuffers(targets, reason) => {
                         Event::LeaveBuffers(targets, reason)
@@ -264,6 +358,20 @@ impl Buffer {
                     query::Event::ImagePreview(path, url) => {
                         Event::ImagePreview(path, url)
                     }
+                    query::Event::ExpandCondensedMessage(server_time, hash) => {
+                        Event::ExpandCondensedMessage(server_time, hash)
+                    }
+                    query::Event::ContractCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Event::ContractCondensedMessage(server_time, hash),
+                    query::Event::InputSent {
+                        history_task,
+                        open_buffers,
+                    } => Event::InputSent {
+                        history_task,
+                        open_buffers,
+                    },
                 });
 
                 (command.map(Message::Query), event)
@@ -273,6 +381,39 @@ impl Buffer {
 
                 (command.map(Message::FileTransfers), None)
             }
+            (
+                Buffer::ChannelDiscovery(state),
+                Message::ChannelList(message),
+            ) => {
+                let (command, event) = state.update(message, config);
+
+                let event = event.map(|event| match event {
+                    channel_discovery::Event::SelectedServer(server) => {
+                        Event::SelectedServer(server)
+                    }
+                    channel_discovery::Event::SendUnsafeList(server) => {
+                        Event::SendUnsafeList(server)
+                    }
+                    channel_discovery::Event::OpenUrl(url) => {
+                        Event::OpenUrl(url)
+                    }
+                    channel_discovery::Event::OpenChannelForServer(
+                        server,
+                        channel,
+                    ) => Event::OpenBuffers(
+                        server,
+                        vec![(
+                            Target::Channel(channel),
+                            config.actions.buffer.click_channel_name,
+                        )],
+                    ),
+                    channel_discovery::Event::ContextMenu(event) => {
+                        Event::ContextMenu(event)
+                    }
+                });
+
+                (command.map(Message::ChannelList), event)
+            }
             (Buffer::Logs(state), Message::Logs(message)) => {
                 let (command, event) =
                     state.update(message, history, clients, config);
@@ -281,9 +422,6 @@ impl Buffer {
                     logs::Event::ContextMenu(event) => {
                         Event::ContextMenu(event)
                     }
-                    logs::Event::OpenBuffer(target, buffer_action) => {
-                        Event::OpenBuffers(vec![(target, buffer_action)])
-                    }
                     logs::Event::History(task) => Event::History(task),
                     logs::Event::MarkAsRead => {
                         Event::MarkAsRead(history::Kind::Logs)
@@ -292,6 +430,13 @@ impl Buffer {
                     logs::Event::ImagePreview(path, url) => {
                         Event::ImagePreview(path, url)
                     }
+                    logs::Event::ExpandCondensedMessage(server_time, hash) => {
+                        Event::ExpandCondensedMessage(server_time, hash)
+                    }
+                    logs::Event::ContractCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Event::ContractCondensedMessage(server_time, hash),
                 });
 
                 (command.map(Message::Logs), event)
@@ -304,9 +449,14 @@ impl Buffer {
                     highlights::Event::ContextMenu(event) => {
                         Event::ContextMenu(event)
                     }
-                    highlights::Event::OpenBuffer(target, buffer_action) => {
-                        Event::OpenBuffers(vec![(target, buffer_action)])
-                    }
+                    highlights::Event::OpenBuffer(
+                        server,
+                        target,
+                        buffer_action,
+                    ) => Event::OpenBuffers(
+                        server,
+                        vec![(target, buffer_action)],
+                    ),
                     highlights::Event::GoToMessage(
                         server,
                         channel,
@@ -317,6 +467,14 @@ impl Buffer {
                     highlights::Event::ImagePreview(path, url) => {
                         Event::ImagePreview(path, url)
                     }
+                    highlights::Event::ExpandCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Event::ExpandCondensedMessage(server_time, hash),
+                    highlights::Event::ContractCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Event::ContractCondensedMessage(server_time, hash),
                 });
 
                 (command.map(Message::Highlights), event)
@@ -363,6 +521,10 @@ impl Buffer {
                 highlights::view(state, clients, history, config, theme)
                     .map(Message::Highlights)
             }
+            Buffer::ChannelDiscovery(state) => {
+                channel_discovery::view(state, clients, config, theme)
+                    .map(Message::ChannelList)
+            }
         }
     }
 
@@ -400,6 +562,9 @@ impl Buffer {
             Buffer::Channel(channel) => channel.focus().map(Message::Channel),
             Buffer::Server(server) => server.focus().map(Message::Server),
             Buffer::Query(query) => query.focus().map(Message::Query),
+            Buffer::ChannelDiscovery(channel_discovery) => {
+                channel_discovery.focus().map(Message::ChannelList)
+            }
         }
     }
 
@@ -408,7 +573,8 @@ impl Buffer {
             Buffer::Empty
             | Buffer::FileTransfers(_)
             | Buffer::Logs(_)
-            | Buffer::Highlights(_) => {}
+            | Buffer::Highlights(_)
+            | Buffer::ChannelDiscovery(_) => {}
             Buffer::Channel(channel) => channel.reset(),
             Buffer::Server(server) => server.reset(),
             Buffer::Query(query) => query.reset(),
@@ -420,36 +586,39 @@ impl Buffer {
         nick: Nick,
         history: &mut history::Manager,
         autocomplete: &Autocomplete,
-    ) -> Task<Message> {
+    ) {
         match self {
             Buffer::Empty
             | Buffer::FileTransfers(_)
             | Buffer::Logs(_)
-            | Buffer::Highlights(_) => Task::none(),
-            Buffer::Server(state) => state
-                .input_view
-                .insert_user(nick, state.buffer.clone(), history, autocomplete)
-                .map(|message| {
-                    Message::Server(server::Message::InputView(message))
-                }),
-            Buffer::Channel(state) => state
-                .input_view
-                .insert_user(nick, state.buffer.clone(), history, autocomplete)
-                .map(|message| {
-                    Message::Channel(channel::Message::InputView(message))
-                }),
-            Buffer::Query(state) => state
-                .input_view
-                .insert_user(nick, state.buffer.clone(), history, autocomplete)
-                .map(|message| {
-                    Message::Query(query::Message::InputView(message))
-                }),
+            | Buffer::Highlights(_)
+            | Buffer::ChannelDiscovery(_) => (),
+            Buffer::Server(state) => state.input_view.insert_user(
+                nick,
+                state.buffer.clone(),
+                history,
+                autocomplete,
+            ),
+            Buffer::Channel(state) => state.input_view.insert_user(
+                nick,
+                state.buffer.clone(),
+                history,
+                autocomplete,
+            ),
+            Buffer::Query(state) => state.input_view.insert_user(
+                nick,
+                state.buffer.clone(),
+                history,
+                autocomplete,
+            ),
         }
     }
 
     pub fn scroll_up_page(&mut self) -> Task<Message> {
         match self {
-            Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
+            Buffer::Empty
+            | Buffer::FileTransfers(_)
+            | Buffer::ChannelDiscovery(_) => Task::none(),
             Buffer::Channel(channel) => {
                 channel.scroll_view.scroll_up_page().map(|message| {
                     Message::Channel(channel::Message::ScrollView(message))
@@ -482,7 +651,9 @@ impl Buffer {
 
     pub fn scroll_down_page(&mut self) -> Task<Message> {
         match self {
-            Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
+            Buffer::Empty
+            | Buffer::FileTransfers(_)
+            | Buffer::ChannelDiscovery(_) => Task::none(),
             Buffer::Channel(channel) => {
                 channel.scroll_view.scroll_down_page().map(|message| {
                     Message::Channel(channel::Message::ScrollView(message))
@@ -515,7 +686,9 @@ impl Buffer {
 
     pub fn scroll_to_start(&mut self, config: &Config) -> Task<Message> {
         match self {
-            Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
+            Buffer::Empty
+            | Buffer::FileTransfers(_)
+            | Buffer::ChannelDiscovery(_) => Task::none(),
             Buffer::Channel(channel) => {
                 channel.scroll_view.scroll_to_start(config).map(|message| {
                     Message::Channel(channel::Message::ScrollView(message))
@@ -549,7 +722,9 @@ impl Buffer {
 
     pub fn scroll_to_end(&mut self, config: &Config) -> Task<Message> {
         match self {
-            Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
+            Buffer::Empty
+            | Buffer::FileTransfers(_)
+            | Buffer::ChannelDiscovery(_) => Task::none(),
             Buffer::Channel(channel) => {
                 channel.scroll_view.scroll_to_end(config).map(|message| {
                     Message::Channel(channel::Message::ScrollView(message))
@@ -587,7 +762,9 @@ impl Buffer {
         config: &Config,
     ) -> Task<Message> {
         match self {
-            Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
+            Buffer::Empty
+            | Buffer::FileTransfers(_)
+            | Buffer::ChannelDiscovery(_) => Task::none(),
             Buffer::Channel(state) => state
                 .scroll_view
                 .scroll_to_message(
@@ -654,7 +831,9 @@ impl Buffer {
         config: &Config,
     ) -> Task<Message> {
         match self {
-            Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
+            Buffer::Empty
+            | Buffer::FileTransfers(_)
+            | Buffer::ChannelDiscovery(_) => Task::none(),
             Buffer::Channel(state) => state
                 .scroll_view
                 .scroll_to_backlog(
@@ -706,9 +885,73 @@ impl Buffer {
         }
     }
 
+    pub fn has_pending_scroll_to(&self) -> bool {
+        match self {
+            Buffer::Empty
+            | Buffer::FileTransfers(_)
+            | Buffer::ChannelDiscovery(_) => false,
+            Buffer::Channel(state) => state.scroll_view.has_pending_scroll_to(),
+            Buffer::Server(state) => state.scroll_view.has_pending_scroll_to(),
+            Buffer::Query(state) => state.scroll_view.has_pending_scroll_to(),
+            Buffer::Logs(state) => state.scroll_view.has_pending_scroll_to(),
+            Buffer::Highlights(state) => {
+                state.scroll_view.has_pending_scroll_to()
+            }
+        }
+    }
+
+    pub fn set_scroll_limit_for_pending_scroll_to(
+        &mut self,
+        history: &history::Manager,
+        config: &Config,
+    ) {
+        match self {
+            Buffer::Empty
+            | Buffer::FileTransfers(_)
+            | Buffer::ChannelDiscovery(_) => (),
+            Buffer::Channel(state) => {
+                state.scroll_view.set_scroll_limit_for_pending_scroll_to(
+                    scroll_view::Kind::Channel(&state.server, &state.target),
+                    history,
+                    config,
+                );
+            }
+            Buffer::Server(state) => {
+                state.scroll_view.set_scroll_limit_for_pending_scroll_to(
+                    scroll_view::Kind::Server(&state.server),
+                    history,
+                    config,
+                );
+            }
+            Buffer::Query(state) => {
+                state.scroll_view.set_scroll_limit_for_pending_scroll_to(
+                    scroll_view::Kind::Query(&state.server, &state.target),
+                    history,
+                    config,
+                );
+            }
+            Buffer::Logs(state) => {
+                state.scroll_view.set_scroll_limit_for_pending_scroll_to(
+                    scroll_view::Kind::Logs,
+                    history,
+                    config,
+                );
+            }
+            Buffer::Highlights(state) => {
+                state.scroll_view.set_scroll_limit_for_pending_scroll_to(
+                    scroll_view::Kind::Highlights,
+                    history,
+                    config,
+                );
+            }
+        }
+    }
+
     pub fn is_scrolled_to_bottom(&self) -> Option<bool> {
         match self {
-            Buffer::Empty | Buffer::FileTransfers(_) => None,
+            Buffer::Empty
+            | Buffer::FileTransfers(_)
+            | Buffer::ChannelDiscovery(_) => None,
             Buffer::Channel(channel) => {
                 Some(channel.scroll_view.is_scrolled_to_bottom())
             }
@@ -730,7 +973,8 @@ impl Buffer {
             Buffer::Empty
             | Buffer::FileTransfers(_)
             | Buffer::Logs(_)
-            | Buffer::Highlights(_) => false,
+            | Buffer::Highlights(_)
+            | Buffer::ChannelDiscovery(_) => false,
             Buffer::Server(state) => state.input_view.close_picker(),
             Buffer::Channel(state) => state.input_view.close_picker(),
             Buffer::Query(state) => state.input_view.close_picker(),
@@ -739,7 +983,9 @@ impl Buffer {
 
     pub fn update_pane_size(&mut self, pane_size: Size, config: &Config) {
         match self {
-            Buffer::Empty | Buffer::FileTransfers(_) => (),
+            Buffer::Empty
+            | Buffer::FileTransfers(_)
+            | Buffer::ChannelDiscovery(_) => (),
             Buffer::Channel(channel) => {
                 channel.scroll_view.update_pane_size(pane_size, config);
             }
