@@ -1,11 +1,12 @@
 use std::path::PathBuf;
 
+use chrono::{DateTime, Utc};
 use data::dashboard::BufferAction;
 use data::preview::{self, Previews};
 use data::server::Server;
 use data::target::{self, Target};
 use data::user::{ChannelUsers, Nick};
-use data::{Config, User, buffer, history, message};
+use data::{Config, Preview, User, buffer, client, history, message};
 use iced::widget::{column, container, row};
 use iced::{Length, Size, Task, padding};
 
@@ -13,6 +14,7 @@ use super::message_view::{ChannelQueryLayout, TargetInfo};
 use super::{context_menu, input_view, scroll_view};
 use crate::Theme;
 use crate::widget::Element;
+use crate::window::Window;
 
 mod topic;
 
@@ -26,7 +28,9 @@ pub enum Message {
 
 pub enum Event {
     ContextMenu(context_menu::Event),
-    OpenBuffers(Vec<(Target, BufferAction)>),
+    OpenBuffers(Server, Vec<(Target, BufferAction)>),
+    OpenInternalBuffer(buffer::Internal),
+    OpenServer(String),
     LeaveBuffers(Vec<Target>, Option<String>),
     History(Task<history::manager::Message>),
     RequestOlderChatHistory,
@@ -35,6 +39,12 @@ pub enum Event {
     MarkAsRead(history::Kind),
     OpenUrl(String),
     ImagePreview(PathBuf, url::Url),
+    ExpandCondensedMessage(DateTime<Utc>, message::Hash),
+    ContractCondensedMessage(DateTime<Utc>, message::Hash),
+    InputSent {
+        history_task: Task<history::manager::Message>,
+        open_buffers: Vec<(Target, BufferAction)>,
+    },
 }
 
 pub fn view<'a>(
@@ -48,12 +58,17 @@ pub fn view<'a>(
     is_focused: bool,
 ) -> Element<'a, Message> {
     let server = &state.server;
+    let connected = matches!(clients.status(server), client::Status::Connected);
     let chantypes = clients.get_chantypes(server);
     let casemapping = clients.get_casemapping(server);
     let prefix = clients.get_prefix(server);
     let channel = &state.target;
-    let buffer = &state.buffer;
-    let input = history.input(buffer);
+    let confirm_message_delivery = clients.get_server_supports_echoes(server)
+        && config.servers.get(server).is_some_and(|server_config| {
+            server_config
+                .confirm_message_delivery
+                .is_target_channel_included(channel, server, casemapping)
+        });
     let our_nick: Option<data::user::NickRef<'_>> =
         clients.nickname(&state.server);
 
@@ -71,6 +86,7 @@ pub fn view<'a>(
     let previews = Some(Previews::new(
         previews,
         &channel.to_target(),
+        server,
         &config.preview,
         casemapping,
     ));
@@ -80,6 +96,8 @@ pub fn view<'a>(
         chantypes,
         casemapping,
         prefix,
+        confirm_message_delivery,
+        connected,
         server,
         theme,
         target: TargetInfo::Channel {
@@ -95,6 +113,15 @@ pub fn view<'a>(
             scroll_view::Kind::Channel(&state.server, channel),
             history,
             previews,
+            Some(|preview: &Preview, source: &message::Source| {
+                preview.visible_for_source(
+                    source,
+                    Some(channel),
+                    Some(server),
+                    casemapping,
+                    &config.preview,
+                )
+            }),
             chathistory_state,
             config,
             theme,
@@ -116,8 +143,8 @@ pub fn view<'a>(
         .unwrap_or_else(|| column![].into());
 
     let show_text_input = match config.buffer.text_input.visibility {
-        data::buffer::TextInputVisibility::Focused => is_focused,
-        data::buffer::TextInputVisibility::Always => true,
+        data::config::buffer::text_input::Visibility::Focused => is_focused,
+        data::config::buffer::text_input::Visibility::Always => true,
     };
 
     let mut channels = clients.get_channels(&state.server);
@@ -126,8 +153,6 @@ pub fn view<'a>(
     let text_input = show_text_input.then(move || {
         input_view::view(
             &state.input_view,
-            input,
-            is_focused,
             our_user,
             !is_connected_to_channel,
             config,
@@ -174,15 +199,20 @@ impl Channel {
     pub fn new(
         server: Server,
         target: target::Channel,
+        history: &history::Manager,
         pane_size: Size,
         config: &Config,
     ) -> Self {
+        let buffer = buffer::Upstream::Channel(server.clone(), target.clone());
+
         Self {
-            buffer: buffer::Upstream::Channel(server.clone(), target.clone()),
+            input_view: input_view::State::new(Some(
+                history.input(&buffer).draft,
+            )),
+            buffer,
             server,
             target,
             scroll_view: scroll_view::State::new(pane_size, config),
-            input_view: input_view::State::new(),
         }
     }
 
@@ -191,6 +221,7 @@ impl Channel {
         message: Message,
         clients: &mut data::client::Map,
         history: &mut history::Manager,
+        main_window: &Window,
         config: &Config,
     ) -> (Task<Message>, Option<Event>) {
         match message {
@@ -208,9 +239,14 @@ impl Channel {
                     scroll_view::Event::ContextMenu(event) => {
                         Some(Event::ContextMenu(event))
                     }
-                    scroll_view::Event::OpenBuffer(target, buffer_action) => {
-                        Some(Event::OpenBuffers(vec![(target, buffer_action)]))
-                    }
+                    scroll_view::Event::OpenBuffer(
+                        server,
+                        target,
+                        buffer_action,
+                    ) => Some(Event::OpenBuffers(
+                        server,
+                        vec![(target, buffer_action)],
+                    )),
                     scroll_view::Event::GoToMessage(..) => None,
                     scroll_view::Event::RequestOlderChatHistory => {
                         Some(Event::RequestOlderChatHistory)
@@ -233,6 +269,16 @@ impl Channel {
                     scroll_view::Event::ImagePreview(path, url) => {
                         Some(Event::ImagePreview(path, url))
                     }
+                    scroll_view::Event::ExpandCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Some(Event::ExpandCondensedMessage(server_time, hash)),
+                    scroll_view::Event::ContractCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => {
+                        Some(Event::ContractCondensedMessage(server_time, hash))
+                    }
                 });
 
                 (command.map(Message::ScrollView), event)
@@ -243,12 +289,16 @@ impl Channel {
                     &self.buffer,
                     clients,
                     history,
+                    main_window,
                     config,
                 );
                 let command = command.map(Message::InputView);
 
                 match event {
-                    Some(input_view::Event::InputSent { history_task }) => {
+                    Some(input_view::Event::InputSent {
+                        history_task,
+                        open_buffers,
+                    }) => {
                         let command = Task::batch(vec![
                             command,
                             self.scroll_view
@@ -256,11 +306,18 @@ impl Channel {
                                 .map(Message::ScrollView),
                         ]);
 
-                        (command, Some(Event::History(history_task)))
-                    }
-                    Some(input_view::Event::OpenBuffers { targets }) => {
-                        (command, Some(Event::OpenBuffers(targets)))
+                        (
+                            command,
+                            Some(Event::InputSent {
+                                history_task,
+                                open_buffers,
+                            }),
+                        )
                     }
+                    Some(input_view::Event::OpenBuffers {
+                        server,
+                        targets,
+                    }) => (command, Some(Event::OpenBuffers(server, targets))),
                     Some(input_view::Event::LeaveBuffers {
                         targets,
                         reason,
@@ -268,6 +325,12 @@ impl Channel {
                     Some(input_view::Event::Cleared { history_task }) => {
                         (command, Some(Event::History(history_task)))
                     }
+                    Some(input_view::Event::OpenInternalBuffer(buffer)) => {
+                        (command, Some(Event::OpenInternalBuffer(buffer)))
+                    }
+                    Some(input_view::Event::OpenServer(server)) => {
+                        (command, Some(Event::OpenServer(server)))
+                    }
                     None => (command, None),
                 }
             }
@@ -281,11 +344,14 @@ impl Channel {
                     topic::Event::ContextMenu(event) => {
                         Event::ContextMenu(event)
                     }
-                    topic::Event::OpenChannel(channel) => {
-                        Event::OpenBuffers(vec![(
-                            Target::Channel(channel),
-                            config.actions.buffer.click_channel_name,
-                        )])
+                    topic::Event::OpenChannel(server, channel) => {
+                        Event::OpenBuffers(
+                            server,
+                            vec![(
+                                Target::Channel(channel),
+                                config.actions.buffer.click_channel_name,
+                            )],
+                        )
                     }
                     topic::Event::OpenUrl(url) => Event::OpenUrl(url),
                 }),
@@ -334,7 +400,7 @@ fn topic<'a>(
             prefix,
             &state.target,
             topic.content.as_ref()?,
-            topic.who.as_ref().map(Nick::as_nickref),
+            topic.who.as_ref(),
             topic.time.as_ref(),
             config.buffer.channel.topic_banner.max_lines,
             users,
