@@ -1,9 +1,9 @@
 use chrono::{DateTime, Local, Utc};
 use data::dashboard::BufferAction;
 use data::user::Nick;
-use data::{Config, Server, User, config, ctcp, isupport, target};
+use data::{Config, Server, User, config, ctcp, isupport, message, target};
 use iced::widget::{Space, button, column, container, row, rule, text};
-use iced::{Length, Padding, padding};
+use iced::{Length, Padding};
 
 use crate::widget::{Element, context_menu, double_pass};
 use crate::{Theme, font, theme, widget};
@@ -18,12 +18,14 @@ pub enum Context<'a> {
     },
     Url(&'a String),
     Timestamp(&'a DateTime<Utc>),
+    NotSentMessage(&'a DateTime<Utc>, &'a message::Hash),
 }
 
 #[derive(Debug, Clone, Copy)]
 pub enum Entry {
     // user context
     Whois,
+    Whowas,
     Query,
     ToggleAccessLevelOp,
     ToggleAccessLevelVoice,
@@ -34,51 +36,98 @@ pub enum Entry {
     CtcpRequestVersion,
     // url context
     CopyUrl,
+    OpenUrl,
     // timestamp context
     Timestamp,
+    // not sent message context
+    DeleteMessage,
+    ResendMessage,
 }
 
 impl Entry {
+    pub fn not_sent_message_list(can_resend: bool) -> Vec<Self> {
+        if can_resend {
+            vec![Entry::DeleteMessage, Entry::ResendMessage]
+        } else {
+            vec![Entry::DeleteMessage]
+        }
+    }
+
     pub fn timestamp_list() -> Vec<Self> {
         vec![Entry::Timestamp]
     }
 
     pub fn url_list() -> Vec<Self> {
-        vec![Entry::CopyUrl]
+        vec![Entry::CopyUrl, Entry::OpenUrl]
     }
 
-    pub fn user_list(is_channel: bool, our_user: Option<&User>) -> Vec<Self> {
+    pub fn user_list(
+        is_channel: bool,
+        user_in_channel: Option<&User>,
+        our_user: Option<&User>,
+        file_transfer_enabled: bool,
+    ) -> Vec<Self> {
+        let user_is_online = user_in_channel.is_some();
+
         if is_channel {
-            if our_user.is_some_and(|u| {
+            if !user_is_online {
+                vec![Entry::UserInfo, Entry::HorizontalRule, Entry::Whowas]
+            } else if our_user.is_some_and(|u| {
                 u.has_access_level(data::user::AccessLevel::Oper)
             }) {
-                vec![
+                let mut list = vec![
                     Entry::UserInfo,
                     Entry::HorizontalRule,
                     Entry::Whois,
                     Entry::Query,
-                    Entry::SendFile,
+                ];
+
+                if file_transfer_enabled {
+                    list.push(Entry::SendFile);
+                }
+
+                list.extend(vec![
                     Entry::HorizontalRule,
                     Entry::ToggleAccessLevelOp,
                     Entry::ToggleAccessLevelVoice,
                     Entry::HorizontalRule,
                     Entry::CtcpRequestVersion,
                     Entry::CtcpRequestTime,
-                ]
+                ]);
+
+                list
             } else {
-                vec![
+                let mut list = vec![
                     Entry::UserInfo,
                     Entry::HorizontalRule,
                     Entry::Whois,
                     Entry::Query,
-                    Entry::SendFile,
+                ];
+
+                if file_transfer_enabled {
+                    list.push(Entry::SendFile);
+                }
+
+                list.extend(vec![
                     Entry::HorizontalRule,
                     Entry::CtcpRequestVersion,
                     Entry::CtcpRequestTime,
-                ]
+                ]);
+
+                list
             }
         } else {
-            vec![Entry::Whois, Entry::SendFile]
+            let mut list = vec![if user_is_online {
+                Entry::Whois
+            } else {
+                Entry::Whowas
+            }];
+
+            if file_transfer_enabled {
+                list.push(Entry::SendFile);
+            }
+
+            list
         }
     }
 
@@ -94,7 +143,25 @@ impl Entry {
                 let message =
                     Message::Whois(server.clone(), user.nickname().to_owned());
 
-                menu_button("Whois".to_string(), Some(message), length, theme)
+                menu_button(
+                    "Whois".to_string(),
+                    Some(message),
+                    length,
+                    theme,
+                    config,
+                )
+            }
+            (Entry::Whowas, Context::User { server, user, .. }) => {
+                let message =
+                    Message::Whowas(server.clone(), user.nickname().to_owned());
+
+                menu_button(
+                    "Whowas".to_string(),
+                    Some(message),
+                    length,
+                    theme,
+                    config,
+                )
             }
             (Entry::Query, Context::User { server, user, .. }) => {
                 let message = Message::Query(
@@ -103,7 +170,13 @@ impl Entry {
                     config.actions.buffer.message_user,
                 );
 
-                menu_button("Message".to_string(), Some(message), length, theme)
+                menu_button(
+                    "Message".to_string(),
+                    Some(message),
+                    length,
+                    theme,
+                    config,
+                )
             }
             (
                 Entry::ToggleAccessLevelOp,
@@ -144,7 +217,7 @@ impl Entry {
                         (String::new(), None)
                     };
 
-                menu_button(label, message, length, theme)
+                menu_button(label, message, length, theme, config)
             }
             (
                 Entry::ToggleAccessLevelVoice,
@@ -185,7 +258,7 @@ impl Entry {
                         (String::new(), None)
                     };
 
-                menu_button(label, message, length, theme)
+                menu_button(label, message, length, theme, config)
             }
             (Entry::SendFile, Context::User { server, user, .. }) => {
                 let message = Message::SendFile(server.clone(), user.clone());
@@ -195,6 +268,7 @@ impl Entry {
                     Some(message),
                     length,
                     theme,
+                    config,
                 )
             }
             (
@@ -228,6 +302,7 @@ impl Entry {
                     Some(message),
                     length,
                     theme,
+                    config,
                 )
             }
             (Entry::CtcpRequestVersion, Context::User { server, user, .. }) => {
@@ -243,6 +318,7 @@ impl Entry {
                     Some(message),
                     length,
                     theme,
+                    config,
                 )
             }
             (Entry::CopyUrl, Context::Url(url)) => {
@@ -253,6 +329,18 @@ impl Entry {
                     Some(message),
                     length,
                     theme,
+                    config,
+                )
+            }
+            (Entry::OpenUrl, Context::Url(url)) => {
+                let message = Message::OpenUrl(url.clone());
+
+                menu_button(
+                    "Open URL".to_string(),
+                    Some(message),
+                    length,
+                    theme,
+                    config,
                 )
             }
             (Entry::Timestamp, Context::Timestamp(date_time)) => {
@@ -271,6 +359,35 @@ impl Entry {
                     Some(message),
                     length,
                     theme,
+                    config,
+                )
+            }
+            (
+                Entry::DeleteMessage,
+                Context::NotSentMessage(server_time, hash),
+            ) => {
+                let message = Message::DeleteMessage(*server_time, *hash);
+
+                menu_button(
+                    "Delete Message".to_string(),
+                    Some(message),
+                    length,
+                    theme,
+                    config,
+                )
+            }
+            (
+                Entry::ResendMessage,
+                Context::NotSentMessage(server_time, hash),
+            ) => {
+                let message = Message::ResendMessage(*server_time, *hash);
+
+                menu_button(
+                    "Re-send Message".to_string(),
+                    Some(message),
+                    length,
+                    theme,
+                    config,
                 )
             }
             _ => row![].into(),
@@ -281,30 +398,41 @@ impl Entry {
 #[derive(Clone, Debug)]
 pub enum Message {
     Whois(Server, Nick),
+    Whowas(Server, Nick),
     Query(Server, target::Query, BufferAction),
     ToggleAccessLevel(Server, target::Channel, Nick, String),
     SendFile(Server, User),
     InsertNickname(Nick),
     CtcpRequest(ctcp::Command, Server, Nick, Option<String>),
     CopyUrl(String),
+    OpenUrl(String),
     CopyTimestamp(DateTime<Utc>, Option<String>),
+    #[allow(clippy::enum_variant_names)]
+    DeleteMessage(DateTime<Utc>, message::Hash),
+    #[allow(clippy::enum_variant_names)]
+    ResendMessage(DateTime<Utc>, message::Hash),
 }
 
 #[derive(Debug, Clone)]
 pub enum Event {
     SendWhois(Server, Nick),
+    SendWhowas(Server, Nick),
     OpenQuery(Server, target::Query, BufferAction),
     ToggleAccessLevel(Server, target::Channel, Nick, String),
     SendFile(Server, User),
     InsertNickname(Nick),
     CtcpRequest(ctcp::Command, Server, Nick, Option<String>),
     CopyUrl(String),
+    OpenUrl(String),
     CopyTimestamp(DateTime<Utc>, Option<String>),
+    DeleteMessage(DateTime<Utc>, message::Hash),
+    ResendMessage(DateTime<Utc>, message::Hash),
 }
 
 pub fn update(message: Message) -> Event {
     match message {
         Message::Whois(server, nick) => Event::SendWhois(server, nick),
+        Message::Whowas(server, nick) => Event::SendWhowas(server, nick),
         Message::Query(server, nick, buffer_action) => {
             Event::OpenQuery(server, nick, buffer_action)
         }
@@ -317,9 +445,16 @@ pub fn update(message: Message) -> Event {
             Event::CtcpRequest(command, server, nick, params)
         }
         Message::CopyUrl(url) => Event::CopyUrl(url),
+        Message::OpenUrl(url) => Event::OpenUrl(url),
         Message::CopyTimestamp(date_time, format) => {
             Event::CopyTimestamp(date_time, format)
         }
+        Message::DeleteMessage(sesrver_time, hash) => {
+            Event::DeleteMessage(sesrver_time, hash)
+        }
+        Message::ResendMessage(sesrver_time, hash) => {
+            Event::ResendMessage(sesrver_time, hash)
+        }
     }
 }
 
@@ -335,7 +470,12 @@ pub fn user<'a>(
     theme: &'a Theme,
     click: &'a config::buffer::NicknameClickAction,
 ) -> Element<'a, Message> {
-    let entries = Entry::user_list(channel.is_some(), our_user);
+    let entries = Entry::user_list(
+        channel.is_some(),
+        current_user,
+        our_user,
+        config.file_transfer.enabled,
+    );
 
     let message = match click {
         data::config::buffer::NicknameClickAction::OpenQuery => Message::Query(
@@ -400,25 +540,56 @@ pub fn timestamp<'a>(
     .into()
 }
 
+pub fn not_sent_message<'a>(
+    content: impl Into<Element<'a, Message>>,
+    server_time: &'a DateTime<Utc>,
+    hash: &'a message::Hash,
+    can_resend: bool,
+    config: &'a Config,
+    theme: &'a Theme,
+) -> Element<'a, Message> {
+    let entries = Entry::not_sent_message_list(can_resend);
+
+    context_menu(
+        context_menu::MouseButton::Left,
+        context_menu::Anchor::Cursor,
+        context_menu::ToggleBehavior::KeepOpen,
+        content,
+        entries,
+        move |entry, length| {
+            entry.view(
+                Some(Context::NotSentMessage(server_time, hash)),
+                length,
+                config,
+                theme,
+            )
+        },
+    )
+    .into()
+}
+
 fn menu_button(
     content: String,
     message: Option<Message>,
     length: Length,
     theme: &Theme,
+    config: &Config,
 ) -> Element<'static, Message> {
     button(
         text(content)
             .style(theme::text::primary)
             .font_maybe(theme::font_style::primary(theme).map(font::get)),
     )
-    .padding(5)
+    .padding(config.context_menu.padding.entry)
     .width(length)
     .on_press_maybe(message)
     .into()
 }
 
-fn right_justified_padding() -> Padding {
-    padding::all(5).right(5.0 + double_pass::horizontal_expansion())
+fn right_justified_padding(config: &Config) -> Padding {
+    let padding = config.context_menu.padding.entry;
+    Padding::from(padding)
+        .right(padding[1] as f32 + double_pass::horizontal_expansion())
 }
 
 fn user_info<'a>(
@@ -476,7 +647,7 @@ fn user_info<'a>(
 
     column![
         container(row![nickname, state].width(length).spacing(4))
-            .padding(right_justified_padding())
+            .padding(right_justified_padding(config))
     ]
     .into()
 }
