@@ -1,9 +1,10 @@
 use std::path::PathBuf;
 
+use chrono::{DateTime, Utc};
 use data::config::buffer::nickname::ShownStatus;
 use data::dashboard::BufferAction;
 use data::target::{self, Target};
-use data::{Config, Server, history, message};
+use data::{Config, Preview, Server, history, message};
 use iced::widget::{container, row, span};
 use iced::{Color, Length, Size, Task};
 
@@ -21,11 +22,13 @@ pub enum Message {
 
 pub enum Event {
     ContextMenu(context_menu::Event),
-    OpenBuffer(Target, BufferAction),
+    OpenBuffer(Server, Target, BufferAction),
     GoToMessage(Server, target::Channel, message::Hash),
     History(Task<history::manager::Message>),
     OpenUrl(String),
     ImagePreview(PathBuf, url::Url),
+    ExpandCondensedMessage(DateTime<Utc>, message::Hash),
+    ContractCondensedMessage(DateTime<Utc>, message::Hash),
 }
 
 pub fn view<'a>(
@@ -41,10 +44,12 @@ pub fn view<'a>(
             scroll_view::Kind::Highlights,
             history,
             None,
+            Option::<fn(&Preview, &message::Source) -> bool>::None,
             None,
             config,
             theme,
-            move |message: &'a data::Message, _, _, _| match &message.target {
+            move |message: &'a data::Message, _, _, _, _| match &message.target
+            {
                 message::Target::Highlights {
                     server,
                     channel,
@@ -153,16 +158,23 @@ pub fn view<'a>(
 
                     let text = message_content::with_context(
                         &message.content,
+                        server,
                         chantypes,
                         casemapping,
                         theme,
                         scroll_view::Message::Link,
+                        None,
                         theme::selectable_text::default,
                         theme::font_style::primary,
                         Option::<fn(Color) -> Color>::None,
                         move |link| match link {
-                            message::Link::User(_) => {
-                                context_menu::Entry::user_list(true, None)
+                            message::Link::User(_, _) => {
+                                context_menu::Entry::user_list(
+                                    true,
+                                    current_user,
+                                    None,
+                                    config.file_transfer.enabled,
+                                )
                             }
                             message::Link::Url(_) => {
                                 context_menu::Entry::url_list()
@@ -236,10 +248,12 @@ pub fn view<'a>(
 
                     let text = message_content(
                         &message.content,
+                        server,
                         chantypes,
                         casemapping,
                         theme,
                         scroll_view::Message::Link,
+                        None,
                         theme::selectable_text::action,
                         theme::font_style::action,
                         Option::<fn(Color) -> Color>::None,
@@ -304,9 +318,11 @@ impl Highlights {
                     scroll_view::Event::ContextMenu(event) => {
                         Some(Event::ContextMenu(event))
                     }
-                    scroll_view::Event::OpenBuffer(target, buffer_action) => {
-                        Some(Event::OpenBuffer(target, buffer_action))
-                    }
+                    scroll_view::Event::OpenBuffer(
+                        server,
+                        target,
+                        buffer_action,
+                    ) => Some(Event::OpenBuffer(server, target, buffer_action)),
                     scroll_view::Event::GoToMessage(
                         server,
                         channel,
@@ -322,6 +338,16 @@ impl Highlights {
                     scroll_view::Event::ImagePreview(path, url) => {
                         Some(Event::ImagePreview(path, url))
                     }
+                    scroll_view::Event::ExpandCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Some(Event::ExpandCondensedMessage(server_time, hash)),
+                    scroll_view::Event::ContractCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => {
+                        Some(Event::ContractCondensedMessage(server_time, hash))
+                    }
                 });
 
                 (command.map(Message::ScrollView), event)
