@@ -1,13 +1,12 @@
 use std::path::PathBuf;
 
-use data::dashboard::BufferAction;
-use data::target::Target;
-use data::{Config, client, history, isupport, message};
+use chrono::{DateTime, Utc};
+use data::{Config, Preview, client, history, message};
 use iced::widget::{container, row};
-use iced::{Color, Length, Size, Task};
+use iced::{Length, Size, Task};
 
 use super::{context_menu, scroll_view};
-use crate::widget::{Element, message_content, selectable_text};
+use crate::widget::{Element, selectable_text};
 use crate::{Theme, font, theme};
 
 #[derive(Debug, Clone)]
@@ -17,11 +16,12 @@ pub enum Message {
 
 pub enum Event {
     ContextMenu(context_menu::Event),
-    OpenBuffer(Target, BufferAction),
     History(Task<history::manager::Message>),
     MarkAsRead,
     OpenUrl(String),
     ImagePreview(PathBuf, url::Url),
+    ExpandCondensedMessage(DateTime<Utc>, message::Hash),
+    ContractCondensedMessage(DateTime<Utc>, message::Hash),
 }
 
 pub fn view<'a>(
@@ -36,10 +36,11 @@ pub fn view<'a>(
             scroll_view::Kind::Logs,
             history,
             None,
+            Option::<fn(&Preview, &message::Source) -> bool>::None,
             None,
             config,
             theme,
-            move |message: &'a data::Message, _, _, _| match message
+            move |message: &'a data::Message, _, _, _, _| match message
                 .target
                 .source()
             {
@@ -82,17 +83,11 @@ pub fn view<'a>(
                             .map(font::get),
                     );
 
-                    let message = message_content(
-                        &message.content,
-                        isupport::DEFAULT_CHANTYPES,
-                        isupport::CaseMap::default(),
-                        theme,
-                        scroll_view::Message::Link,
-                        theme::selectable_text::logs,
-                        theme::font_style::primary,
-                        Option::<fn(Color) -> Color>::None,
-                        config,
-                    );
+                    let message = selectable_text(message.text())
+                        .font_maybe(
+                            theme::font_style::primary(theme).map(font::get),
+                        )
+                        .style(theme::selectable_text::logs);
 
                     Some(
                         row![
@@ -153,9 +148,7 @@ impl Logs {
                     scroll_view::Event::ContextMenu(event) => {
                         Some(Event::ContextMenu(event))
                     }
-                    scroll_view::Event::OpenBuffer(target, buffer_action) => {
-                        Some(Event::OpenBuffer(target, buffer_action))
-                    }
+                    scroll_view::Event::OpenBuffer(_, _, _) => None,
                     scroll_view::Event::GoToMessage(_, _, _) => None,
                     scroll_view::Event::RequestOlderChatHistory => None,
                     scroll_view::Event::PreviewChanged => None,
@@ -167,6 +160,16 @@ impl Logs {
                     scroll_view::Event::ImagePreview(path, url) => {
                         Some(Event::ImagePreview(path, url))
                     }
+                    scroll_view::Event::ExpandCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Some(Event::ExpandCondensedMessage(server_time, hash)),
+                    scroll_view::Event::ContractCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => {
+                        Some(Event::ContractCondensedMessage(server_time, hash))
+                    }
                 });
 
                 (command.map(Message::ScrollView), event)
