@@ -1,19 +1,21 @@
+use chrono::{TimeDelta, Utc};
+use data::config::buffer::Dimmed;
 use data::config::buffer::nickname::ShownStatus;
 use data::isupport::{CaseMap, PrefixMap};
 use data::server::Server;
 use data::user::ChannelUsers;
 use data::{Config, User, message, target};
-use iced::Color;
-use iced::advanced::text;
-use iced::widget::{column, container, row};
+use iced::widget::text::LineHeight;
+use iced::widget::{Space, button, column, container, row, text};
+use iced::{Color, Length, alignment};
 
 use super::context_menu::{self, Context};
 use super::scroll_view::LayoutMessage;
 use crate::buffer::scroll_view::Message;
 use crate::widget::{
-    Element, message_content, message_marker, selectable_text, tooltip,
+    Element, Marker, message_content, message_marker, selectable_text, tooltip,
 };
-use crate::{Theme, font, theme};
+use crate::{Theme, font, icon, theme};
 
 #[derive(Clone, Copy)]
 pub enum TargetInfo<'a> {
@@ -58,6 +60,8 @@ pub struct ChannelQueryLayout<'a> {
     pub chantypes: &'a [char],
     pub casemapping: CaseMap,
     pub prefix: &'a [PrefixMap],
+    pub confirm_message_delivery: bool,
+    pub connected: bool,
     pub server: &'a Server,
     pub theme: &'a Theme,
     pub target: TargetInfo<'a>,
@@ -135,14 +139,14 @@ impl<'a> ChannelQueryLayout<'a> {
     fn format_prefixes(
         &self,
         message: &'a data::Message,
-        max_nick_width: Option<f32>,
+        right_aligned_width: Option<f32>,
         max_prefix_width: Option<f32>,
     ) -> Option<Element<'a, Message>> {
         message
             .target
             .prefixes()
             .map_or(
-                max_nick_width.and_then(|_| {
+                right_aligned_width.and_then(|_| {
                     max_prefix_width.map(|width| {
                         selectable_text("")
                             .width(width)
@@ -176,9 +180,20 @@ impl<'a> ChannelQueryLayout<'a> {
     fn format_user_message(
         &self,
         message: &'a data::Message,
-        max_nick_width: Option<f32>,
+        right_aligned_width: Option<f32>,
         user: &'a User,
+        hide_nickname: bool,
     ) -> (Element<'a, Message>, Element<'a, Message>) {
+        let not_sent = self.confirm_message_delivery
+            && message.command.is_some()
+            && matches!(message.direction, message::Direction::Sent)
+            && Utc::now().signed_duration_since(message.server_time)
+                > TimeDelta::seconds(10);
+
+        let dimmed = not_sent.then_some(Dimmed::new(None));
+        let dimmed_background_tuple = dimmed
+            .map(|dimmed| (dimmed, self.theme.styles().buffer.background));
+
         let with_access_levels = self.config.buffer.nickname.show_access_levels;
         let truncate = self.config.buffer.nickname.truncate;
         let user_in_channel = self
@@ -194,17 +209,21 @@ impl<'a> ChannelQueryLayout<'a> {
             ShownStatus::Historical => false,
         };
 
-        let nickname_style = theme::selectable_text::nickname(
+        let nickname_style = theme::selectable_text::dimmed(
+            theme::selectable_text::nickname(
+                self.theme,
+                self.config,
+                match self.config.buffer.nickname.shown_status {
+                    ShownStatus::Current => user_in_channel.unwrap_or(user),
+                    ShownStatus::Historical => user,
+                },
+                is_user_offline,
+            ),
             self.theme,
-            self.config,
-            match self.config.buffer.nickname.shown_status {
-                ShownStatus::Current => user_in_channel.unwrap_or(user),
-                ShownStatus::Historical => user,
-            },
-            is_user_offline,
+            dimmed_background_tuple,
         );
 
-        let mut text = selectable_text(
+        let mut nick_text = selectable_text(
             self.config
                 .buffer
                 .nickname
@@ -217,13 +236,13 @@ impl<'a> ChannelQueryLayout<'a> {
                 .map(font::get),
         );
 
-        if let Some(width) = max_nick_width {
-            text = text.width(width).align_x(text::Alignment::Right);
+        if let Some(width) = right_aligned_width {
+            nick_text = nick_text.width(width).align_x(text::Alignment::Right);
         }
 
         let nick = tooltip(
             context_menu::user(
-                text,
+                nick_text,
                 self.server,
                 self.prefix,
                 self.target.channel(),
@@ -241,21 +260,55 @@ impl<'a> ChannelQueryLayout<'a> {
             self.theme,
         );
 
+        let nick_element: Element<_> = if hide_nickname {
+            let width = match self.config.buffer.nickname.alignment {
+                data::buffer::Alignment::Left
+                | data::buffer::Alignment::Top => 0.0,
+                data::buffer::Alignment::Right => {
+                    right_aligned_width.unwrap_or_default()
+                }
+            };
+            Space::new().width(width).into()
+        } else {
+            nick
+        };
+
         let formatter = *self;
 
+        let message_style = move |message_theme: &Theme| {
+            theme::selectable_text::dimmed(
+                theme::selectable_text::default(message_theme),
+                message_theme,
+                dimmed_background_tuple,
+            )
+        };
+
+        let color_transformation = dimmed.map(|dimmed| {
+            move |color: Color| -> Color {
+                dimmed.transform_color(
+                    color,
+                    formatter.theme.styles().buffer.background,
+                )
+            }
+        });
+
         let message_content = message_content::with_context(
             &message.content,
+            self.server,
             self.chantypes,
             self.casemapping,
             self.theme,
             Message::Link,
-            theme::selectable_text::default,
+            None,
+            message_style,
             theme::font_style::primary,
-            Option::<fn(Color) -> Color>::None,
+            color_transformation,
             move |link| match link {
-                message::Link::User(_) => context_menu::Entry::user_list(
+                message::Link::User(_, _) => context_menu::Entry::user_list(
                     formatter.target.is_channel(),
+                    user_in_channel,
                     formatter.target.our_user(),
+                    formatter.config.file_transfer.enabled,
                 ),
                 message::Link::Url(_) => context_menu::Entry::url_list(),
                 _ => vec![],
@@ -273,13 +326,52 @@ impl<'a> ChannelQueryLayout<'a> {
             self.config,
         );
 
-        (nick, Element::from(container(message_content)))
+        let content = if not_sent {
+            let font_size = 0.85
+                * self.config.font.size.map_or(theme::TEXT_SIZE, f32::from);
+            let icon_size =
+                LineHeight::default().to_absolute(font_size.into()).0;
+
+            Element::from(column![
+                message_content,
+                context_menu::not_sent_message(
+                    button(
+                        row![
+                            icon::not_sent()
+                                .style(|theme, status| {
+                                    theme::svg::error(theme, status)
+                                })
+                                .height(icon_size)
+                                .width(Length::Shrink),
+                            text(" Message failed to send")
+                                .style(theme::text::error)
+                                .size(font_size)
+                        ]
+                        .align_y(alignment::Vertical::Center)
+                    )
+                    .style(|theme, status| {
+                        theme::button::bare(theme, status)
+                    })
+                    .padding(0),
+                    &message.server_time,
+                    &message.hash,
+                    message.command.is_some() && self.connected,
+                    self.config,
+                    self.theme,
+                )
+                .map(Message::ContextMenu)
+            ])
+        } else {
+            Element::from(message_content)
+        };
+
+        (nick_element, content)
     }
 
     fn format_server_message(
         &self,
         message: &'a data::Message,
-        max_nick_width: Option<f32>,
+        right_aligned_width: Option<f32>,
         server: Option<&'a message::source::Server>,
     ) -> (Element<'a, Message>, Element<'a, Message>) {
         let formatter = *self;
@@ -287,37 +379,57 @@ impl<'a> ChannelQueryLayout<'a> {
         let dimmed = formatter.config.buffer.server_messages.dimmed(server);
 
         let message_style = move |message_theme: &Theme| {
-            let mut style =
-                theme::selectable_text::server(message_theme, server);
-
-            if let Some(dimmed) = dimmed {
-                style.color = style.color.map(|color| {
-                    dimmed.transform_color(
-                        color,
-                        formatter.theme.styles().buffer.background,
-                    )
-                });
-            }
-
-            style
+            theme::selectable_text::dimmed(
+                theme::selectable_text::server(message_theme, server),
+                message_theme,
+                dimmed.map(|dimmed| {
+                    (*dimmed, formatter.theme.styles().buffer.background)
+                }),
+            )
         };
         let message_font_style = move |message_theme: &Theme| {
             theme::font_style::server(message_theme, server)
         };
 
+        let link = message.expanded.then_some(
+            message::Link::ContractCondensedMessage(
+                message.server_time,
+                message.hash,
+            ),
+        );
+
+        let marker_style = move |message_theme: &Theme| {
+            if message.expanded || message.condensed.is_some() {
+                theme::selectable_text::condensed_marker(message_theme)
+            } else {
+                message_style(message_theme)
+            }
+        };
+
         let marker = message_marker(
-            max_nick_width,
-            self.theme,
-            message_style,
-            message_font_style,
+            if message.expanded {
+                if message.condensed.is_some() {
+                    Marker::Contract
+                } else {
+                    Marker::None
+                }
+            } else {
+                Marker::Dot
+            },
+            right_aligned_width,
+            self.config,
+            marker_style,
+            link.clone().map(Message::Link),
         );
 
         let message_content = message_content::with_context(
             &message.content,
+            formatter.server,
             formatter.chantypes,
             formatter.casemapping,
             self.theme,
             Message::Link,
+            link,
             message_style,
             message_font_style,
             Some(|color: Color| -> Color {
@@ -331,10 +443,21 @@ impl<'a> ChannelQueryLayout<'a> {
                 }
             }),
             move |link| match link {
-                message::Link::User(_) => context_menu::Entry::user_list(
-                    formatter.target.is_channel(),
-                    formatter.target.our_user(),
-                ),
+                message::Link::User(_, user) => {
+                    let user_in_channel = formatter
+                        .target
+                        .users()
+                        .into_iter()
+                        .flatten()
+                        .find(|u| *u == user);
+
+                    context_menu::Entry::user_list(
+                        formatter.target.is_channel(),
+                        user_in_channel,
+                        formatter.target.our_user(),
+                        formatter.config.file_transfer.enabled,
+                    )
+                }
                 message::Link::Url(_) => context_menu::Entry::url_list(),
                 _ => vec![],
             },
@@ -357,30 +480,51 @@ impl<'a> ChannelQueryLayout<'a> {
     fn format_condensed_message(
         &self,
         message: &'a data::Message,
-        spacer_width: Option<f32>,
+        right_aligned_width: Option<f32>,
     ) -> (Element<'a, Message>, Element<'a, Message>) {
-        let spacer = spacer_width.map(|width| selectable_text("").width(width));
+        let formatter = *self;
+
+        let dimmed = formatter.config.buffer.server_messages.condense.dimmed;
 
         let message_style = move |message_theme: &Theme| {
-            theme::selectable_text::server(message_theme, None)
+            theme::selectable_text::dimmed(
+                theme::selectable_text::server(message_theme, None),
+                message_theme,
+                dimmed.map(|dimmed| {
+                    (dimmed, formatter.theme.styles().buffer.background)
+                }),
+            )
         };
         let message_font_style = move |message_theme: &Theme| {
             theme::font_style::server(message_theme, None)
         };
 
-        let formatter = *self;
+        let link = message::Link::ExpandCondensedMessage(
+            message.server_time,
+            message.hash,
+        );
+        let moved_link = link.clone();
+
+        let marker = message_marker(
+            Marker::Expand,
+            right_aligned_width,
+            self.config,
+            theme::selectable_text::condensed_marker,
+            Some(Message::Link(link.clone())),
+        );
+
         let message_content = message_content::with_context(
             &message.content,
+            formatter.server,
             formatter.chantypes,
             formatter.casemapping,
             self.theme,
-            Message::Link,
+            move |_| Message::Link(moved_link.clone()),
+            Some(link),
             message_style,
             message_font_style,
             Some(|color: Color| -> Color {
-                if let Some(dimmed) =
-                    formatter.config.buffer.server_messages.condense.dimmed
-                {
+                if let Some(dimmed) = dimmed {
                     dimmed.transform_color(
                         color,
                         formatter.theme.styles().buffer.background,
@@ -390,10 +534,21 @@ impl<'a> ChannelQueryLayout<'a> {
                 }
             }),
             move |link| match link {
-                message::Link::User(_) => context_menu::Entry::user_list(
-                    formatter.target.is_channel(),
-                    formatter.target.our_user(),
-                ),
+                message::Link::User(_, user) => {
+                    let user_in_channel = formatter
+                        .target
+                        .users()
+                        .into_iter()
+                        .flatten()
+                        .find(|u| *u == user);
+
+                    context_menu::Entry::user_list(
+                        formatter.target.is_channel(),
+                        user_in_channel,
+                        formatter.target.our_user(),
+                        formatter.config.file_transfer.enabled,
+                    )
+                }
                 message::Link::Url(_) => context_menu::Entry::url_list(),
                 _ => vec![],
             },
@@ -410,7 +565,7 @@ impl<'a> ChannelQueryLayout<'a> {
             self.config,
         );
 
-        (spacer.into(), container(message_content).into())
+        (marker, container(message_content).into())
     }
 
     fn content_on_new_line(&self, message: &data::Message) -> bool {
@@ -450,13 +605,17 @@ impl<'a> LayoutMessage<'a> for ChannelQueryLayout<'a> {
     fn format(
         &self,
         message: &'a data::Message,
-        max_nick_width: Option<f32>,
+        right_aligned_width: Option<f32>,
         max_prefix_width: Option<f32>,
-        max_excess_timestamp_width: Option<f32>,
+        range_timestamp_excess_width: Option<f32>,
+        hide_nickname: bool,
     ) -> Option<Element<'a, Message>> {
         let timestamp = self.format_timestamp(message);
-        let prefixes =
-            self.format_prefixes(message, max_nick_width, max_prefix_width);
+        let prefixes = self.format_prefixes(
+            message,
+            right_aligned_width,
+            max_prefix_width,
+        );
 
         let row = row![timestamp, selectable_text(" "), prefixes];
 
@@ -464,39 +623,73 @@ impl<'a> LayoutMessage<'a> for ChannelQueryLayout<'a> {
             match message.target.source() {
                 message::Source::User(user) => Some(self.format_user_message(
                     message,
-                    max_nick_width,
+                    right_aligned_width,
                     user,
+                    hide_nickname,
                 )),
                 message::Source::Server(server_message) => {
                     Some(self.format_server_message(
                         message,
-                        max_nick_width,
+                        right_aligned_width,
                         server_message.as_ref(),
                     ))
                 }
                 message::Source::Action(_) => {
                     let marker = message_marker(
-                        max_nick_width,
-                        self.theme,
+                        Marker::Dot,
+                        right_aligned_width,
+                        self.config,
                         theme::selectable_text::action,
-                        theme::font_style::action,
+                        None,
                     );
 
-                    let message_content = message_content(
+                    let formatter = *self;
+                    let message_content = message_content::with_context(
                         &message.content,
-                        self.chantypes,
-                        self.casemapping,
-                        self.theme,
+                        formatter.server,
+                        formatter.chantypes,
+                        formatter.casemapping,
+                        formatter.theme,
                         Message::Link,
+                        None,
                         theme::selectable_text::action,
                         theme::font_style::action,
                         Option::<fn(Color) -> Color>::None,
-                        self.config,
+                        move |link| match link {
+                            message::Link::User(_, user) => {
+                                let user_in_channel = formatter
+                                    .target
+                                    .users()
+                                    .into_iter()
+                                    .flatten()
+                                    .find(|u| *u == user);
+
+                                context_menu::Entry::user_list(
+                                    formatter.target.is_channel(),
+                                    user_in_channel,
+                                    formatter.target.our_user(),
+                                    formatter.config.file_transfer.enabled,
+                                )
+                            }
+                            message::Link::Url(_) => {
+                                context_menu::Entry::url_list()
+                            }
+                            _ => vec![],
+                        },
+                        move |link, entry, length| {
+                            entry
+                                .view(
+                                    formatter.link_context(link),
+                                    length,
+                                    formatter.config,
+                                    formatter.theme,
+                                )
+                                .map(Message::ContextMenu)
+                        },
+                        formatter.config,
                     );
 
-                    let text_container = container(message_content);
-
-                    Some((marker, text_container.into()))
+                    Some((marker, container(message_content).into()))
                 }
                 message::Source::Internal(
                     message::source::Internal::Status(status),
@@ -509,18 +702,21 @@ impl<'a> LayoutMessage<'a> for ChannelQueryLayout<'a> {
                     };
 
                     let marker = message_marker(
-                        max_nick_width,
-                        self.theme,
+                        Marker::Dot,
+                        right_aligned_width,
+                        self.config,
                         message_style,
-                        message_font_style,
+                        None,
                     );
 
                     let message = message_content(
                         &message.content,
+                        self.server,
                         self.chantypes,
                         self.casemapping,
                         self.theme,
                         Message::Link,
+                        None,
                         message_style,
                         message_font_style,
                         Option::<fn(Color) -> Color>::None,
@@ -535,23 +731,38 @@ impl<'a> LayoutMessage<'a> for ChannelQueryLayout<'a> {
                 message::Source::Internal(
                     message::source::Internal::Condensed(end_server_time),
                 ) => {
-                    let spacer_width = if message.server_time
-                        != *end_server_time
-                    {
-                        max_nick_width.map(|max_nick_width| {
-                            max_nick_width
-                                - max_excess_timestamp_width.unwrap_or_default()
-                        })
-                    } else {
-                        max_nick_width
-                    };
+                    let right_aligned_width =
+                        if message.server_time != *end_server_time {
+                            right_aligned_width.map(|right_aligned_width| {
+                                right_aligned_width
+                                    - range_timestamp_excess_width
+                                        .unwrap_or_default()
+                            })
+                        } else {
+                            right_aligned_width
+                        };
 
                     (!message.text().is_empty()).then_some(
-                        self.format_condensed_message(message, spacer_width),
+                        self.format_condensed_message(
+                            message,
+                            right_aligned_width,
+                        ),
                     )
                 }
             }?;
-        let row = row.push(middle).push(selectable_text(" "));
+
+        // When hiding consecutive nicknames (left-aligned only), insert a single space
+        // to maintain visual separation. Right-aligned nicknames always get a space since
+        // they don't create a gap when hidden (alignment pushes content to the same position).
+        let maybe_space = if !hide_nickname
+            || self.config.buffer.nickname.alignment.is_right()
+        {
+            Some(selectable_text(" "))
+        } else {
+            None
+        };
+        let row = row.push(middle).push(maybe_space);
+
         if self.content_on_new_line(message) {
             Some(container(column![row, content]).into())
         } else {
