@@ -1,10 +1,11 @@
 use std::path::PathBuf;
 
+use chrono::{DateTime, Utc};
 use data::dashboard::BufferAction;
 use data::preview::{self, Previews};
 use data::target::{self, Target};
 use data::user::Nick;
-use data::{Config, Server, User, buffer, history, message};
+use data::{Config, Preview, Server, User, buffer, client, history, message};
 use iced::widget::{column, container, space};
 use iced::{Length, Size, Task};
 
@@ -12,6 +13,7 @@ use super::message_view::{ChannelQueryLayout, TargetInfo};
 use super::{context_menu, input_view, scroll_view};
 use crate::Theme;
 use crate::widget::Element;
+use crate::window::Window;
 
 #[derive(Debug, Clone)]
 pub enum Message {
@@ -21,7 +23,9 @@ pub enum Message {
 
 pub enum Event {
     ContextMenu(context_menu::Event),
-    OpenBuffers(Vec<(Target, BufferAction)>),
+    OpenBuffers(Server, Vec<(Target, BufferAction)>),
+    OpenInternalBuffer(buffer::Internal),
+    OpenServer(String),
     LeaveBuffers(Vec<Target>, Option<String>),
     History(Task<history::manager::Message>),
     RequestOlderChatHistory,
@@ -30,6 +34,12 @@ pub enum Event {
     MarkAsRead(history::Kind),
     OpenUrl(String),
     ImagePreview(PathBuf, url::Url),
+    ExpandCondensedMessage(DateTime<Utc>, message::Hash),
+    ContractCondensedMessage(DateTime<Utc>, message::Hash),
+    InputSent {
+        history_task: Task<history::manager::Message>,
+        open_buffers: Vec<(Target, BufferAction)>,
+    },
 }
 
 pub fn view<'a>(
@@ -42,14 +52,19 @@ pub fn view<'a>(
     is_focused: bool,
 ) -> Element<'a, Message> {
     let server = &state.server;
+    let connected = matches!(clients.status(server), client::Status::Connected);
     let chantypes = clients.get_chantypes(server);
     let casemapping = clients.get_casemapping(server);
     let prefix = clients.get_prefix(server);
     let query = &state.target;
+    let confirm_message_delivery = clients.get_server_supports_echoes(server)
+        && config.servers.get(server).is_some_and(|server_config| {
+            server_config
+                .confirm_message_delivery
+                .is_target_query_included(query, server, casemapping)
+        });
     let status = clients.status(server);
-    let buffer = &state.buffer;
-    let input = history.input(buffer);
-    let our_nick = clients.nickname(&state.server);
+    let our_nick = clients.nickname(server);
     let our_user = our_nick.map(|our_nick| User::from(Nick::from(our_nick)));
 
     let chathistory_state =
@@ -58,6 +73,7 @@ pub fn view<'a>(
     let previews = Some(Previews::new(
         previews,
         &query.to_target(),
+        server,
         &config.preview,
         casemapping,
     ));
@@ -67,6 +83,8 @@ pub fn view<'a>(
         chantypes,
         casemapping,
         prefix,
+        confirm_message_delivery,
+        connected,
         server,
         theme,
         target: TargetInfo::Query,
@@ -78,6 +96,7 @@ pub fn view<'a>(
             scroll_view::Kind::Query(server, query),
             history,
             previews,
+            Option::<fn(&Preview, &message::Source) -> bool>::None,
             chathistory_state,
             config,
             theme,
@@ -88,8 +107,8 @@ pub fn view<'a>(
     .height(Length::Fill);
 
     let show_text_input = match config.buffer.text_input.visibility {
-        data::buffer::TextInputVisibility::Focused => is_focused,
-        data::buffer::TextInputVisibility::Always => true,
+        data::config::buffer::text_input::Visibility::Focused => is_focused,
+        data::config::buffer::text_input::Visibility::Always => true,
     };
 
     let text_input = show_text_input.then(|| {
@@ -97,8 +116,6 @@ pub fn view<'a>(
             space::vertical().height(4),
             input_view::view(
                 &state.input_view,
-                input,
-                is_focused,
                 our_user.as_ref(),
                 !status.connected(),
                 config,
@@ -130,15 +147,20 @@ impl Query {
     pub fn new(
         server: Server,
         target: target::Query,
+        history: &history::Manager,
         pane_size: Size,
         config: &Config,
     ) -> Self {
+        let buffer = buffer::Upstream::Query(server.clone(), target.clone());
+
         Self {
-            buffer: buffer::Upstream::Query(server.clone(), target.clone()),
+            input_view: input_view::State::new(Some(
+                history.input(&buffer).draft,
+            )),
+            buffer,
             server,
             target,
             scroll_view: scroll_view::State::new(pane_size, config),
-            input_view: input_view::State::new(),
         }
     }
 
@@ -147,6 +169,7 @@ impl Query {
         message: Message,
         clients: &mut data::client::Map,
         history: &mut history::Manager,
+        main_window: &Window,
         config: &Config,
     ) -> (Task<Message>, Option<Event>) {
         match message {
@@ -164,9 +187,14 @@ impl Query {
                     scroll_view::Event::ContextMenu(event) => {
                         Some(Event::ContextMenu(event))
                     }
-                    scroll_view::Event::OpenBuffer(target, buffer_action) => {
-                        Some(Event::OpenBuffers(vec![(target, buffer_action)]))
-                    }
+                    scroll_view::Event::OpenBuffer(
+                        server,
+                        target,
+                        buffer_action,
+                    ) => Some(Event::OpenBuffers(
+                        server,
+                        vec![(target, buffer_action)],
+                    )),
                     scroll_view::Event::GoToMessage(_, _, _) => None,
                     scroll_view::Event::RequestOlderChatHistory => {
                         Some(Event::RequestOlderChatHistory)
@@ -189,6 +217,16 @@ impl Query {
                     scroll_view::Event::ImagePreview(path, url) => {
                         Some(Event::ImagePreview(path, url))
                     }
+                    scroll_view::Event::ExpandCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => Some(Event::ExpandCondensedMessage(server_time, hash)),
+                    scroll_view::Event::ContractCondensedMessage(
+                        server_time,
+                        hash,
+                    ) => {
+                        Some(Event::ContractCondensedMessage(server_time, hash))
+                    }
                 });
 
                 (command.map(Message::ScrollView), event)
@@ -199,12 +237,16 @@ impl Query {
                     &self.buffer,
                     clients,
                     history,
+                    main_window,
                     config,
                 );
                 let command = command.map(Message::InputView);
 
                 match event {
-                    Some(input_view::Event::InputSent { history_task }) => {
+                    Some(input_view::Event::InputSent {
+                        history_task,
+                        open_buffers,
+                    }) => {
                         let command = Task::batch(vec![
                             command,
                             self.scroll_view
@@ -212,11 +254,18 @@ impl Query {
                                 .map(Message::ScrollView),
                         ]);
 
-                        (command, Some(Event::History(history_task)))
-                    }
-                    Some(input_view::Event::OpenBuffers { targets }) => {
-                        (command, Some(Event::OpenBuffers(targets)))
+                        (
+                            command,
+                            Some(Event::InputSent {
+                                history_task,
+                                open_buffers,
+                            }),
+                        )
                     }
+                    Some(input_view::Event::OpenBuffers {
+                        server,
+                        targets,
+                    }) => (command, Some(Event::OpenBuffers(server, targets))),
                     Some(input_view::Event::LeaveBuffers {
                         targets,
                         reason,
@@ -224,6 +273,12 @@ impl Query {
                     Some(input_view::Event::Cleared { history_task }) => {
                         (command, Some(Event::History(history_task)))
                     }
+                    Some(input_view::Event::OpenInternalBuffer(buffer)) => {
+                        (command, Some(Event::OpenInternalBuffer(buffer)))
+                    }
+                    Some(input_view::Event::OpenServer(server)) => {
+                        (command, Some(Event::OpenServer(server)))
+                    }
                     None => (command, None),
                 }
             }
