@@ -4,6 +4,7 @@ use std::time::Duration;
 
 use chrono::{DateTime, Local, NaiveDate, NaiveTime, Utc};
 use data::buffer::DateSeparators;
+use data::config::buffer::nickname::HideConsecutiveEnabled;
 use data::dashboard::BufferAction;
 use data::isupport::ChatHistoryState;
 use data::message::{self, Limit};
@@ -22,8 +23,7 @@ use self::correct_viewport::correct_viewport;
 use self::keyed::keyed;
 use super::context_menu;
 use crate::widget::{
-    Element, MESSAGE_MARKER_TEXT, notify_visibility, on_resize,
-    selectable_text, tooltip,
+    Element, notify_visibility, on_resize, selectable_text, tooltip,
 };
 use crate::{Theme, font, icon, theme};
 
@@ -58,7 +58,7 @@ pub enum Message {
 #[derive(Debug, Clone)]
 pub enum Event {
     ContextMenu(context_menu::Event),
-    OpenBuffer(Target, BufferAction),
+    OpenBuffer(Server, Target, BufferAction),
     GoToMessage(Server, target::Channel, message::Hash),
     RequestOlderChatHistory,
     PreviewChanged,
@@ -66,6 +66,8 @@ pub enum Event {
     MarkAsRead,
     OpenUrl(String),
     ImagePreview(PathBuf, url::Url),
+    ExpandCondensedMessage(DateTime<Utc>, message::Hash),
+    ContractCondensedMessage(DateTime<Utc>, message::Hash),
 }
 
 #[derive(Debug, Clone, Copy)]
@@ -108,9 +110,10 @@ pub trait LayoutMessage<'a> {
     fn format(
         &self,
         message: &'a data::Message,
-        max_nick_width: Option<f32>,
+        right_aligned_width: Option<f32>,
         max_prefix_width: Option<f32>,
-        max_excess_timestamp_width: Option<f32>,
+        range_timestamp_excess_width: Option<f32>,
+        hide_nickname: bool,
     ) -> Option<Element<'a, Message>>;
 }
 
@@ -121,29 +124,79 @@ where
         Option<f32>,
         Option<f32>,
         Option<f32>,
+        bool,
     ) -> Option<Element<'a, Message>>,
 {
     fn format(
         &self,
         message: &'a data::Message,
-        max_nick_width: Option<f32>,
+        right_aligned_width: Option<f32>,
         max_prefix_width: Option<f32>,
-        max_excess_timestamp_width: Option<f32>,
+        range_timestamp_excess_width: Option<f32>,
+        hide_nickname: bool,
     ) -> Option<Element<'a, Message>> {
         self(
             message,
-            max_nick_width,
+            right_aligned_width,
             max_prefix_width,
-            max_excess_timestamp_width,
+            range_timestamp_excess_width,
+            hide_nickname,
         )
     }
 }
 
+/// Check if a message has a visible image preview
+fn has_visible_preview(
+    message: &data::Message,
+    state: &State,
+    previews: Option<Previews>,
+    visible_for_source: &Option<impl Fn(&Preview, &message::Source) -> bool>,
+) -> bool {
+    if let message::Content::Fragments(fragments) = &message.content
+        && let Some(previews) = previews
+        && let Some(visible_urls) =
+            state.visible_url_messages.get(&message.hash)
+    {
+        let urls = fragments
+            .iter()
+            .filter_map(message::Fragment::url)
+            .collect::<Vec<_>>();
+
+        return urls.iter().any(|url| {
+            // Check if URL is hidden by user
+            if message.hidden_urls.contains(url) {
+                return false;
+            }
+
+            // Check if URL is in visible URLs list
+            if !visible_urls.contains(url) {
+                return false;
+            }
+
+            // Check if preview is loaded and visible for source
+            if let Some(preview::State::Loaded(preview)) = previews.get(url) {
+                let is_visible_for_source =
+                    if let Some(visible_for_source) = visible_for_source {
+                        visible_for_source(preview, message.target.source())
+                    } else {
+                        true
+                    };
+
+                return is_visible_for_source;
+            }
+
+            false
+        });
+    }
+    false
+}
+
 pub fn view<'a>(
     state: &State,
     kind: Kind,
     history: &'a history::Manager,
     previews: Option<Previews<'a>>,
+    visible_for_source: Option<impl Fn(&Preview, &message::Source) -> bool>,
     chathistory_state: Option<ChatHistoryState>,
     config: &'a Config,
     theme: &'a Theme,
@@ -159,7 +212,7 @@ pub fn view<'a>(
         new_messages,
         max_nick_chars,
         max_prefix_chars,
-        max_excess_timestamp_chars,
+        range_timestamp_extra_chars,
         cleared,
         ..
     }) = history.get_messages(&kind.into(), Some(state.limit), &config.buffer)
@@ -184,12 +237,14 @@ pub fn view<'a>(
 
         let top_row_button = button(text(content).size(divider_font_size))
             .padding([3, 5])
-            .style(|theme, status| theme::button::primary(theme, status, false))
+            .style(|theme, status| {
+                theme::button::secondary(theme, status, false)
+            })
             .on_press_maybe(message);
 
         Some(
             row![space::horizontal(), top_row_button, space::horizontal()]
-                .padding(padding::top(2).bottom(6))
+                .padding(padding::top(4).bottom(6))
                 .width(Length::Fill)
                 .align_y(iced::Alignment::Center),
         )
@@ -205,41 +260,75 @@ pub fn view<'a>(
         .map_or_else(Utc::now, |message| message.server_time);
     let status = state.status;
 
-    let max_nick_width = max_nick_chars.map(|len| {
-        font::width_from_chars(
-            usize::max(
-                len,
-                usize::max(
-                    max_excess_timestamp_chars.unwrap_or_default(),
-                    MESSAGE_MARKER_TEXT.chars().count(),
-                ),
-            ),
-            &config.font,
-        )
+    let right_aligned_width = max_nick_chars.map(|max_nick_chars| {
+        let max_nick_width =
+            font::width_from_chars(max_nick_chars, &config.font);
+        let message_marker_width = font::width_of_message_marker(&config.font);
+        let range_timestamp_extra_width = range_timestamp_extra_chars.map_or(
+            0.0,
+            |range_timestamp_extra_chars| {
+                font::width_from_chars(
+                    range_timestamp_extra_chars,
+                    &config.font,
+                )
+            },
+        );
+
+        max_nick_width.max(range_timestamp_extra_width + message_marker_width)
     });
 
     let max_prefix_width =
         max_prefix_chars.map(|len| font::width_from_chars(len, &config.font));
 
-    let max_excess_timestamp_width = max_excess_timestamp_chars
+    let range_timestamp_excess_width = range_timestamp_extra_chars
         .map(|len| font::width_from_chars(len, &config.font));
 
     let message_rows = |last_date: Option<NaiveDate>,
                         messages: &[&'a data::Message]| {
         messages
             .iter()
-            .filter_map(|message| {
-                formatter
+            .scan(Option::<&data::Message>::None, |prev_message, message| {
+                let hide_nickname =
+                    if let HideConsecutiveEnabled::Enabled(duration) =
+                        config.buffer.nickname.hide_consecutive.enabled
+                    {
+                        !config.buffer.nickname.alignment.is_top() &&
+                        matches!(message.target.source(), message::Source::User(_)) &&
+                        // don't hide if prev message has visible preview (when show_after_previews is enabled)
+                        !(config.buffer.nickname.hide_consecutive.show_after_previews
+                                    && prev_message.is_some_and(|prev_msg| {
+                                        has_visible_preview(
+                                            prev_msg,
+                                            state,
+                                            previews,
+                                            &visible_for_source,
+                                        )
+                            })) &&
+                        prev_message.is_some_and(|prev_message| {
+                            matches!(
+                                (message.target.source(), prev_message.target.source()),
+                                (message::Source::User(user), message::Source::User(prev_user)) if user == prev_user
+                            ) && duration.is_none_or(|duration| message.server_time - prev_message.server_time < duration)
+                        })
+                    } else {
+                        false
+                    };
+
+                *prev_message = Some(message);
+
+                Some(formatter
                     .format(
                         message,
-                        max_nick_width,
+                        right_aligned_width,
                         max_prefix_width,
-                        max_excess_timestamp_width,
+                        range_timestamp_excess_width,
+                        hide_nickname,
                     )
                     .map(|element| {
                         (message, keyed(keyed::Key::message(message), element))
-                    })
+                    }))
             })
+            .flatten()
             .scan(last_date, |last_date, (message, element)| {
                 let date =
                     message.server_time.with_timezone(&Local).date_naive();
@@ -250,10 +339,9 @@ pub fn view<'a>(
 
                 let content = if let (
                     message::Content::Fragments(fragments),
-                    Some(previews),
-                    true,
+                    Some(previews)
                 ) =
-                    (&message.content, previews, config.preview.enabled)
+                    (&message.content, previews)
                 {
                     let urls = fragments
                         .iter()
@@ -283,17 +371,26 @@ pub fn view<'a>(
                                         |(a, b)| a == message.hash && b == idx,
                                     );
 
-                                column = column.push(preview_row(
-                                    message,
-                                    preview,
-                                    url,
-                                    idx,
-                                    max_nick_width,
-                                    max_prefix_width,
-                                    is_hovered,
-                                    config,
-                                    theme,
-                                ));
+                                let is_visible_for_source =
+                                    if let Some(visible_for_source) = &visible_for_source {
+                                        visible_for_source(preview, message.target.source())
+                                    } else {
+                                        true
+                                    };
+
+                                if is_visible_for_source {
+                                    column = column.push(preview_row(
+                                        message,
+                                        preview,
+                                        url,
+                                        idx,
+                                        right_aligned_width,
+                                        max_prefix_width,
+                                        is_hovered,
+                                        config,
+                                        theme,
+                                    ));
+                                }
                             }
                         }
 
@@ -395,20 +492,31 @@ pub fn view<'a>(
     };
 
     let divider = if show_backlog_divier {
-        row![
-            container(rule::horizontal(1))
-                .width(Length::Fill)
-                .padding(padding::right(6)),
-            text("backlog")
-                .size(divider_font_size)
-                .style(theme::text::secondary)
-                .font_maybe(theme::font_style::secondary(theme).map(font::get)),
-            container(rule::horizontal(1))
-                .width(Length::Fill)
-                .padding(padding::left(6))
-        ]
-        .padding(2)
-        .align_y(iced::Alignment::Center)
+        match &config.buffer.backlog_separator.text {
+            data::buffer::BacklogText::Hidden => row![
+                container(rule::horizontal(1).style(theme::rule::backlog))
+                    .padding([2, 0])
+                    .width(Length::Fill)
+            ]
+            .padding(2)
+            .align_y(iced::Alignment::Center),
+            data::buffer::BacklogText::Text(separator_text) => row![
+                container(rule::horizontal(1).style(theme::rule::backlog))
+                    .width(Length::Fill)
+                    .padding(padding::right(6)),
+                text(separator_text)
+                    .size(divider_font_size)
+                    .style(theme::text::secondary)
+                    .font_maybe(
+                        theme::font_style::secondary(theme).map(font::get)
+                    ),
+                container(rule::horizontal(1).style(theme::rule::backlog))
+                    .width(Length::Fill)
+                    .padding(padding::left(6))
+            ]
+            .padding(2)
+            .align_y(iced::Alignment::Center),
+        }
     } else {
         row![]
     };
@@ -643,10 +751,11 @@ impl State {
                     Some(Event::ContextMenu(context_menu::update(message))),
                 );
             }
-            Message::Link(message::Link::Channel(channel)) => {
+            Message::Link(message::Link::Channel(server, channel)) => {
                 return (
                     Task::none(),
                     Some(Event::OpenBuffer(
+                        server,
                         Target::Channel(channel),
                         config.actions.buffer.click_channel_name,
                     )),
@@ -655,12 +764,13 @@ impl State {
             Message::Link(message::Link::Url(url)) => {
                 return (Task::none(), Some(Event::OpenUrl(url)));
             }
-            Message::Link(message::Link::User(user)) => {
+            Message::Link(message::Link::User(server, user)) => {
                 let event = match config.buffer.nickname.click {
                     data::config::buffer::NicknameClickAction::OpenQuery => {
                         let query = target::Query::from(user);
 
                         Event::OpenBuffer(
+                            server,
                             Target::Query(query),
                             config.actions.buffer.click_username,
                         )
@@ -731,6 +841,24 @@ impl State {
                     );
                 }
             }
+            Message::Link(message::Link::ExpandCondensedMessage(
+                server_time,
+                hash,
+            )) => {
+                return (
+                    Task::none(),
+                    Some(Event::ExpandCondensedMessage(server_time, hash)),
+                );
+            }
+            Message::Link(message::Link::ContractCondensedMessage(
+                server_time,
+                hash,
+            )) => {
+                return (
+                    Task::none(),
+                    Some(Event::ContractCondensedMessage(server_time, hash)),
+                );
+            }
             Message::RequestOlderChatHistory => {
                 if let Some(server) = kind.server() {
                     self.status = Status::Unlocked;
@@ -790,7 +918,6 @@ impl State {
             }
             Message::PendingScrollTo => {
                 if let Some(key) = &self.pending_scroll_to {
-                    println!("pending_scroll_to → Task");
                     let scroll_to = keyed::find(self.scrollable.clone(), *key)
                         .map(Message::ScrollTo);
 
@@ -943,13 +1070,63 @@ impl State {
         })
     }
 
+    pub fn has_pending_scroll_to(&self) -> bool {
+        self.pending_scroll_to.is_some()
+    }
+
+    pub fn set_scroll_limit_for_pending_scroll_to(
+        &mut self,
+        kind: Kind,
+        history: &history::Manager,
+        config: &Config,
+    ) {
+        let Some(key) = self.pending_scroll_to else {
+            return;
+        };
+
+        let Some(history::View {
+            total,
+            old_messages,
+            new_messages,
+            ..
+        }) = history.get_messages(&kind.into(), None, &config.buffer)
+        else {
+            return;
+        };
+
+        let offset = match key {
+            keyed::Key::Message(message) => {
+                let Some(pos) = old_messages
+                    .iter()
+                    .chain(&new_messages)
+                    .position(|m| m.hash == message)
+                else {
+                    return;
+                };
+
+                // Get all messages from bottom until 1 before message
+                total - pos + 1
+            }
+            keyed::Key::Divider => {
+                // Get all messages from bottom until 1 before backlog
+                total - old_messages.len() + 1
+            }
+            keyed::Key::Preview(_, _) => return,
+        };
+
+        self.limit = Limit::Bottom(
+            offset.max(step_messages(2.0 * self.pane_size.height, config)),
+        );
+    }
+
     pub fn visible_urls(&self) -> impl Iterator<Item = &url::Url> {
         self.visible_url_messages.values().flatten()
     }
 }
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, Default)]
 pub enum Status {
+    #[default]
     Bottom,
     Unlocked,
 }
@@ -1023,16 +1200,10 @@ impl Status {
     }
 }
 
-impl Default for Status {
-    fn default() -> Self {
-        Self::Bottom
-    }
-}
-
 fn step_messages(height: f32, config: &Config) -> usize {
     let line_height = theme::line_height(&config.font);
 
-    (height / line_height) as usize
+    (height / line_height).max(8.0) as usize
 }
 
 mod keyed {
@@ -1287,7 +1458,7 @@ fn preview_row<'a>(
     preview: &'a Preview,
     url: &url::Url,
     idx: usize,
-    max_nick_width: Option<f32>,
+    right_aligned_width: Option<f32>,
     max_prefix_width: Option<f32>,
     is_hovered: bool,
     config: &'a Config,
@@ -1312,23 +1483,43 @@ fn preview_row<'a>(
                                     .map(font::get)
                             ),
                         description.as_ref().map(|description| {
-                            text(description)
-                                .shaping(text::Shaping::Advanced)
-                                .style(theme::text::secondary)
-                                .font_maybe(
-                                    theme::font_style::secondary(theme)
-                                        .map(font::get),
-                                )
+                            container(
+                                text(description)
+                                    .shaping(text::Shaping::Advanced)
+                                    .wrapping(text::Wrapping::WordOrGlyph)
+                                    .style(theme::text::secondary)
+                                    .font_maybe(
+                                        theme::font_style::secondary(theme)
+                                            .map(font::get),
+                                    ),
+                            )
+                            .clip(false)
+                            .max_height(
+                                config.preview.card.description_max_height,
+                            )
                         }),
                         config.preview.card.show_image.then_some(
                             container(
-                                image(path).content_fit(ContentFit::ScaleDown)
+                                image(path)
+                                    .border_radius(
+                                        if config
+                                            .preview
+                                            .card
+                                            .round_image_corners
+                                        {
+                                            4
+                                        } else {
+                                            0
+                                        }
+                                    )
+                                    .content_fit(ContentFit::ScaleDown)
                             )
-                            .max_height(200)
+                            .padding(Padding::default().top(8))
+                            .max_height(config.preview.card.image_max_height)
                         ),
                     ]
                     .spacing(8)
-                    .max_width(400),
+                    .max_width(config.preview.card.max_width),
                 )
                 .padding(8),
             )
@@ -1338,9 +1529,17 @@ fn preview_row<'a>(
         data::Preview::Image(preview::Image { path, url, .. }) => keyed(
             keyed::Key::Preview(message.hash, idx),
             button(
-                container(image(path).content_fit(ContentFit::ScaleDown))
-                    .max_width(550)
-                    .max_height(350),
+                container(
+                    image(path)
+                        .border_radius(if config.preview.image.round_corners {
+                            4
+                        } else {
+                            0
+                        })
+                        .content_fit(ContentFit::ScaleDown),
+                )
+                .max_width(config.preview.image.max_width)
+                .max_height(config.preview.image.max_height),
             )
             .on_press(match config.preview.image.action {
                 data::config::preview::ImageAction::OpenUrl => {
@@ -1361,12 +1560,15 @@ fn preview_row<'a>(
         .map(|timestamp| {
             selectable_text(" ".repeat(timestamp.chars().count()))
         });
+    let space = selectable_text(" ");
 
     let aligned_content = match &config.buffer.nickname.alignment {
-        data::buffer::Alignment::Left => row![timestamp_gap, content].into(),
+        data::buffer::Alignment::Left => {
+            row![timestamp_gap, space, content].into()
+        }
         data::buffer::Alignment::Right => {
             let prefixes = message.target.prefixes().map_or(
-                max_nick_width.and_then(|_| {
+                right_aligned_width.and_then(|_| {
                     max_prefix_width
                         .map(|width| selectable_text("").width(width))
                 }),
@@ -1392,7 +1594,6 @@ fn preview_row<'a>(
                 },
             );
 
-            let space = selectable_text(" ");
             let with_access_levels = config.buffer.nickname.show_access_levels;
             let truncate = config.buffer.nickname.truncate;
 
@@ -1411,7 +1612,7 @@ fn preview_row<'a>(
                     ),
                 );
 
-                if let Some(width) = max_nick_width {
+                if let Some(width) = right_aligned_width {
                     nick = nick.width(width);
                 }
 
@@ -1421,9 +1622,11 @@ fn preview_row<'a>(
             };
 
             let timestamp_nickname_row =
-                row![timestamp_gap, prefixes, nick, space,];
+                row![timestamp_gap, space, prefixes, nick];
 
-            row![timestamp_nickname_row, content].into()
+            let space = selectable_text(" ");
+
+            row![timestamp_nickname_row, space, content].into()
         }
         data::buffer::Alignment::Top => content,
     };
@@ -1549,8 +1752,8 @@ mod correct_viewport {
                                 let mut operation = scrollable::scroll_to(
                                     scrollable.clone(),
                                     scrollable::AbsoluteOffset {
-                                        x: 0.0,
-                                        y: new_offset,
+                                        x: None,
+                                        y: Some(new_offset),
                                     },
                                 );
                                 inner
@@ -1714,7 +1917,7 @@ mod correct_viewport {
                 state: &mut dyn Scrollable,
             ) {
                 if id.is_some_and(|id| *id == self.target) {
-                    state.scroll_to(self.offset);
+                    state.scroll_to(self.offset.into());
                 }
             }
 
