@@ -29,9 +29,10 @@ fn filtered_events(
             modifiers,
             ..
         }) if c.as_str() == "c" && modifiers.command() => Some(Event::Copy),
-        iced::Event::Mouse(mouse::Event::ButtonPressed(
-            mouse::Button::Left,
-        )) if ignored(status) => Some(Event::LeftClick),
+        iced::Event::Mouse(mouse::Event::ButtonPressed {
+            button: mouse::Button::Left,
+            ..
+        }) if ignored(status) => Some(Event::LeftClick),
         iced::Event::Mouse(mouse::Event::ButtonReleased(
             mouse::Button::Left,
         )) if cfg!(target_os = "linux") && ignored(status) => {
