@@ -1,25 +1,31 @@
-use std::{
-    env, mem,
-    sync::mpsc,
-    thread,
-    time::{Duration, Instant},
-};
+use std::sync::mpsc;
+use std::time::{Duration, Instant};
+use std::{env, mem, thread};
 
 use chrono::Utc;
+use data::config;
+pub use data::log::{Error, Record};
 use log::Log;
 use tokio::sync::mpsc as tokio_mpsc;
 use tokio_stream::wrappers::ReceiverStream;
 
-pub use data::log::{Error, Record};
-
-pub fn setup(is_debug: bool) -> Result<ReceiverStream<Vec<Record>>, Error> {
-    let level_filter = env::var("RUST_LOG")
+pub fn setup(
+    is_debug: bool,
+    config: config::Logs,
+) -> Result<ReceiverStream<Vec<Record>>, Error> {
+    let env_rust_log = env::var("RUST_LOG")
         .ok()
         .as_deref()
         .map(str::parse::<log::Level>)
-        .transpose()?
-        .unwrap_or(log::Level::Debug)
-        .to_level_filter();
+        .transpose()?;
+
+    let file_level_filter = env_rust_log
+        .map_or(log::LevelFilter::from(config.file_level), |env| {
+            env.to_level_filter()
+        });
+
+    let channel_level_filter = env_rust_log
+        .map_or(log::LevelFilter::Trace, |env| env.to_level_filter());
 
     let mut io_sink = fern::Dispatch::new().format(|out, message, record| {
         out.finish(format_args!(
@@ -38,14 +44,26 @@ pub fn setup(is_debug: bool) -> Result<ReceiverStream<Vec<Record>>, Error> {
         io_sink = io_sink.chain(log_file);
     }
 
-    let (channel_sink, receiver) = channel_logger();
-
-    fern::Dispatch::new()
+    io_sink = io_sink
         .level(log::LevelFilter::Off)
         .level_for("panic", log::LevelFilter::Error)
         .level_for("iced_wgpu", log::LevelFilter::Info)
-        .level_for("data", level_filter)
-        .level_for("halloy", level_filter)
+        .level_for("data", file_level_filter)
+        .level_for("ipc", file_level_filter)
+        .level_for("halloy", file_level_filter);
+
+    let (channel_sink, receiver) = channel_logger();
+
+    let channel_sink = fern::Dispatch::new()
+        .chain(channel_sink)
+        .level(log::LevelFilter::Off)
+        .level_for("panic", log::LevelFilter::Error)
+        .level_for("iced_wgpu", log::LevelFilter::Info)
+        .level_for("data", channel_level_filter)
+        .level_for("ipc", channel_level_filter)
+        .level_for("halloy", channel_level_filter);
+
+    fern::Dispatch::new()
         .chain(io_sink)
         .chain(channel_sink)
         .apply()?;
@@ -54,9 +72,6 @@ pub fn setup(is_debug: bool) -> Result<ReceiverStream<Vec<Record>>, Error> {
 }
 
 fn channel_logger() -> (Box<dyn Log>, ReceiverStream<Vec<Record>>) {
-    let (log_sender, log_receiver) = mpsc::channel();
-    let (async_sender, async_receiver) = tokio_mpsc::channel(1);
-
     struct Sink {
         sender: mpsc::Sender<Record>,
     }
@@ -77,6 +92,9 @@ fn channel_logger() -> (Box<dyn Log>, ReceiverStream<Vec<Record>>) {
         fn flush(&self) {}
     }
 
+    let (log_sender, log_receiver) = mpsc::channel();
+    let (async_sender, async_receiver) = tokio_mpsc::channel(1);
+
     thread::spawn(move || {
         const BATCH_SIZE: usize = 25;
         const BATCH_TIMEOUT: Duration = Duration::from_millis(250);
@@ -94,8 +112,10 @@ fn channel_logger() -> (Box<dyn Log>, ReceiverStream<Vec<Record>>) {
             {
                 timeout = Instant::now();
 
-                let _ = async_sender
-                    .blocking_send(mem::replace(&mut batch, Vec::with_capacity(BATCH_SIZE)));
+                let _ = async_sender.blocking_send(mem::replace(
+                    &mut batch,
+                    Vec::with_capacity(BATCH_SIZE),
+                ));
             }
         }
     });
