@@ -18,10 +18,11 @@ enum NotificationDelayKey {
     Disconnected,
     Reconnected,
     DirectMessage(Box<str>),
-    Highlight,
+    Highlight(Box<str>),
     FileTransferRequest(Box<str>),
     MonitoredOnline,
     MonitoredOffline,
+    Channel(Box<str>),
 }
 
 impl From<&Notification> for NotificationDelayKey {
@@ -35,7 +36,11 @@ impl From<&Notification> for NotificationDelayKey {
                     user.nickname().as_normalized_str().into(),
                 )
             }
-            Notification::Highlight { .. } => NotificationDelayKey::Highlight,
+            Notification::Highlight { channel, .. } => {
+                NotificationDelayKey::Highlight(
+                    channel.as_normalized_str().into(),
+                )
+            }
             Notification::FileTransferRequest { nick, .. } => {
                 NotificationDelayKey::FileTransferRequest(
                     nick.as_normalized_str().into(),
@@ -47,6 +52,11 @@ impl From<&Notification> for NotificationDelayKey {
             Notification::MonitoredOffline(..) => {
                 NotificationDelayKey::MonitoredOffline
             }
+            Notification::Channel { channel, .. } => {
+                NotificationDelayKey::Channel(
+                    channel.as_normalized_str().into(),
+                )
+            }
         }
     }
 }
@@ -59,7 +69,12 @@ pub struct Notifications {
 impl Notifications {
     pub fn new(config: &Config) -> Self {
         // Load sounds from different sources.
-        let sounds = config.notifications.load_sounds();
+        let sounds =
+            config.notifications.load_sounds(
+                config.highlights.matches.iter().filter_map(
+                    |highlight_match| highlight_match.sound.as_deref(),
+                ),
+            );
 
         Self {
             recent_notifications: HashMap::new(),
@@ -80,6 +95,7 @@ impl Notifications {
                     notification,
                     "Connected",
                     &server.to_string(),
+                    None,
                 );
             }
             Notification::Disconnected => {
@@ -88,6 +104,7 @@ impl Notifications {
                     notification,
                     "Disconnected",
                     &server.to_string(),
+                    None,
                 );
             }
             Notification::Reconnected => {
@@ -96,6 +113,7 @@ impl Notifications {
                     notification,
                     "Reconnected",
                     &server.to_string(),
+                    None,
                 );
             }
             Notification::MonitoredOnline(targets) => {
@@ -108,6 +126,7 @@ impl Notifications {
                         "Monitored users are online"
                     },
                     &join_targets(targets.iter().map(User::as_str).collect()),
+                    None,
                 );
             }
             Notification::MonitoredOffline(targets) => {
@@ -120,13 +139,20 @@ impl Notifications {
                         "Monitored users are offline"
                     },
                     &join_targets(targets.iter().map(Nick::as_str).collect()),
+                    None,
                 );
             }
-            Notification::FileTransferRequest { nick, filename } => {
-                if config
-                    .file_transfer_request
-                    .should_notify(vec![nick.to_string()])
-                {
+            Notification::FileTransferRequest {
+                nick,
+                casemapping,
+                filename,
+            } => {
+                if config.file_transfer_request.should_notify(
+                    &User::from(nick.clone()),
+                    None,
+                    server,
+                    *casemapping,
+                ) {
                     let (title, body) = if config
                         .file_transfer_request
                         .show_content
@@ -147,14 +173,21 @@ impl Notifications {
                         notification,
                         title,
                         body,
+                        None,
                     );
                 }
             }
-            Notification::DirectMessage { user, message } => {
-                if config
-                    .direct_message
-                    .should_notify(vec![user.nickname().to_string()])
-                {
+            Notification::DirectMessage {
+                user,
+                casemapping,
+                message,
+            } => {
+                if config.direct_message.should_notify(
+                    user,
+                    None,
+                    server,
+                    *casemapping,
+                ) {
                     let (title, body) = if config.direct_message.show_content {
                         (
                             &format!(
@@ -178,19 +211,26 @@ impl Notifications {
                         notification,
                         title,
                         body,
+                        None,
                     );
                 }
             }
             Notification::Highlight {
                 user,
                 channel,
+                casemapping,
                 message,
                 description,
+                sound,
             } => {
-                if config.highlight.should_notify(vec![
-                    channel.to_string(),
-                    user.nickname().to_string(),
-                ]) {
+                if config.highlight.should_notify(
+                    user,
+                    Some(channel),
+                    server,
+                    *casemapping,
+                ) {
+                    // Description is expected to be expanded by the calling
+                    // routine when show_content is true
                     if config.highlight.show_content {
                         self.execute(
                             &config.highlight,
@@ -200,6 +240,7 @@ impl Notifications {
                                 user.nickname()
                             ),
                             message,
+                            sound.as_deref(),
                         );
                     } else {
                         self.execute(
@@ -210,6 +251,47 @@ impl Notifications {
                                 user.nickname()
                             ),
                             &server.name,
+                            sound.as_deref(),
+                        );
+                    }
+                }
+            }
+            Notification::Channel {
+                user,
+                channel,
+                casemapping,
+                message,
+            } => {
+                if let Some(notification_config) =
+                    config.channels.get(channel.as_str())
+                    && notification_config.should_notify(
+                        user,
+                        None,
+                        server,
+                        *casemapping,
+                    )
+                {
+                    if notification_config.show_content {
+                        self.execute(
+                            notification_config,
+                            notification,
+                            &format!(
+                                "{} sent a message in {channel} on {server}",
+                                user.nickname()
+                            ),
+                            message,
+                            None,
+                        );
+                    } else {
+                        self.execute(
+                            notification_config,
+                            notification,
+                            &format!(
+                                "{} sent a message in {channel}",
+                                user.nickname()
+                            ),
+                            &server.name,
+                            None,
                         );
                     }
                 }
@@ -223,6 +305,7 @@ impl Notifications {
         notification: &Notification,
         title: &str,
         body: &str,
+        sound_name: Option<&str>,
     ) {
         let now = Utc::now();
         let delay_key = notification.into();
@@ -240,8 +323,9 @@ impl Notifications {
             toast::show(title, body);
         }
 
-        if let Some(sound_name) = &config.sound
-            && let Some(sound) = self.sounds.get(sound_name)
+        if let Some(sound) = sound_name
+            .or(config.sound.as_deref())
+            .and_then(|sound_name| self.sounds.get(sound_name))
         {
             audio::play(sound.clone());
         }
