@@ -2,13 +2,12 @@ use std::time::Duration;
 
 use data::config::{self, Config, sidebar};
 use data::dashboard::{BufferAction, BufferFocusedAction};
-use data::{Version, buffer, file_transfer, history, server};
+use data::{Version, buffer, file_transfer, history, isupport, server, target};
 use iced::widget::{
     Column, Row, Scrollable, Space, button, column, container, pane_grid, row,
     rule, scrollable, space, stack, text,
 };
 use iced::{Alignment, Length, Padding, Task, padding};
-use itertools::Either;
 use tokio::time;
 
 use super::{Focus, Panes, Server};
@@ -27,6 +26,7 @@ pub enum Message {
     Swap(window::Id, pane_grid::Pane),
     Detach(buffer::Upstream),
     Leave(buffer::Upstream),
+    CloseAllQueries(Server, Vec<target::Query>),
     ToggleInternalBuffer(buffer::Internal),
     ToggleCommandBar,
     ToggleThemeEditor,
@@ -38,6 +38,7 @@ pub enum Message {
     ReloadComplete,
     MarkAsRead(buffer::Upstream),
     MarkServerAsRead(Server),
+    QuitApplication,
 }
 
 #[derive(Debug, Clone)]
@@ -50,6 +51,7 @@ pub enum Event {
     Swap(window::Id, pane_grid::Pane),
     Detach(buffer::Upstream),
     Leave(buffer::Upstream),
+    CloseAllQueries(Server, Vec<target::Query>),
     ToggleInternalBuffer(buffer::Internal),
     ToggleCommandBar,
     ToggleThemeEditor,
@@ -59,6 +61,7 @@ pub enum Event {
     ConfigReloaded(Result<Config, config::Error>),
     MarkAsRead(buffer::Upstream),
     MarkServerAsRead(Server),
+    QuitApplication,
 }
 
 #[derive(Clone)]
@@ -90,6 +93,12 @@ impl Sidebar {
         message: Message,
     ) -> (Task<Message>, Option<Event>) {
         match message {
+            Message::CloseAllQueries(server, queries) => {
+                (Task::none(), Some(Event::CloseAllQueries(server, queries)))
+            }
+            Message::QuitApplication => {
+                (Task::none(), Some(Event::QuitApplication))
+            }
             Message::New(source) => (Task::none(), Some(Event::New(source))),
             Message::Popout(source) => {
                 (Task::none(), Some(Event::Popout(source)))
@@ -155,21 +164,32 @@ impl Sidebar {
 
     fn user_menu_button<'a>(
         &self,
-        keyboard: &'a data::config::Keyboard,
+        config: &'a Config,
         history: &'a history::Manager,
         file_transfers: &'a file_transfer::Manager,
         version: &'a Version,
         theme: &'a Theme,
     ) -> Element<'a, Message> {
-        let base = button(icon::menu()).padding(5).width(Length::Shrink);
+        let keyboard = &config.keyboard;
+        let base = button(icon::menu().size(theme::ICON_SIZE + 2.0))
+            .padding(5)
+            .width(Length::Shrink);
 
-        let menu = Menu::list(version.is_old());
+        let menu = Menu::list(
+            config.sidebar.user_menu.show_new_version_indicator
+                && version.is_old(),
+            config.file_transfer.enabled,
+        );
 
         let logs_has_unread = history.has_unread(&history::Kind::Logs);
 
-        // we show notification dot if theres a new version, or if theres transfers.
+        // Show notification dot if theres a new version, if there're transfers,
+        // or if the logs have unread messages.
         let show_notification_dot =
-            version.is_old() || !file_transfers.is_empty() || logs_has_unread;
+            (config.sidebar.user_menu.show_new_version_indicator
+                && version.is_old())
+                || (!file_transfers.is_empty() && config.file_transfer.enabled)
+                || logs_has_unread;
 
         if menu.is_empty() {
             base.into()
@@ -187,12 +207,11 @@ impl Sidebar {
                              keybind: Option<&data::shortcut::KeyBind>,
                              icon: Text<'a>,
                              message: Message| {
-                                button(
-                                    row![
-                                        icon.width(Length::Fixed(12.0)),
-                                        title,
-                                        keybind.map(|kb| {
-                                            text(format!("({kb})"))
+                                let keybind = keybind.and_then(|kb| match kb {
+                                    data::shortcut::KeyBind::Bind {
+                                        ..
+                                    } => Some(
+                                        text(format!("({kb})"))
                                             .shaping(text::Shaping::Advanced)
                                             .size(theme::TEXT_SIZE - 2.0)
                                             .style(theme::text::secondary)
@@ -201,19 +220,33 @@ impl Sidebar {
                                                     theme,
                                                 )
                                                 .map(font::get),
-                                            )
-                                        }),
+                                            ),
+                                    ),
+                                    data::shortcut::KeyBind::Unbind => None,
+                                });
+
+                                button(
+                                    row![
+                                        icon.width(Length::Fixed(12.0)),
+                                        title,
+                                        keybind
                                     ]
                                     .spacing(8)
                                     .align_y(iced::Alignment::Center),
                                 )
                                 .width(length)
-                                .padding(5)
+                                .padding(config.context_menu.padding.entry)
                                 .on_press(message)
                                 .into()
                             };
 
                         match menu {
+                            Menu::QuitApplication => context_button(
+                                text("Quit Halloy"),
+                                Some(&keyboard.quit_application),
+                                icon::quit(),
+                                Message::QuitApplication,
+                            ),
                             Menu::RefreshConfig => context_button(
                                 text("Reload config file"),
                                 Some(&keyboard.reload_configuration),
@@ -260,6 +293,14 @@ impl Sidebar {
                                     buffer::Internal::Highlights,
                                 ),
                             ),
+                            Menu::ChannelDiscovery => context_button(
+                                text("Channel Discovery"),
+                                None,
+                                icon::channel_discovery(),
+                                Message::ToggleInternalBuffer(
+                                    buffer::Internal::ChannelDiscovery(None),
+                                ),
+                            ),
                             Menu::Logs => context_button(
                                 text("Logs")
                                     .style(if logs_has_unread {
@@ -366,20 +407,27 @@ impl Sidebar {
         }
 
         let content = |width| {
-            let user_menu_button = config.sidebar.show_user_menu.then(|| {
-                self.user_menu_button(
-                    &config.keyboard,
-                    history,
-                    file_transfers,
-                    version,
-                    theme,
-                )
-            });
+            let user_menu_button =
+                config.sidebar.user_menu.enabled.then(|| {
+                    self.user_menu_button(
+                        config,
+                        history,
+                        file_transfers,
+                        version,
+                        theme,
+                    )
+                });
 
             let mut buffers = vec![];
             let mut client_enumeration = 0;
 
+            if config.sidebar.position.is_horizontal() {
+                buffers.push(space::horizontal().width(4).into());
+            }
+
             for server in servers.keys() {
+                let casemapping = clients.get_casemapping(server);
+
                 let button =
                     |buffer: buffer::Upstream,
                      connected: bool,
@@ -388,19 +436,17 @@ impl Sidebar {
                      has_unread: bool,
                      has_highlight: bool| {
                         upstream_buffer_button(
+                            config,
                             panes,
                             focus,
                             buffer,
                             connected,
-                            config.actions.sidebar.buffer,
-                            config.actions.sidebar.focused_buffer,
-                            config.sidebar.position,
-                            config.sidebar.server_icon_size,
-                            config.sidebar.unread_indicator,
                             server_has_unread,
                             supports_detach,
+                            casemapping,
                             has_unread,
                             has_highlight,
+                            history,
                             width,
                             theme,
                         )
@@ -497,17 +543,19 @@ impl Sidebar {
                             if config.sidebar.position.is_horizontal() {
                                 if client_enumeration < clients.len() {
                                     buffers.push(
-                                        container(rule::vertical(1))
-                                            .padding(padding::top(6))
-                                            .height(20)
-                                            .width(12)
-                                            .align_x(Alignment::Center)
+                                        space::horizontal()
+                                            .width(
+                                                config.sidebar.spacing.server,
+                                            )
                                             .into(),
                                     );
                                 }
                             } else {
-                                buffers
-                                    .push(space::vertical().height(12).into());
+                                buffers.push(
+                                    space::vertical()
+                                        .height(config.sidebar.spacing.server)
+                                        .into(),
+                                );
                             }
                         }
                     }
@@ -614,6 +662,7 @@ enum Menu {
     CommandBar,
     ThemeEditor,
     Highlights,
+    ChannelDiscovery,
     Logs,
     FileTransfers,
     Version,
@@ -621,10 +670,11 @@ enum Menu {
     HorizontalRule,
     Documentation,
     OpenConfigFile,
+    QuitApplication,
 }
 
 impl Menu {
-    fn list(has_new_version: bool) -> Vec<Self> {
+    fn list(has_new_version: bool, file_transfer_enabled: bool) -> Vec<Self> {
         let mut list = vec![Self::Version];
 
         if has_new_version {
@@ -635,29 +685,38 @@ impl Menu {
             Self::HorizontalRule,
             Self::CommandBar,
             Self::Documentation,
-            Self::FileTransfers,
+        ]);
+
+        if file_transfer_enabled {
+            list.push(Self::FileTransfers);
+        }
+
+        list.extend([
+            Self::ChannelDiscovery,
             Self::Highlights,
             Self::Logs,
             Self::OpenConfigFile,
             Self::RefreshConfig,
             Self::ThemeEditor,
+            Self::QuitApplication,
         ]);
 
         list
     }
 }
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
 enum Entry {
-    MarkServerAsRead,
+    Close(window::Id, pane_grid::Pane),
+    CloseAllQueries,
+    Detach,
+    Leave,
     MarkAsRead,
+    MarkServerAsRead,
     NewPane,
     Popout,
     Replace,
-    Close(window::Id, pane_grid::Pane),
     Swap(window::Id, pane_grid::Pane),
-    Leave,
-    Detach,
 }
 
 impl Entry {
@@ -668,60 +727,53 @@ impl Entry {
         focus: Focus,
         supports_detach: bool,
     ) -> Vec<Self> {
-        [
+        use Entry::*;
+        use itertools::Itertools;
+
+        itertools::chain!(
             match buffer {
-                buffer::Upstream::Server(_) => {
-                    vec![Entry::MarkServerAsRead]
-                }
+                buffer::Upstream::Server(_) =>
+                    vec![CloseAllQueries, MarkServerAsRead],
                 buffer::Upstream::Channel(_, _) => vec![],
                 buffer::Upstream::Query(_, _) => vec![],
             },
             match open {
-                None => Either::Left(
-                    vec![
-                        Entry::MarkAsRead,
-                        Entry::NewPane,
-                        Entry::Popout,
-                        Entry::Replace,
-                    ]
+                None => vec![MarkAsRead, NewPane, Popout, Replace]
                     .into_iter()
                     .chain(
                         (matches!(buffer, buffer::Upstream::Channel(_, _))
                             && supports_detach)
-                            .then_some(Entry::Detach),
-                    ),
-                ),
-                Some((window, pane)) => Either::Right(
-                    (num_panes > 1)
-                        .then_some(Entry::Close(window, pane))
-                        .into_iter()
-                        .chain(
-                            (Focus { window, pane } != focus)
-                                .then_some(Entry::Swap(window, pane)),
-                        ),
-                ),
-            }
-            .chain(Some(Entry::Leave))
-            .collect(),
-        ]
-        .concat()
+                            .then_some(Detach),
+                    )
+                    .collect_vec(),
+                Some((window, pane)) => (num_panes > 1)
+                    .then_some(Close(window, pane))
+                    .into_iter()
+                    .chain(
+                        (Focus { window, pane } != focus)
+                            .then_some(Swap(window, pane)),
+                    )
+                    .collect_vec(),
+            },
+            vec![Leave]
+        )
+        .sorted()
+        .collect_vec()
     }
 }
 
 fn upstream_buffer_button<'a>(
+    config: &'a Config,
     panes: &'a Panes,
     focus: Focus,
     buffer: buffer::Upstream,
     connected: bool,
-    buffer_action: BufferAction,
-    focused_buffer_action: Option<BufferFocusedAction>,
-    position: sidebar::Position,
-    server_icon_size: u32,
-    unread_indicator: sidebar::UnreadIndicator,
     server_has_unread: bool,
     supports_detach: bool,
+    casemapping: isupport::CaseMap,
     has_unread: bool,
     has_highlight: bool,
+    history: &'a history::Manager,
     width: Length,
     theme: &'a Theme,
 ) -> Element<'a, Message> {
@@ -737,23 +789,54 @@ fn upstream_buffer_button<'a>(
         .then_some((window_id, pane))
     });
 
-    let show_highlight_icon =
-        has_highlight && unread_indicator.has_unread_highlight_icon();
-    let show_unread_icon = has_unread && unread_indicator.has_unread_icon();
-    let show_unread_title = has_unread && unread_indicator.title;
-    let show_highlight_unread_title = has_highlight && unread_indicator.title;
+    let should_indicate_unread = buffer.channel().is_none_or(|channel| {
+        config.sidebar.unread_indicator.should_indicate_unread(
+            channel,
+            buffer.server(),
+            casemapping,
+        )
+    });
+    let is_unread_query =
+        matches!(buffer, buffer::Upstream::Query(_, _)) && has_unread;
+    let has_highlight = has_highlight
+        || (is_unread_query
+            && config.sidebar.unread_indicator.query_as_highlight);
+
+    let show_highlight_icon = has_highlight
+        && config.sidebar.unread_indicator.has_unread_highlight_icon()
+        && should_indicate_unread;
+    let show_unread_icon = has_unread
+        && config.sidebar.unread_indicator.has_unread_icon()
+        && should_indicate_unread;
+    let show_unread_title = has_unread
+        && config.sidebar.unread_indicator.title
+        && should_indicate_unread;
+    let show_highlight_unread_title = has_highlight
+        && config.sidebar.unread_indicator.title
+        && should_indicate_unread;
 
     let buffer_title_style = if show_highlight_unread_title {
         theme::text::highlight_indicator
     } else if show_unread_title {
         theme::text::unread_indicator
+    } else if !connected {
+        if matches!(&buffer, buffer::Upstream::Server(_)) {
+            theme::text::error
+        } else {
+            theme::text::secondary
+        }
     } else {
         theme::text::primary
     };
 
     let buffer_title_font = theme::font_style::primary(theme).map(font::get);
 
-    let icon_tuple = if let buffer::Upstream::Server(server) = &buffer {
+    // check for server icon first (only for server buffers with icon size configured)
+    let icon_tuple = if let (
+        buffer::Upstream::Server(server),
+        data::config::sidebar::ServerIcon::Size(size),
+    ) = (&buffer, &config.sidebar.server_icon)
+    {
         Some((
             if server.is_bouncer_network() {
                 icon::link()
@@ -771,45 +854,58 @@ fn upstream_buffer_button<'a>(
             } else {
                 theme::text::error
             })
-            .size(server_icon_size),
-            server_icon_size,
+            .size(*size),
+            *size,
         ))
-    } else if show_highlight_icon
+    }
+    // fall through to unread/highlight icons for all buffers (including server)
+    else if show_highlight_icon
         && let Some(highlight_icon) =
-            icon::from_icon(unread_indicator.highlight_icon)
+            icon::from_icon(config.sidebar.unread_indicator.highlight_icon)
     {
         Some((
             highlight_icon
                 .style(theme::text::highlight_indicator)
-                .size(unread_indicator.highlight_icon_size),
-            unread_indicator.highlight_icon_size,
+                .size(config.sidebar.unread_indicator.highlight_icon_size),
+            config.sidebar.unread_indicator.highlight_icon_size,
         ))
     } else if show_unread_icon
-        && let Some(unread_icon) = icon::from_icon(unread_indicator.icon)
+        && let Some(unread_icon) =
+            icon::from_icon(config.sidebar.unread_indicator.icon)
     {
         Some((
             unread_icon
                 .style(theme::text::unread_indicator)
-                .size(unread_indicator.icon_size),
-            unread_indicator.icon_size,
+                .size(config.sidebar.unread_indicator.icon_size),
+            config.sidebar.unread_indicator.icon_size,
         ))
     } else {
         None
     };
 
-    let (left_padding, icon) = if position.is_horizontal() {
+    let (left_padding, icon) = if config.sidebar.position.is_horizontal() {
         icon_tuple.map_or((0, None), |(icon, icon_size)| {
             (icon_size + 8, Some(container(icon).center_y(Length::Fill)))
         })
     } else {
+        let server_icon_size = match config.sidebar.server_icon {
+            data::config::sidebar::ServerIcon::Size(size) => size,
+            data::config::sidebar::ServerIcon::Hidden => 0,
+        };
         let max_icon_size = server_icon_size.max(
-            unread_indicator
+            config
+                .sidebar
+                .unread_indicator
                 .has_unread_icon()
-                .then_some(unread_indicator.icon_size)
+                .then_some(config.sidebar.unread_indicator.icon_size)
                 .max(
-                    unread_indicator
+                    config
+                        .sidebar
+                        .unread_indicator
                         .has_unread_highlight_icon()
-                        .then_some(unread_indicator.highlight_icon_size),
+                        .then_some(
+                            config.sidebar.unread_indicator.highlight_icon_size,
+                        ),
                 )
                 .unwrap_or(0),
         );
@@ -872,10 +968,13 @@ fn upstream_buffer_button<'a>(
                     open.is_some(),
                 )
             })
+            .padding(config.sidebar.padding.buffer)
             .on_press({
                 match is_focused {
                     Some((window, pane)) => {
-                        if let Some(focus_action) = focused_buffer_action {
+                        if let Some(focus_action) =
+                            config.actions.sidebar.focused_buffer
+                        {
                             match focus_action {
                                 BufferFocusedAction::ClosePane => {
                                     Message::Close(window, pane)
@@ -891,7 +990,7 @@ fn upstream_buffer_button<'a>(
                         if let Some((window, pane)) = open {
                             Message::Focus(window, pane)
                         } else {
-                            match buffer_action {
+                            match config.actions.sidebar.buffer {
                                 BufferAction::NewPane => {
                                     Message::New(buffer.clone())
                                 }
@@ -921,6 +1020,25 @@ fn upstream_buffer_button<'a>(
             entries,
             move |entry, length| {
                 let (content, message) = match entry {
+                    Entry::CloseAllQueries => {
+                        let queries = history
+                            .get_unique_queries(buffer.server())
+                            .into_iter()
+                            .cloned()
+                            .collect::<Vec<_>>();
+
+                        (
+                            "Close all queries",
+                            if queries.is_empty() {
+                                None
+                            } else {
+                                Some(Message::CloseAllQueries(
+                                    buffer.server().clone(),
+                                    queries,
+                                ))
+                            },
+                        )
+                    }
                     Entry::MarkServerAsRead => (
                         "Mark entire server as read",
                         if server_has_unread {
@@ -977,7 +1095,7 @@ fn upstream_buffer_button<'a>(
 
                 button(text(content))
                     .width(length)
-                    .padding(5)
+                    .padding(config.context_menu.padding.entry)
                     .style(|theme, status| {
                         theme::button::primary(theme, status, false)
                     })
