@@ -342,9 +342,10 @@ fn picker<'a, Message: 'a>(
                     picker.handle_from_color(color, bounds, handle_radius);
 
                 match event {
-                    iced::Event::Mouse(mouse::Event::ButtonPressed(
-                        mouse::Button::Left,
-                    ))
+                    iced::Event::Mouse(mouse::Event::ButtonPressed {
+                        button: mouse::Button::Left,
+                        ..
+                    })
                     | iced::Event::Touch(touch::Event::FingerPressed {
                         ..
                     }) if state.is_none() => {
