@@ -52,6 +52,7 @@ pub fn context_menu<'a, T, Message, Theme, Renderer>(
         anchor,
         toggle_behavior,
         menu: None,
+        mouse_interaction_on_hover: mouse::Interaction::Pointer,
     }
 }
 
@@ -64,6 +65,7 @@ pub struct ContextMenu<'a, T, Message, Theme, Renderer> {
     toggle_behavior: ToggleBehavior,
     // Cached, recreated during overlay if menu is open
     menu: Option<Element<'a, Message, Theme, Renderer>>,
+    mouse_interaction_on_hover: mouse::Interaction,
 }
 
 #[derive(Debug)]
@@ -87,7 +89,7 @@ pub enum Status {
     Open {
         position: Point,
         // Keep context menu open if button press is inside specified bounds
-        keep_open_bounds: Option<Rectangle>,
+        keep_open_bounds: Option<(Vector, Size)>,
     },
 }
 
@@ -99,7 +101,7 @@ impl Status {
         }
     }
 
-    pub fn keep_open_bounds(&self) -> Option<&Rectangle> {
+    pub fn keep_open_bounds(&self) -> Option<&(Vector, Size)> {
         match self {
             Status::Closed => None,
             Status::Open {
@@ -109,6 +111,18 @@ impl Status {
     }
 }
 
+impl<'a, T, Message, Theme, Renderer>
+    ContextMenu<'a, T, Message, Theme, Renderer>
+{
+    pub fn mouse_interaction_on_hover(
+        mut self,
+        interaction: mouse::Interaction,
+    ) -> Self {
+        self.mouse_interaction_on_hover = interaction;
+        self
+    }
+}
+
 impl<'a, T, Message, Theme, Renderer> Widget<Message, Theme, Renderer>
     for ContextMenu<'a, T, Message, Theme, Renderer>
 where
@@ -212,22 +226,28 @@ where
     ) {
         // is this a mouse event we are waiting for?
         let is_mouse_event =
-            matches!(event, Event::Mouse(mouse::Event::ButtonPressed(_)));
+            matches!(event, Event::Mouse(mouse::Event::ButtonPressed { .. }));
 
         if is_mouse_event {
             let state = tree.state.downcast_mut::<State>();
             let prev_status = state.status;
 
             // is this a mouse event for that we should do something?
-            let is_activation_mouse_event = *event
-                == Event::Mouse(mouse::Event::ButtonPressed(
-                    self.activation_button,
-                ));
+            let is_activation_mouse_event =
+                if let Event::Mouse(mouse::Event::ButtonPressed {
+                    button, ..
+                }) = event
+                    && *button == self.activation_button
+                {
+                    true
+                } else {
+                    false
+                };
 
             let position = if is_activation_mouse_event {
                 match self.anchor {
                     Anchor::Widget => {
-                        cursor.position_over(layout.bounds()).map(|_| {
+                        cursor.is_over(layout.bounds()).then_some({
                             let widget = layout.bounds();
                             Point::new(
                                 widget.x + widget.width,
@@ -263,10 +283,19 @@ where
                     Status::Closed,
                     ToggleBehavior::Close,
                     Some(position),
-                ) => Status::Open {
-                    position,
-                    keep_open_bounds: Some(layout.bounds()),
-                },
+                ) => {
+                    let layout_bounds = layout.bounds();
+
+                    // Position may be relative to containing scrollable, so
+                    // store bounds as offset vector and size
+                    Status::Open {
+                        position,
+                        keep_open_bounds: Some((
+                            layout_bounds.position() - position,
+                            layout_bounds.size(),
+                        )),
+                    }
+                }
                 (_, Status::Open { .. }, _, None)
                 | (true, Status::Open { .. }, ToggleBehavior::Close, Some(_)) => {
                     Status::Closed
@@ -306,7 +335,7 @@ where
         _renderer: &Renderer,
     ) -> mouse::Interaction {
         if cursor.is_over(layout.bounds()) {
-            mouse::Interaction::Pointer
+            self.mouse_interaction_on_hover
         } else {
             mouse::Interaction::default()
         }
@@ -577,11 +606,16 @@ where
         clipboard: &mut dyn Clipboard,
         shell: &mut Shell<'_, Message>,
     ) {
-        if let Event::Mouse(mouse::Event::ButtonPressed(_)) = &event
+        if let Event::Mouse(mouse::Event::ButtonPressed { .. }) = &event
             && cursor.position_over(layout.bounds()).is_none()
             && self.state.status.keep_open_bounds().is_none_or(
-                |keep_open_bounds| {
-                    cursor.position_over(*keep_open_bounds).is_none()
+                |(keep_open_vector, keep_open_size)| {
+                    let keep_open_bounds = Rectangle::new(
+                        self.position + *keep_open_vector,
+                        *keep_open_size,
+                    );
+
+                    cursor.position_over(keep_open_bounds).is_none()
                 },
             )
         {
