@@ -1,18 +1,21 @@
 use data::appearance::theme::{FontStyle, randomize_color};
-use data::{Config, isupport, message, target};
+use data::{Config, Server, isupport, message, target};
 use iced::widget::span;
 use iced::widget::text::Span;
 use iced::{Color, Length, border};
+use unicode_segmentation::UnicodeSegmentation;
 
 use super::{Element, Renderer, selectable_rich_text, selectable_text};
 use crate::{Theme, font, theme};
 
 pub fn message_content<'a, M: 'a>(
     content: &'a message::Content,
+    server: &'a Server,
     chantypes: &[char],
     casemapping: isupport::CaseMap,
     theme: &'a Theme,
     on_link: impl Fn(message::Link) -> M + 'a,
+    default_link: Option<message::Link>,
     style: impl Fn(&Theme) -> selectable_text::Style + 'a,
     font_style: impl Fn(&Theme) -> Option<FontStyle>,
     color_transformation: Option<impl Fn(Color) -> Color>,
@@ -20,10 +23,12 @@ pub fn message_content<'a, M: 'a>(
 ) -> Element<'a, M> {
     message_content_impl::<(), M>(
         content,
+        server,
         chantypes,
         casemapping,
         theme,
         on_link,
+        default_link,
         style,
         font_style,
         color_transformation,
@@ -34,10 +39,12 @@ pub fn message_content<'a, M: 'a>(
 
 pub fn with_context<'a, T: Copy + 'a, M: 'a>(
     content: &'a message::Content,
+    server: &'a Server,
     chantypes: &[char],
     casemapping: isupport::CaseMap,
     theme: &'a Theme,
     on_link: impl Fn(message::Link) -> M + 'a,
+    default_link: Option<message::Link>,
     style: impl Fn(&Theme) -> selectable_text::Style + 'a,
     font_style: impl Fn(&Theme) -> Option<FontStyle>,
     color_transformation: Option<impl Fn(Color) -> Color>,
@@ -47,10 +54,12 @@ pub fn with_context<'a, T: Copy + 'a, M: 'a>(
 ) -> Element<'a, M> {
     message_content_impl(
         content,
+        server,
         chantypes,
         casemapping,
         theme,
         on_link,
+        default_link,
         style,
         font_style,
         color_transformation,
@@ -62,10 +71,12 @@ pub fn with_context<'a, T: Copy + 'a, M: 'a>(
 #[allow(clippy::type_complexity)]
 fn message_content_impl<'a, T: Copy + 'a, M: 'a>(
     content: &'a message::Content,
+    server: &'a Server,
     chantypes: &[char],
     casemapping: isupport::CaseMap,
     theme: &'a Theme,
     on_link: impl Fn(message::Link) -> M + 'a,
+    default_link: Option<message::Link>,
     style: impl Fn(&Theme) -> selectable_text::Style + 'a,
     font_style: impl Fn(&Theme) -> Option<FontStyle>,
     color_transformation: Option<impl Fn(Color) -> Color>,
@@ -76,10 +87,23 @@ fn message_content_impl<'a, T: Copy + 'a, M: 'a>(
     config: &Config,
 ) -> Element<'a, M> {
     match content {
-        data::message::Content::Plain(text) => selectable_text(text)
-            .font_maybe(font_style(theme).map(font::get))
-            .style(style)
-            .into(),
+        data::message::Content::Plain(text) => {
+            if let Some(only_emojis_size) = config.font.only_emojis_size
+                && UnicodeSegmentation::graphemes(text.as_str(), true)
+                    .all(|grapheme| emojis::get(grapheme).is_some())
+            {
+                selectable_text(text)
+                    .font_maybe(font_style(theme).map(font::get))
+                    .size(f32::from(only_emojis_size))
+                    .style(style)
+                    .into()
+            } else {
+                selectable_text(text)
+                    .font_maybe(font_style(theme).map(font::get))
+                    .style(style)
+                    .into()
+            }
+        }
         data::message::Content::Fragments(fragments) => {
             let mut text = selectable_rich_text::<
                 M,
@@ -101,7 +125,7 @@ fn message_content_impl<'a, T: Copy + 'a, M: 'a>(
                             }
                         };
 
-                        match fragment {
+                        let span = match fragment {
                             data::message::Fragment::Text(s) => span(s),
                             data::message::Fragment::Channel(s) => {
                                 span(s.as_str())
@@ -117,6 +141,7 @@ fn message_content_impl<'a, T: Copy + 'a, M: 'a>(
                                         theme.styles().buffer.url.color,
                                     ))
                                     .link(message::Link::Channel(
+                                        server.clone(),
                                         target::Channel::from_str(
                                             s.as_str(),
                                             chantypes,
@@ -154,7 +179,10 @@ fn message_content_impl<'a, T: Copy + 'a, M: 'a>(
                                             .map(font::get),
                                     )
                                     .color(transform_color(color))
-                                    .link(message::Link::User(user.clone()))
+                                    .link(message::Link::User(
+                                        server.clone(),
+                                        user.clone(),
+                                    ))
                             }
                             data::message::Fragment::HighlightNick(
                                 user,
@@ -190,7 +218,10 @@ fn message_content_impl<'a, T: Copy + 'a, M: 'a>(
                                     )
                                     .color(transform_color(color))
                                     .background(theme.styles().buffer.highlight)
-                                    .link(message::Link::User(user.clone()))
+                                    .link(message::Link::User(
+                                        server.clone(),
+                                        user.clone(),
+                                    ))
                             }
                             data::message::Fragment::HighlightMatch(text) => {
                                 span(text.as_str())
@@ -270,6 +301,7 @@ fn message_content_impl<'a, T: Copy + 'a, M: 'a>(
                             data::message::Fragment::Condensed {
                                 text,
                                 source,
+                                ..
                             } => span(text.as_str())
                                 .font_maybe(
                                     theme::font_style::server(
@@ -286,6 +318,14 @@ fn message_content_impl<'a, T: Copy + 'a, M: 'a>(
                                     .color
                                     .map(transform_color),
                                 ),
+                        };
+
+                        if span.link.is_none()
+                            && let Some(default_link) = &default_link
+                        {
+                            span.link(default_link.clone())
+                        } else {
+                            span
                         }
                     })
                     .collect::<Vec<_>>(),
