@@ -385,9 +385,10 @@ where
         }
 
         match event {
-            iced::Event::Mouse(mouse::Event::ButtonPressed(
-                mouse::Button::Left,
-            ))
+            iced::Event::Mouse(mouse::Event::ButtonPressed {
+                button: mouse::Button::Left,
+                ..
+            })
             | iced::Event::Touch(touch::Event::FingerPressed { .. }) => {
                 if let Some(position) = cursor.position_in(bounds)
                     && let Some(span) = state.paragraph.hit_span(position)
@@ -459,6 +460,7 @@ where
                     align_y: self.align_y,
                     shaping: Shaping::Advanced,
                     wrapping: text::Wrapping::WordOrGlyph,
+                    hint_factor: renderer.scale_factor(),
                 };
 
                 // Check spoiler
@@ -511,9 +513,10 @@ where
                     );
                 }
             }
-            iced::Event::Mouse(mouse::Event::ButtonPressed(
-                mouse::Button::Right,
-            )) => {
+            iced::Event::Mouse(mouse::Event::ButtonPressed {
+                button: mouse::Button::Right,
+                ..
+            }) => {
                 if let Some(position) = cursor.position_in(bounds)
                     && let Some((link_entries, _)) = &self.context_menu
                     && let Some((link, entries)) =
@@ -601,8 +604,8 @@ where
                                 ),
                             bounds.size()
                                 + Size::new(
-                                    span.padding.horizontal(),
-                                    span.padding.vertical(),
+                                    span.padding.left + span.padding.right,
+                                    span.padding.top + span.padding.bottom,
                                 ),
                         );
 
@@ -867,6 +870,7 @@ where
             align_y,
             shaping: Shaping::Advanced,
             wrapping: text::Wrapping::WordOrGlyph,
+            hint_factor: renderer.scale_factor(),
         };
 
         if state.spans != spans {
@@ -894,6 +898,7 @@ where
                 align_y,
                 shaping: Shaping::Advanced,
                 wrapping: text::Wrapping::WordOrGlyph,
+                hint_factor: renderer.scale_factor(),
             }) {
                 text::Difference::None => {}
                 text::Difference::Bounds => {
