@@ -163,6 +163,7 @@ where
                 align_y: self.format.align_y,
                 shaping: self.format.shaping,
                 wrapping: self.format.wrapping,
+                hint_factor: renderer.scale_factor(),
             });
 
             state.paragraph.min_bounds()
@@ -189,9 +190,10 @@ where
         state.hovered = cursor.is_over(bounds);
 
         match event {
-            iced::Event::Mouse(mouse::Event::ButtonPressed(
-                mouse::Button::Left,
-            ))
+            iced::Event::Mouse(mouse::Event::ButtonPressed {
+                button: mouse::Button::Left,
+                ..
+            })
             | iced::Event::Touch(touch::Event::FingerPressed { .. }) => {
                 if let Some(cursor) = cursor.position() {
                     state.interaction =
