@@ -12,6 +12,7 @@ pub struct Window {
     pub position: Option<Point>,
     pub size: Size,
     pub focused: bool,
+    pub fullscreen: Option<Size>,
 }
 
 impl Window {
@@ -21,6 +22,7 @@ impl Window {
             position: None,
             size: Size::default(),
             focused: false,
+            fullscreen: None,
         }
     }
 
@@ -29,6 +31,18 @@ impl Window {
         self.size = size;
         self.focused = true;
     }
+
+    /// Toggle fullscreen mode. When entering fullscreen, stores the current
+    /// windowed size. When exiting, restores the previous windowed size.
+    pub fn toggle_fullscreen(&mut self) {
+        self.fullscreen = match self.fullscreen {
+            Some(size) => {
+                self.size = size;
+                None
+            }
+            None => Some(self.size),
+        };
+    }
 }
 
 impl From<Window> for data::Window {
@@ -36,6 +50,7 @@ impl From<Window> for data::Window {
         data::Window {
             position: window.position,
             size: window.size,
+            fullscreen: window.fullscreen,
         }
     }
 }
@@ -90,6 +105,7 @@ pub fn settings(config: &Config) -> Settings {
             override_redirect: false,
         },
         decorations: config.platform_specific.linux.decorations,
+        transparent: true,
         ..Default::default()
     }
 }
@@ -105,6 +121,7 @@ pub fn settings(config: &Config) -> Settings {
             fullsize_content_view: true,
         },
         decorations: config.platform_specific.macos.decorations,
+        transparent: true,
         ..Default::default()
     }
 }
@@ -128,6 +145,7 @@ pub fn settings(config: &Config) -> Settings {
                 )
                 .ok(),
                 decorations: config.platform_specific.windows.decorations,
+                transparent: true,
                 ..Default::default()
             },
             None => Settings::default(),
